/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name : stdint.h
* $Rev: 823 $
* $Date:: 2014-04-21 16:45:10 +0900#$
* Description : This header file is a subset of stdint.h in c99
*             : We released this header file as is.
******************************************************************************/
#ifndef     STDINT_H
#define     STDINT_H

typedef signed char         int8_t;
typedef unsigned char       uint8_t;
typedef signed char         int_least8_t;
typedef unsigned char       uint_least8_t;
typedef signed int          int_fast8_t;
typedef unsigned int        uint_fast8_t;

typedef signed short        int16_t;
typedef unsigned short      uint16_t;
typedef signed short        int_least16_t;
typedef unsigned short      uint_least16_t;
typedef signed int          int_fast16_t;
typedef unsigned int        uint_fast16_t;

typedef signed int          int32_t;
typedef unsigned int        uint32_t;
typedef signed int          int_least32_t;
typedef unsigned int        uint_least32_t;
typedef signed int          int_fast32_t;
typedef unsigned int        uint_fast32_t;

typedef signed long long    int64_t;
typedef unsigned long long  uint64_t;
typedef signed long long    int_least64_t;
typedef unsigned long long  uint_least64_t;
typedef signed long long    int_fast64_t;
typedef unsigned long long  uint_fast64_t;

typedef signed int          intptr_t;
typedef unsigned int        uintptr_t;

/* typedef signed int          ptrdiff_t;  */ /* stddef.h */

typedef signed long long    intmax_t;
typedef unsigned long long  uintmax_t;



#define INT8_MIN         -128
#define INT8_MAX          127
#define UINT8_MAX         255
#define INT_LEAST8_MIN   INT8_MIN
#define INT_LEAST8_MAX   INT8_MAX
#define UINT_LEAST8_MAX  UINT8_MAX
#define INT_FAST8_MIN    INT32_MIN
#define INT_FAST8_MAX    INT32_MAX
#define UINT_FAST8_MAX   UINT32_MAX

#define INT16_MIN         -32768
#define INT16_MAX          32767
#define UINT16_MAX         65535
#define INT_LEAST16_MIN   INT16_MIN
#define INT_LEAST16_MAX   INT16_MAX
#define UINT_LEAST16_MAX  UINT16_MAX
#define INT_FAST16_MIN    INT32_MIN
#define INT_FAST16_MAX    INT32_MAX
#define UINT_FAST16_MAX   UINT32_MAX

#define INT32_MIN         (~0x7FFFFFFF)
#define INT32_MAX         0x7FFFFFFF
#define UINT32_MAX        0xFFFFFFFFu
#define INT_LEAST32_MIN   INT32_MIN
#define INT_LEAST32_MAX   INT32_MAX
#define UINT_LEAST32_MAX  UINT32_MAX
#define INT_FAST32_MIN    INT32_MIN
#define INT_FAST32_MAX    INT32_MAX
#define UINT_FAST32_MAX   UINT32_MAX

#define INT64_MIN         (~0x7FFFFFFFFFFFFFFFLL)
#define INT64_MAX         0x7FFFFFFFFFFFFFFFLL
#define UINT64_MAX        0xFFFFFFFFFFFFFFFFuLL
#define INT_LEAST64_MIN   INT64_MIN
#define INT_LEAST64_MAX   INT64_MAX
#define UINT_LEAST64_MAX  UINT64_MAX
#define INT_FAST64_MIN    INT64_MIN
#define INT_FAST64_MAX    INT64_MAX
#define UINT_FAST64_MAX   UINT64_MAX

#define INTPTR_MIN   INT32_MIN
#define INTPTR_MAX   INT32_MAX
#define UINTPTR_MAX  UINT32_MAX

#define PTRDIFF_MIN  INT32_MIN
#define PTRDIFF_MAX  INT32_MAX

#define SIG_ATOMIC_MIN  INT32_MIN
#define SIG_ATOMIC_MAX  INT32_MAX

#define INTMAX_MIN   INT64_MIN
#define INTMAX_MAX   INT64_MAX
#define UINTMAX_MAX  UINT64_MAX

#define SIZE_MAX  UINT32_MAX

#define WINT_MIN  INT32_MIN
#define WINT_MAX  INT32_MAX



#define INT8_C(x)   (x)
#define INT16_C(x)  (x)
#define INT32_C(x)  (x)
#define INT64_C(x)  (x ## LL)

#define UINT8_C(x)  (x ## u)
#define UINT16_C(x) (x ## u)
#define UINT32_C(x) (x ## u)
#define UINT64_C(x) (x ## uLL)

#define INTMAX_C    INT64_C
#define UINTMAX_C   UINT64_C

#endif  /* STDINT_H */

/* End of File */
