/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : loader_param_nor_boot.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : Loader parameter for 16-bit/32-bit bus boot mode
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : Loader parameter for 16bit or 32bit bus boot mode 
* Limitation   : This source file assumes that RZ/T1 boot mode is
*                16-bit bus boot mode (NOR flash). 
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include <stdint.h>
#include "r_system.h"
#if defined (__CC_ARM)
#include "boot_common.h"
#include "nor_boot.h"
#endif

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
#if defined (__ICCARM__)

/* This program is allocated to section "ldr_param" */ 
#pragma location="ldr_param"

    __root const uint32_t loader_param_BSC[19] =
    {   
        0x00000000,  // CACHE_FLG     @4000_0000H
        0x10000C00,  // CS0BCR_V      @4000_0004H
        0x00001340,  // CS0WCR_V      @4000_0008H
        0x00000000,  // DUMMY0        @4000_000CH
        0x00000000,  // BSC_FLG       @4000_0010H
        0x4000004C,  // LDR_ADDR_NML  @4000_0014H
        0x00006000,  // LDR_SIZE_NML  @4000_0018H
        0x00802000,  // DEST_ADDR_NML @4000_001CH
        0x00000000,  // DUMMY1        @4000_0020H
        0x00000000,  // DUMMY2        @4000_0024H
        0x00000000,  // DUMMY3        @4000_0028H
        0x00000000,  // DUMMY4        @4000_002CH
        0x00000000,  // DUMMY5        @4000_0030H
        0x00000000,  // DUMMY6        @4000_0034H
        0x00000000,  // DUMMY7        @4000_0038H
        0x00000000,  // DUMMY8        @4000_003CH
        0x00000000,  // DUMMY9        @4000_0040H
        0x00000000,  // DUMMY10       @4000_0044H
        0x0000F00C   // CHECK_SUM     @4000_0048H
    };

#endif  // __ICCARM__

#if defined (__GNUC__)
/*******************************************************************************
Parameter Information for the Loader in NOR Boot Mode
*******************************************************************************/
const unsigned int Loader_Param[]  __attribute__ ((section (".loader_param"))) = {
        0x00000000,  // CACHE_FLG     @4000_0000H
        0x10000C00,  // CS0BCR_V      @4000_0004H
        0x00001340,  // CS0WCR_V      @4000_0008H
        0x00000000,  // DUMMY0        @4000_000CH
        0x00000000,  // BSC_FLG       @4000_0010H
        0x4000004C,  // LDR_ADDR_NML  @4000_0014H
        0x00006000,  // LDR_SIZE_NML  @4000_0018H
        0x00802000,  // DEST_ADDR_NML @4000_001CH
        0x00000000,  // DUMMY1        @4000_0020H
        0x00000000,  // DUMMY2        @4000_0024H
        0x00000000,  // DUMMY3        @4000_0028H
        0x00000000,  // DUMMY4        @4000_002CH
        0x00000000,  // DUMMY5        @4000_0030H
        0x00000000,  // DUMMY6        @4000_0034H
        0x00000000,  // DUMMY7        @4000_0038H
        0x00000000,  // DUMMY8        @4000_003CH
        0x00000000,  // DUMMY9        @4000_0040H
        0x00000000,  // DUMMY10       @4000_0044H
        0x0000F00C   // CHECK_SUM     @4000_0048H
};
#endif  // __GNUC__

#if defined (__CC_ARM)

#pragma arm section rodata = "CONST_LOADER_TABLE"

const stBOOT_NORBSC    nor_loader_table =
{
    CACHE_DISABLE,                                   /* cache_flag              */

    0x10000C00,                                      /* cs0bcr                  */
    0x00001340,                                      /* cs0wcr                  */
    0x00000000,                                      /* dummy                   */
    NOR_FINALIZE_DISABLE,                            /* finalize_flag           */

    (uint32_t)&Load$$LR$$LOAD_MODULE2$$Base,         /* prog_start_addr_nml     */
    (uint32_t)&Load$$LR$$LOAD_MODULE2$$Length,       /* prog_area_size_nml      */
    (uint32_t)&Image$$LOADER_RESET_HANDLER$$Base,    /* prog_start_ram_addr_nml */

    0xffffffff,                                      /* duumy1                  */
    0xffffffff,                                      /* duumy2                  */
    0xffffffff,                                      /* duumy3                  */
    0xffffffff,                                      /* duumy4                  */
    0xffffffff,                                      /* duumy5                  */
    0xffffffff,                                      /* duumy6                  */
    0xffffffff,                                      /* duumy7                  */
    0xffffffff,                                      /* duumy8                  */
    0xffffffff,                                      /* duumy9                  */
    0xffffffff,                                      /* duumy10                 */

    0x17320508                                       /* check_sum(DUMMY)        */
};
#pragma arm section rodata

#endif  // __CC_ARM

/* End of File */


