/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : loader_param_serial_boot.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : Loader parameter for SPI boot mode
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : Loader parameter for SPI boot mode 
* Limitation   : This source file assumes that RZ/T1 boot mode is
*                SPI boot mode (Serial flash). 
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include <stdint.h>
#include "r_system.h"
#if defined (__CC_ARM)
#include "boot_common.h"
#include "sflash_boot.h"
#endif

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private variables and functions
*******************************************************************************/
#ifdef __ICCARM__

/* This program is allocated to section "ldr_param" */ 
#pragma location="ldr_param"

    __root const uint32_t loader_param_SPI[19] =
    {   
        0x00000000,  // CACHE_FLG     @3000_0000H
        0x00000000,  // SSLDR_V       @3000_0004H
        0x00000200,  // SPBCR_V       @3000_0008H
        0x00010101,  // DRCR_V        @3000_000CH
        0x00000000,  // SPIBSC_FLG    @3000_0010H
        0x3000004C,  // LDR_ADDR_NML  @3000_0014H
        0x00006000,  // LDR_SIZE_NML  @3000_0018H
        0x00802000,  // DEST_ADDR_NML @3000_001CH
        0x00000000,  // DUMMY1        @3000_0020H
        0x00000000,  // DUMMY2        @3000_0024H
        0x00000000,  // DUMMY3        @3000_0028H
        0x00000000,  // DUMMY4        @3000_002CH
        0x00000000,  // DUMMY5        @3000_0030H
        0x00000000,  // DUMMY6        @3000_0034H
        0x00000000,  // DUMMY7        @3000_0038H
        0x00000000,  // DUMMY8        @3000_003CH
        0x00000000,  // DUMMY9        @3000_0040H
        0x00000000,  // DUMMY10       @3000_0044H
        0x0000B3CE   // CHECK_SUM     @3000_0048H
    };

#endif  // __ICCARM__

#if defined (__GNUC__)
/*******************************************************************************
Parameter Information for the Loader in NOR Boot Mode
*******************************************************************************/
const unsigned int Loader_Param[]  __attribute__ ((section (".loader_param"))) = {
        0x00000000,  // CACHE_FLG     @3000_0000H
        0x00000000,  // SSLDR_V       @3000_0004H
        0x00000200,  // SPBCR_V       @3000_0008H
        0x00010101,  // DRCR_V        @3000_000CH
        0x00000000,  // SPIBSC_FLG    @3000_0010H
        0x3000004C,  // LDR_ADDR_NML  @3000_0014H
        0x00006000,  // LDR_SIZE_NML  @3000_0018H
        0x00802000,  // DEST_ADDR_NML @3000_001CH
        0x00000000,  // DUMMY1        @3000_0020H
        0x00000000,  // DUMMY2        @3000_0024H
        0x00000000,  // DUMMY3        @3000_0028H
        0x00000000,  // DUMMY4        @3000_002CH
        0x00000000,  // DUMMY5        @3000_0030H
        0x00000000,  // DUMMY6        @3000_0034H
        0x00000000,  // DUMMY7        @3000_0038H
        0x00000000,  // DUMMY8        @3000_003CH
        0x00000000,  // DUMMY9        @3000_0040H
        0x00000000,  // DUMMY10       @3000_0044H
        0x0000B3CE   // CHECK_SUM     @3000_0048H
};
#endif  // __GNUC__

#if defined (__CC_ARM)

#pragma arm section rodata = "CONST_LOADER_TABLE"

const stBOOT_SPIBSC    spibsc_loader_table =
{
    CACHE_DISABLE,                                   /* cache_flag              */

    0x00000000,                                      /* ssldr                   */
    0x00000200,                                      /* spbcr(37.5Mbps)         */
                                                     /*  SPBR=2,BRDV=0(@150MHz) */
    0x00010101,                                      /* drcr                    */
                                                     /*  SSLN=0(not Negate)     */
                                                     /*  RBURST=1(2 Burst)      */
                                                     /*  RCF=0                  */
                                                     /*  RBE=1(Burst Enable)    */
                                                     /*  SSLE=1(KEEP)           */
    SFLASH_FINALIZE_DISABLE,                         /* finalize_flag           */

    (uint32_t)&Load$$LR$$LOAD_MODULE2$$Base,         /* prog_start_addr_nml     */
    (uint32_t)&Load$$LR$$LOAD_MODULE2$$Length,       /* prog_area_size_nml      */
    (uint32_t)&Image$$LOADER_RESET_HANDLER$$Base,    /* prog_start_ram_addr_nml */

    0xffffffff,                                      /* duumy1                  */
    0xffffffff,                                      /* duumy2                  */
    0xffffffff,                                      /* duumy3                  */
    0xffffffff,                                      /* duumy4                  */
    0xffffffff,                                      /* duumy5                  */
    0xffffffff,                                      /* duumy6                  */
    0xffffffff,                                      /* duumy7                  */
    0xffffffff,                                      /* duumy8                  */
    0xffffffff,                                      /* duumy9                  */
    0xffffffff,                                      /* duumy10                 */

    0x17320508                                       /* check_sum(DUMMY)        */
};
#pragma arm section rodata

#endif  // __CC_ARM

/* End of File */


