/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : Renesas_RZ_T1.h
* Version      : 1.0
* Device       : R7S910017
* Abstract     : Cortex-R4F Macro definition
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : CMSIS Cortex-R4F Core Peripheral Access Layer Header File for the Renesas_RZ_T1
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

#ifndef __Renesas_RZ_T1__
#define __Renesas_RZ_T1__

#ifdef __cplusplus
extern "C" {
#endif


/* -------------------------  Interrupt Number Definition  ------------------------ */

typedef enum IRQn
{
  INTCR4_IRQn         =   1,
  INTCTI_IRQn         =   2,
  FPUEX_IRQn          =   3,
  IRQ0_IRQn           =   4,
  IRQ1_IRQn           =   5,
  IRQ2_IRQn           =   6,
  IRQ3_IRQn           =   7,
  IRQ4_IRQn           =   8,
  IRQ5_IRQn           =   9,
  IRQ6_IRQn           =  10,
  IRQ7_IRQn           =  11,
  IRQ8_IRQn           =  12,
  IRQ9_IRQn           =  13,
  IRQ10_IRQn          =  14,
  IRQ11_IRQn          =  15,
  IRQ12_IRQn          =  16,
  IRQ13_IRQn          =  17,
  IRQ14_IRQn          =  18,
  IRQ15_IRQn          =  19,

  ERRD_IRQn           =  20,
/* CMT Unit0        */
  CMI0_IRQn           =  21,
  CMI1_IRQn           =  22,
/* CMT_Unit1        */
  CMI2_IRQn           =  23,
  CMI3_IRQn           =  24,
/* CMTW Unit0       */
  CMWI0_IRQn          =  25,
  IC0I0_IRQn          =  26,
  IC1I0_IRQn          =  27,
  OC0I0_IRQn          =  28,
  OC1I0_IRQn          =  29,
/* CMTW Unit1       */
  CMWI1_IRQn          =  30,
  IC0I1_IRQn          =  31,
  IC1I1_IRQn          =  32,
  OC0I1_IRQn          =  33,
  OC1I1_IRQn          =  34,
/* ADC Unit0        */
  S12ADI0_IRQn        =  35,
  S2GBADI0_IRQn       =  36,
  S12CMPI0_IRQn       =  37,
/* ADC Unit1        */
  S12ADI1_IRQn        =  38,
  S2GBADI1_IRQn       =  39,
  S12CMPI1_IRQn       =  40,
/* USB              */
  USBHI_IRQn          =  41,
  USBFI_IRQn          =  42,
  USBFDR1_IRQn        =  43,
  USBFDR2_IRQn        =  44,
/* Switch with IEEE1588 */
  ETHSWI_IRQn         =  45,
  ETHSWDLRI_IRQn      =  46,
  ETHSWSOI_IRQn       =  47,
/* Ether PHY        */
  ETHPHYI0_IRQn       =  48,
  ETHPHYI1_IRQn       =  49,
  ETHPHYI2_IRQn       =  50,
/* Ether MAC        */
  ETHDMAIR_IRQn       =  51,
  ETHDMAIT_IRQn       =  52,
  ETHRFI_IRQn         =  53,
/* MTU3a            */
  TGIA6_IRQn          =  54,
  TGIB6_IRQn          =  55,
  TGIC6_IRQn          =  56,
  TGID6_IRQn          =  57,
  TCIV6_IRQn          =  58,
  TGIA7_IRQn          =  59,
  TGIB7_IRQn          =  60,
  TGIC7_IRQn          =  61,
  TGID7_IRQn          =  62,
  TGIV7_IRQn          =  63,
/* Ether MAC        */
  ETHMMAI_IRQn        =  64,
  ETHPPIT_IRQn        =  65,
  ETHIT_IRQn          =  66,
  ETHRFIV_IRQn        =  67,
  ETHTFIU_IRQn        =  68,
  ETHTFIE_IRQn        =  69,
  ETHRFE_IRQn         =  70,
  ETHLPIST_IRQn       =  71,
  ETHLPIEND_IRQn      =  72,
/* EtherCAT Slave   */
  ETHCSI0_IRQn        =  73,
  ETHCSI1_IRQn        =  74,
  ETHCI_IRQn          =  75,
  ETHCSOFI_IRQn       =  76,
  ETHCEOFI_IRQn       =  77,
  ETHCWDTI_IRQn       =  78,
  ETHCRSTI_IRQn       =  79,
/* RSPI Unit0       */
  SPRI0_IRQn          =  80,
  SPTI0_IRQn          =  81,
  SPEI0_IRQn          =  82,
  SPII0_IRQn          =  83,
/* RSPI Unit1       */
  SPRI1_IRQn          =  84,
  SPTI1_IRQn          =  85,
  SPEI1_IRQn          =  86,
  SPII1_IRQn          =  87,
/* RSPI Unit2       */
  SPRI2_IRQn          =  88,
  SPTI2_IRQn          =  89,
  SPEI2_IRQn          =  90,
  SPII2_IRQn          =  91,
/* RSPI Unit2       */
  SPRI3_IRQn          =  92,
  SPTI3_IRQn          =  93,
  SPEI3_IRQn          =  94,
  SPII3_IRQn          =  95,
/* SCIFA Unit0      */
  BRIF0_IRQn          =  96,
  RXIF0_IRQn          =  97,
  TXIF0_IRQn          =  98,
  DRIF0_IRQn          =  99,
/* SCIFA Unit1      */
  BRIF1_IRQn          = 100,
  RXIF1_IRQn          = 101,
  TXIF1_IRQn          = 102,
  DRIF1_IRQn          = 103,
/* RSCAN            */
  CANRFI_IRQn         = 104,
  CANFIR0_IRQn        = 105,
  CANTI0_IRQn         = 106,
  CANFIR1_IRQn        = 107,
  CANTI1_IRQn         = 108,
/* SCIFA Unit1      */
  BRIF2_IRQn          = 109,
  RXIF2_IRQn          = 110,
  TXIF2_IRQn          = 111,
  DRIF2_IRQn          = 112,
/* SCIFA Unit3      */
  BRIF3_IRQn          = 113,
  RXIF3_IRQn          = 114,
  TXIF3_IRQn          = 115,
  DRIF3_IRQn          = 116,
/* SCIFA Unit4      */
  BRIF4_IRQn          = 117,
  RXIF4_IRQn          = 118,
  TXIF4_IRQn          = 119,
  DRIF4_IRQn          = 120,
/* RIIC Unit0       */
  REI0_IRQn           = 121,
  RXI0_IRQn           = 122,
  TXI0_IRQn           = 123,
/* RIIC Unit1       */
  REI1_IRQn           = 124,
  RXI1_IRQn           = 125,
  TXI1_IRQn           = 126,
/* SSI              */
  SSIRXI              = 127,
  SSITXI              = 128,
/* Reserved         */
  VEC_129_IRQn        = 129,
  VEC_130_IRQn        = 130,
  VEC_131_IRQn        = 131,
  VEC_132_IRQn        = 132,
  VEC_133_IRQn        = 133,
  VEC_134_IRQn        = 134,
  VEC_135_IRQn        = 135,
  VEC_136_IRQn        = 136,
  VEC_137_IRQn        = 137,
  VEC_138_IRQn        = 138,
  VEC_139_IRQn        = 139,
  VEC_140_IRQn        = 140,
  VEC_141_IRQn        = 141,
  VEC_142_IRQn        = 142,
  VEC_143_IRQn        = 143,
  VEC_144_IRQn        = 144,
/* MTU3a            */
  TGIA0_IRQn          = 145,
  TGIB0_IRQn          = 146,
  TGIC0_IRQn          = 147,
  TGID0_IRQn          = 148,
  TGIE0_IRQn          = 149,
  TGIF0_IRQn          = 150,
  TGIV0_IRQn          = 151,
  TGIA1_IRQn          = 152,
  TGIB1_IRQn          = 153,
  TGIV1_IRQn          = 154,
  TGIU1_IRQn          = 155,
  TGIA2_IRQn          = 156,
  TGIB2_IRQn          = 157,
  TGIV2_IRQn          = 158,
  IGIU2_IRQn          = 159,
  TGIA3_IRQn          = 160,
  TGIB3_IRQn          = 161,
  TGIC3_IRQn          = 162,
  TGID3_IRQn          = 163,
  TGIV3_IRQn          = 164,
  TGIA4_IRQn          = 165,
  TGIB4_IRQn          = 166,
  TGIC4_IRQn          = 167,
  TGID4_IRQn          = 168,
  TCIV4_IRQn          = 169,
  TGIU5_IRQn          = 170,
  TGIV5_IRQn          = 171,
  TGIW5_IRQn          = 172,
  TGIA8_IRQn          = 173,
  TGIB8_IRQn          = 174,
  TGIC8_IRQn          = 175,
  TGID8_IRQn          = 176,
  TCIV8_IRQn          = 177,
/* GPTA             */
  GTCIA0_IRQn         = 178,
  GTCIB0_IRQn         = 179,
  GTCIC0_IRQn         = 180,
  GTCID0_IRQn         = 181,
  GTCIE0_IRQn         = 182,
  GTCIF0_IRQn         = 183,
  GDTE0_IRQn          = 184,
  GTCIV0_IRQn         = 185,
  GTCIU0_IRQn         = 186,
  GTCIA1_IRQn         = 187,
  GTCIB1_IRQn         = 188,
  GTCIC1_IRQn         = 189,
  GTCID1_IRQn         = 190,
  GTCIE1_IRQn         = 191,
  GTCIF1_IRQn         = 192,
  GDTE1_IRQn          = 193,
  GTCIV1_IRQn         = 194,
  GTCIU1_IRQn         = 195,
  GTCIA2_IRQn         = 196,
  GTCIB2_IRQn         = 197,
  GTCIC2_IRQn         = 198,
  GTCID2_IRQn         = 199,
  GTCIE2_IRQn         = 200,
  GTCIF2_IRQn         = 201,
  GDTE2_IRQn          = 202,
  GTCIV2_IRQn         = 203,
  GTCIU2_IRQn         = 204,
  GTCIA3_IRQn         = 205,
  GTCIB3_IRQn         = 206,
  GTCIC3_IRQn         = 207,
  GTCID3_IRQn         = 208,
  GTCIE3_IRQn         = 209,
  GTCIF3_IRQn         = 210,
  GDTE3_IRQn          = 211,
  GTCIV3_IRQn         = 212,
  GTCIU3_IRQn         = 213,
  ETGIP_IRQn          = 214,
  ETGIN_IRQn          = 215,
/* TPUa Unit0       */
  TGI0A_IRQn          = 216,
  TGI0B_IRQn          = 217,
  TGI0C_IRQn          = 218,
  TGI0D_IRQn          = 219,
  TCI0V_IRQn          = 220,
  TGI1A_IRQn          = 221,
  TGI1B_IRQn          = 222,
  TCI1V_IRQn          = 223,
  TCI1U_IRQn          = 224,
  TGI2A_IRQn          = 225,
  TGI2B_IRQn          = 226,
  TCI2V_IRQn          = 227,
  TCI2U_IRQn          = 228,
  TGI3A_IRQn          = 229,
  TGI3B_IRQn          = 230,
  TGI3C_IRQn          = 231,
  TGI3D_IRQn          = 232,
  TCI3V_IRQn          = 233,
  TGI4A_IRQn          = 234,
  TGI4B_IRQn          = 235,
  TCI4V_IRQn          = 236,
  TCI4U_IRQn          = 237,
  TGI5A_IRQn          = 238,
  TGI5B_IRQn          = 239,
  TCI5V_IRQn          = 240,
  TCI5U_IRQn          = 241,
/* ELC              */
  ELCIRQ1_IRQn        = 242,
  ELCIRQ2_IRQn        = 243,
/* Reserved         */
  VEC_244_IRQn        = 244,
  VEC_245_IRQn        = 245,
/* Ether MAC        */
  ETHDRIE_IRQn        = 246,
  ETHDTIE_IRQn        = 247,
/* DMAC             */
  DMAINT0_IRQn        = 248,
  DMAINT1_IRQn        = 249,
  DMAINT2_IRQn        = 250,
  DMASRQ0_IRQn        = 251,
  DMASRQ1_IRQn        = 252,
/* Reserved         */
  VEC_253_IRQn        = 253,
/* CM3              */
  SRQCM3_IRQn         = 254,
/* Reserved         */
  VEC_255_IRQn        = 255,
/* POE3             */
  OEI1_IRQn           = 256,
  OEI2_IRQn           = 257,
  OEI3_IRQn           = 258,
  OEI4_IRQn           = 259,
/* RIIC Unit0       */
  EEI0_IRQn           = 260,
/* RIIC Unitq       */
  EEI1_IRQn           = 261,
/* RSCAN            */
  CANGE_IRQn          = 262,
  CANIE0_IRQn         = 263,
  CANIE1_IRQn         = 264,
/* BSC              */
  BSCCMI_IRQn         = 265,
/* SSI              */
  SSIF                = 266,
/* TPUa Unit1       */
  TGI6A_IRQn          = 267,
  TGI6B_IRQn          = 268,
  TGI6C_IRQn          = 269,
  TGI6D_IRQn          = 270,
  TCI6V_IRQn          = 271,
  TGI7A_IRQn          = 272,
  TGI7B_IRQn          = 273,
  TCI7V_IRQn          = 274,
  TCI7U_IRQn          = 275,
  TGI8A_IRQn          = 276,
  TGI8B_IRQn          = 277,
  TCI8V_IRQn          = 278,
  TCI8U_IRQn          = 279,
  TGI9A_IRQn          = 280,
  TGI9B_IRQn          = 281,
  TGI9C_IRQn          = 282,
  TGI9D_IRQn          = 283,
  TCI9V_IRQn          = 284,
  TGI10A_IRQn         = 285,
  TGI10B_IRQn         = 286,
  TCI10V_IRQn         = 287,
  TCI10U_IRQn         = 288,
  TGI11A_IRQn         = 289,
  TGI11B_IRQn         = 290,
/* TPUa Unit1       */
  TCI11V_IRQn         = 291,
  TCI11U_IRQn         = 292,
/* DMAC             */
  DMAERR0_IRQn        = 293,
  DMAERR1_IRQn        = 294,
/* Reserved         */
  VEC_295_IRQn        = 295,
  VEC_296_IRQn        = 296,
  VEC_297_IRQn        = 297,
  VEC_298_IRQn        = 298,
/* CMT Unit2        */
  CMI4_IRQn           = 299,
  CMI5_IRQn           = 300
} IRQn_Type;

#define Renesas_RZ_T1_IRQ_MAX  300

/* --------  Configuration of the Cortex-R4F Processor and Core Peripherals  ------- */
#define __CR4_REV                 0x0000    /*!< Core revision r0                                */

#include <core_cr4f.h>
#include "system_Renesas_RZ_T1.h"


/******************************************************************************/
/*                Device Specific Peripheral Section                          */
/******************************************************************************/
/** @addtogroup Renesas_RZ_T1_Peripherals Renesas_RZ_T1 Peripherals
  Renesas_RZ_T1 Device Specific Peripheral registers structures
  @{
*/

#if defined ( __CC_ARM   )
#pragma anon_unions
#endif

#include "vic.h"

#if defined ( __CC_ARM   )
#pragma no_anon_unions
#endif

/*@}*/ /* end of group Renesas_RZ_A1_Peripherals */


/******************************************************************************/
/*                         Peripheral memory map                              */
/******************************************************************************/
/** @addtogroup Renesas_RZ_T1_MemoryMap Renesas_RZ_T1 Memory Mapping
  @{
*/

/* R7S72100 CPU board  */
#define Renesas_RZ_T1_NORFLASH_BASE0               (0x00000000UL)                        /*!< (FLASH0    ) Base Address */
#define Renesas_RZ_T1_NORFLASH_BASE1               (0x04000000UL)                        /*!< (FLASH1    ) Base Address */
#define Renesas_RZ_T1_SDRAM_BASE0                  (0x08000000UL)                        /*!< (SDRAM0    ) Base Address */
#define Renesas_RZ_T1_SDRAM_BASE1                  (0x0C000000UL)                        /*!< (SDRAM1    ) Base Address */
#define Renesas_RZ_T1_USER_AREA0                   (0x10000000UL)                        /*!< (USER0     ) Base Address */
#define Renesas_RZ_T1_USER_AREA1                   (0x14000000UL)                        /*!< (USER1     ) Base Address */
#define Renesas_RZ_T1_SPI_IO0                      (0x18000000UL)                        /*!< (SPI_IO0   ) Base Address */
#define Renesas_RZ_T1_SPI_IO1                      (0x1C000000UL)                        /*!< (SPI_IO1   ) Base Address */
#define Renesas_RZ_T1_ONCHIP_SRAM_BASE             (0x20000000UL)                        /*!< (SRAM_OC   ) Base Address */
#define Renesas_RZ_T1_SPI_MIO_BASE                 (0x3fe00000UL)                        /*!< (SPI_MIO   ) Base Address */
#define Renesas_RZ_T1_BSC_BASE                     (0x3ff00000UL)                        /*!< (BSC       ) Base Address */
#define Renesas_RZ_T1_PERIPH_BASE0                 (0xe8000000UL)                        /*!< (PERIPH0   ) Base Address */
#define Renesas_RZ_T1_PERIPH_BASE1                 (0xfcf00000UL)                        /*!< (PERIPH1   ) Base Address */
#define Renesas_RZ_T1_VIC_BASE                     (0xA0010000UL)                        /*!< (VIC    T  ) Base Address */

/*@}*/ /* end of group Renesas_RZ_T1_MemoryMap */

/******************************************************************************/
/*                         Clock Settings                                     */
/******************************************************************************/
/** @addtogroup Renesas_RZ_A1_H_Clocks Renesas_RZ_A1 Clock definitions
  @{
*/

/*
 * Clock Mode 0 settings
 * SW1-4(MD_CLK):ON
 * SW1-5(MD_CLKS):ON
 * FRQCR=0x1035
 *   CLKEN2    = 0b - unstable
 *   CLKEN[1:0]=01b - Output, Low, Low
 *   IFC[1:0]  =00b - CPU clock is 1/1 PLL clock
 * FRQCR2=0x0001
 *   GFC[1:0]  =01b - Graphic clock is 2/3 bus clock
 */
#define CM0_RENESAS_RZ_A1_CLKIN  ( 13333333u)
#define CM0_RENESAS_RZ_A1_CLKO   ( 66666666u)
#define CM0_RENESAS_RZ_A1_I_CLK  (400000000u)
#define CM0_RENESAS_RZ_A1_G_CLK  (266666666u)
#define CM0_RENESAS_RZ_A1_B_CLK  (133333333u)
#define CM0_RENESAS_RZ_A1_P1_CLK ( 66666666u)
#define CM0_RENESAS_RZ_A1_P0_CLK ( 33333333u)

/*
 * Clock Mode 1 settings
 * SW1-4(MD_CLK):OFF
 * SW1-5(MD_CLKS):ON
 * FRQCR=0x1335
 *   CLKEN2    = 0b - unstable
 *   CLKEN[1:0]=01b - Output, Low, Low
 *   IFC[1:0]  =11b - CPU clock is 1/3 PLL clock
 * FRQCR2=0x0003
 *   GFC[1:0]  =11b - graphic clock is 1/3 bus clock
 */
#define CM1_RENESAS_RZ_A1_CLKIN  ( 48000000u)
#define CM1_RENESAS_RZ_A1_CLKO   ( 64000000u)
#define CM1_RENESAS_RZ_A1_I_CLK  (128000000u)
#define CM1_RENESAS_RZ_A1_G_CLK  (128000000u)
#define CM1_RENESAS_RZ_A1_B_CLK  (128000000u)
#define CM1_RENESAS_RZ_A1_P1_CLK ( 64000000u)
#define CM1_RENESAS_RZ_A1_P0_CLK ( 32000000u)

/*@}*/ /* end of group Renesas_RZ_A1_Clocks */

/******************************************************************************/
/*                         CPG   Settings                                     */
/******************************************************************************/
/** @addtogroup Renesas_RZ_A1_H_CPG Renesas_RZ_A1 CPG Bit definitions
  @{
*/

#define CPG_FRQCR_SHIFT_CKOEN2  (14)
#define CPG_FRQCR_BIT_CKOEN2    (0x1 << CPG_FRQCR_SHIFT_CKOEN2)
#define CPG_FRQCR_SHIFT_CKOEN0  (12)
#define CPG_FRQCR_BITS_CKOEN0   (0x3 << CPG_FRQCR_SHIFT_CKOEN0)
#define CPG_FRQCR_SHIFT_IFC     (8)
#define CPG_FRQCR_BITS_IFC      (0x3 << CPG_FRQCR_SHIFT_IFC)

#define CPG_FRQCR2_SHIFT_GFC    (0)
#define CPG_FRQCR2_BITS_GFC     (0x3 << CPG_FRQCR2_SHIFT_GFC)


#define CPG_STBCR1_BIT_STBY     (0x80u)
#define CPG_STBCR1_BIT_DEEP     (0x40u)
#define CPG_STBCR2_BIT_HIZ      (0x80u)
#define CPG_STBCR2_BIT_MSTP20   (0x01u) /* CoreSight */
#define CPG_STBCR3_BIT_MSTP37   (0x80u) /* IEBus */
#define CPG_STBCR3_BIT_MSTP36   (0x40u) /* IrDA */
#define CPG_STBCR3_BIT_MSTP35   (0x20u) /* LIN0 */
#define CPG_STBCR3_BIT_MSTP34   (0x10u) /* LIN1 */
#define CPG_STBCR3_BIT_MSTP33   (0x08u) /* Multi-Function Timer */
#define CPG_STBCR3_BIT_MSTP32   (0x04u) /* CAN */
#define CPG_STBCR3_BIT_MSTP31   (0x02u) /* A/D converter (analog voltage) */
#define CPG_STBCR3_BIT_MSTP30   (0x01u) /* Motor Control PWM Timer */
#define CPG_STBCR4_BIT_MSTP47   (0x80u) /* SCIF0 */
#define CPG_STBCR4_BIT_MSTP46   (0x40u) /* SCIF1 */
#define CPG_STBCR4_BIT_MSTP45   (0x20u) /* SCIF2 */
#define CPG_STBCR4_BIT_MSTP44   (0x10u) /* SCIF3 */
#define CPG_STBCR4_BIT_MSTP43   (0x08u) /* SCIF4 */
#define CPG_STBCR4_BIT_MSTP42   (0x04u) /* SCIF5 */
#define CPG_STBCR4_BIT_MSTP41   (0x02u) /* SCIF6 */
#define CPG_STBCR4_BIT_MSTP40   (0x01u) /* SCIF7 */
#define CPG_STBCR5_BIT_MSTP57   (0x80u) /* SCI0 */
#define CPG_STBCR5_BIT_MSTP56   (0x40u) /* SCI1 */
#define CPG_STBCR5_BIT_MSTP55   (0x20u) /* Sound Generator0 */
#define CPG_STBCR5_BIT_MSTP54   (0x10u) /* Sound Generator1 */
#define CPG_STBCR5_BIT_MSTP53   (0x08u) /* Sound Generator2 */
#define CPG_STBCR5_BIT_MSTP52   (0x04u) /* Sound Generator3 */
#define CPG_STBCR5_BIT_MSTP51   (0x02u) /* OSTM0 */
#define CPG_STBCR5_BIT_MSTP50   (0x01u) /* OSTM1 */
#define CPG_STBCR6_BIT_MSTP67   (0x80u) /* A/D converter (clock) */
#define CPG_STBCR6_BIT_MSTP66   (0x40u) /* Capture Engine */
#define CPG_STBCR6_BIT_MSTP65   (0x20u) /* Display out comparison0 */
#define CPG_STBCR6_BIT_MSTP64   (0x10u) /* Display out comparison1 */
#define CPG_STBCR6_BIT_MSTP63   (0x08u) /* Dynamic Range compression0 */
#define CPG_STBCR6_BIT_MSTP62   (0x04u) /* Dynamic Range compression1 */
#define CPG_STBCR6_BIT_MSTP61   (0x02u) /* JPEG Decoder */
#define CPG_STBCR6_BIT_MSTP60   (0x01u) /* Realtime Clock */
#define CPG_STBCR7_BIT_MSTP77   (0x80u) /* Video Decoder0 */
#define CPG_STBCR7_BIT_MSTP76   (0x40u) /* Video Decoder1 */
#define CPG_STBCR7_BIT_MSTP74   (0x10u) /* Ethernet */
#define CPG_STBCR7_BIT_MSTP73   (0x04u) /* NAND Flash Memory Controller */
#define CPG_STBCR7_BIT_MSTP71   (0x02u) /* USB0 */
#define CPG_STBCR7_BIT_MSTP70   (0x01u) /* USB1 */
#define CPG_STBCR8_BIT_MSTP87   (0x80u) /* IMR-LS2_0 */
#define CPG_STBCR8_BIT_MSTP86   (0x40u) /* IMR-LS2_1 */
#define CPG_STBCR8_BIT_MSTP85   (0x20u) /* IMR-LSD */
#define CPG_STBCR8_BIT_MSTP84   (0x10u) /* MMC Host Interface */
#define CPG_STBCR8_BIT_MSTP83   (0x08u) /* MediaLB */
#define CPG_STBCR8_BIT_MSTP82   (0x04u) /* EthernetAVB */
#define CPG_STBCR8_BIT_MSTP81   (0x02u) /* SCUX */
#define CPG_STBCR9_BIT_MSTP97   (0x80u) /* RIIC0 */
#define CPG_STBCR9_BIT_MSTP96   (0x40u) /* RIIC1 */
#define CPG_STBCR9_BIT_MSTP95   (0x20u) /* RIIC2 */
#define CPG_STBCR9_BIT_MSTP94   (0x10u) /* RIIC3 */
#define CPG_STBCR9_BIT_MSTP93   (0x08u) /* SPI Multi I/O Bus Controller0 */
#define CPG_STBCR9_BIT_MSTP92   (0x04u) /* SPI Multi I/O Bus Controller1 */
#define CPG_STBCR9_BIT_MSTP91   (0x02u) /* VDC5_0 */
#define CPG_STBCR9_BIT_MSTP90   (0x01u) /* VDC5_1 */
#define CPG_STBCR10_BIT_MSTP107 (0x80u) /* RSPI0 */
#define CPG_STBCR10_BIT_MSTP106 (0x40u) /* RSPI1 */
#define CPG_STBCR10_BIT_MSTP105 (0x20u) /* RSPI2 */
#define CPG_STBCR10_BIT_MSTP104 (0x10u) /* RSPI3 */
#define CPG_STBCR10_BIT_MSTP103 (0x08u) /* RSPI4 */
#define CPG_STBCR10_BIT_MSTP102 (0x04u) /* ROMDEC */
#define CPG_STBCR10_BIT_MSTP101 (0x02u) /* SPIDF */
#define CPG_STBCR10_BIT_MSTP100 (0x01u) /* OpenVG */
#define CPG_STBCR11_BIT_MSTP115 (0x20u) /* SSIF0 */
#define CPG_STBCR11_BIT_MSTP114 (0x10u) /* SSIF1 */
#define CPG_STBCR11_BIT_MSTP113 (0x08u) /* SSIF2 */
#define CPG_STBCR11_BIT_MSTP112 (0x04u) /* SSIF3 */
#define CPG_STBCR11_BIT_MSTP111 (0x02u) /* SSIF4 */
#define CPG_STBCR11_BIT_MSTP110 (0x01u) /* SSIF5 */
#define CPG_STBCR12_BIT_MSTP123 (0x08u) /* SD Host Interface00 */
#define CPG_STBCR12_BIT_MSTP122 (0x04u) /* SD Host Interface01 */
#define CPG_STBCR12_BIT_MSTP121 (0x02u) /* SD Host Interface10 */
#define CPG_STBCR12_BIT_MSTP120 (0x01u) /* SD Host Interface11 */
#define CPG_STBCR13_BIT_MSTP132 (0x04u) /* PFV1 */
#define CPG_STBCR13_BIT_MSTP131 (0x02u) /* PFV0 */
#define CPG_SWRSTCR1_BIT_AXTALE (0x80u) /* AUDIO_X1 */
#define CPG_SWRSTCR1_BIT_SRST16 (0x40u) /* SSIF0 */
#define CPG_SWRSTCR1_BIT_SRST15 (0x20u) /* SSIF1 */
#define CPG_SWRSTCR1_BIT_SRST14 (0x10u) /* SSIF2 */
#define CPG_SWRSTCR1_BIT_SRST13 (0x08u) /* SSIF3 */
#define CPG_SWRSTCR1_BIT_SRST12 (0x04u) /* SSIF4 */
#define CPG_SWRSTCR1_BIT_SRST11 (0x02u) /* SSIF5 */
#define CPG_SWRSTCR2_BIT_SRST21 (0x02u) /* JPEG Decoder */
#define CPG_SWRSTCR3_BIT_SRST32 (0x04u) /* OpenVG */
#define CPG_SYSCR1_BIT_VRAME4   (0x10u) /* VRAM E Page4 */
#define CPG_SYSCR1_BIT_VRAME3   (0x08u) /* VRAM E Page3 */
#define CPG_SYSCR1_BIT_VRAME2   (0x04u) /* VRAM E Page2 */
#define CPG_SYSCR1_BIT_VRAME1   (0x02u) /* VRAM E Page1 */
#define CPG_SYSCR1_BIT_VRAME0   (0x01u) /* VRAM E Page0 */
#define CPG_SYSCR2_BIT_VRAMWE4  (0x10u) /* VRAM WE Page4 */
#define CPG_SYSCR2_BIT_VRAMWE3  (0x08u) /* VRAM WE Page3 */
#define CPG_SYSCR2_BIT_VRAMWE2  (0x04u) /* VRAM WE Page2 */
#define CPG_SYSCR2_BIT_VRAMWE1  (0x02u) /* VRAM WE Page1 */
#define CPG_SYSCR2_BIT_VRAMWE0  (0x01u) /* VRAM WE Page0 */
#define CPG_SYSCR3_BIT_RRAMWE3  (0x08u) /* RRAM WE Page3 */
#define CPG_SYSCR3_BIT_RRAMWE2  (0x04u) /* RRAM WE Page2 */
#define CPG_SYSCR3_BIT_RRAMWE1  (0x02u) /* RRAM WE Page1 */
#define CPG_SYSCR3_BIT_RRAMWE0  (0x01u) /* RRAM WE Page0 */
#define CPG_CPUSTS_BIT_ISBUSY   (0x10u) /* State during Changing of the Frequency of CPU and Return from Software Standby */
#define CPG_STBREQ1_BIT_STBRQ15 (0x20u) /* CoreSight */
#define CPG_STBREQ1_BIT_STBRQ13 (0x08u) /* JPEG Control */
#define CPG_STBREQ1_BIT_STBRQ12 (0x04u) /* EthernetAVB */
#define CPG_STBREQ1_BIT_STBRQ10 (0x01u) /* Capture Engine */
#define CPG_STBREQ2_BIT_STBRQ27 (0x80u) /* MediaLB */
#define CPG_STBREQ2_BIT_STBRQ26 (0x40u) /* Ethernet */
#define CPG_STBREQ2_BIT_STBRQ25 (0x20u) /* VDC5_0 */
#define CPG_STBREQ2_BIT_STBRQ24 (0x10u) /* VCD5_1 */
#define CPG_STBREQ2_BIT_STBRQ23 (0x08u) /* IMR_LS2_0 */
#define CPG_STBREQ2_BIT_STBRQ22 (0x04u) /* IMR_LS2_1 */
#define CPG_STBREQ2_BIT_STBRQ21 (0x02u) /* IMR_LSD */
#define CPG_STBREQ2_BIT_STBRQ20 (0x01u) /* OpenVG */
#define CPG_STBACK1_BIT_STBAK15 (0x20u) /* CoreSight */
#define CPG_STBACK1_BIT_STBAK13 (0x08u) /* JPEG Control */
#define CPG_STBACK1_BIT_STBAK12 (0x04u) /* EthernetAVB */
#define CPG_STBACK1_BIT_STBAK10 (0x01u) /* Capture Engine */
#define CPG_STBACK2_BIT_STBAK27 (0x80u) /* MediaLB */
#define CPG_STBACK2_BIT_STBAK26 (0x40u) /* Ethernet */
#define CPG_STBACK2_BIT_STBAK25 (0x20u) /* VDC5_0 */
#define CPG_STBACK2_BIT_STBAK24 (0x10u) /* VCD5_1 */
#define CPG_STBACK2_BIT_STBAK23 (0x08u) /* IMR_LS2_0 */
#define CPG_STBACK2_BIT_STBAK22 (0x04u) /* IMR_LS2_1 */
#define CPG_STBACK2_BIT_STBAK21 (0x02u) /* IMR_LSD */
#define CPG_STBACK2_BIT_STBAK20 (0x01u) /* OpenVG */
#define CPG_RRAMKP_BIT_RRAMKP3  (0x08u) /* RRAM KP Page3 */
#define CPG_RRAMKP_BIT_RRAMKP2  (0x04u) /* RRAM KP Page2 */
#define CPG_RRAMKP_BIT_RRAMKP1  (0x02u) /* RRAM KP Page1 */
#define CPG_RRAMKP_BIT_RRAMKP0  (0x01u) /* RRAM KP Page0 */
#define CPG_DSCTR_BIT_EBUSKEEPE (0x80u) /* Retention of External Memory Control Pin State */
#define CPG_DSCTR_BIT_RAMBOOT   (0x40u) /* Selection of Method after Returning from Deep Standby Mode */
#define CPG_DSSSR_BIT_P6_2      (0x4000u) /* P6_2 */
#define CPG_DSSSR_BIT_P3_9      (0x2000u) /* P3_9 */
#define CPG_DSSSR_BIT_P3_1      (0x1000u) /* P3_1 */
#define CPG_DSSSR_BIT_P2_12     (0x0800u) /* P2_12 */
#define CPG_DSSSR_BIT_P8_7      (0x0400u) /* P8_7 */
#define CPG_DSSSR_BIT_P3_3      (0x0200u) /* P3_3 */
#define CPG_DSSSR_BIT_NMI       (0x0100u) /* NMI */
#define CPG_DSSSR_BIT_RTCAR     (0x0040u) /* RTCAR */
#define CPG_DSSSR_BIT_P6_4      (0x0020u) /* P6_4 */
#define CPG_DSSSR_BIT_P5_9      (0x0010u) /* P5_9 */
#define CPG_DSSSR_BIT_P7_8      (0x0008u) /* P7_8 */
#define CPG_DSSSR_BIT_P2_15     (0x0004u) /* P2_15 */
#define CPG_DSSSR_BIT_P9_1      (0x0002u) /* P9_1 */
#define CPG_DSSSR_BIT_P8_2      (0x0001u) /* P8_2 */
#define CPG_DSESR_BIT_P6_2E     (0x4000u) /* P6_2 */
#define CPG_DSESR_BIT_P3_9E     (0x2000u) /* P3_9 */
#define CPG_DSESR_BIT_P3_1E     (0x1000u) /* P3_1 */
#define CPG_DSESR_BIT_P2_12E    (0x0800u) /* P2_12 */
#define CPG_DSESR_BIT_P8_7E     (0x0400u) /* P8_7 */
#define CPG_DSESR_BIT_P3_3E     (0x0200u) /* P3_3 */
#define CPG_DSESR_BIT_NMIE      (0x0100u) /* NMI */
#define CPG_DSESR_BIT_P6_4E     (0x0020u) /* P6_4 */
#define CPG_DSESR_BIT_P5_9E     (0x0010u) /* P5_9 */
#define CPG_DSESR_BIT_P7_8E     (0x0008u) /* P7_8 */
#define CPG_DSESR_BIT_P2_15E    (0x0004u) /* P2_15 */
#define CPG_DSESR_BIT_P9_1E     (0x0002u) /* P9_1 */
#define CPG_DSESR_BIT_P8_2E     (0x0001u) /* P8_2 */
#define CPG_DSFR_BIT_IOKEEP     (0x8000u) /* Release of Pin State Retention */
#define CPG_DSFR_BIT_P6_2F      (0x4000u) /* P6_2 */
#define CPG_DSFR_BIT_P3_9F      (0x2000u) /* P3_9 */
#define CPG_DSFR_BIT_P3_1F      (0x1000u) /* P3_1 */
#define CPG_DSFR_BIT_P2_12F     (0x0800u) /* P2_12 */
#define CPG_DSFR_BIT_P8_7F      (0x0400u) /* P8_7 */
#define CPG_DSFR_BIT_P3_3F      (0x0200u) /* P3_3 */
#define CPG_DSFR_BIT_NMIF       (0x0100u) /* NMI */
#define CPG_DSFR_BIT_RTCARF     (0x0040u) /* RTCAR */
#define CPG_DSFR_BIT_P6_4F      (0x0020u) /* P6_4 */
#define CPG_DSFR_BIT_P5_9F      (0x0010u) /* P5_9 */
#define CPG_DSFR_BIT_P7_8F      (0x0008u) /* P7_8 */
#define CPG_DSFR_BIT_P2_15F     (0x0004u) /* P2_15 */
#define CPG_DSFR_BIT_P9_1F      (0x0002u) /* P9_1 */
#define CPG_DSFR_BIT_P8_2F      (0x0001u) /* P8_2 */
#define CPG_XTALCTR_BIT_GAIN1   (0x02u)   /* RTC_X3, RTC_X4 */
#define CPG_XTALCTR_BIT_GAIN0   (0x01u)   /* EXTAL, XTAL */

/*@}*/ /* end of group Renesas_RZ_A1_CPG */

/******************************************************************************/
/*                        GPIO   Settings                                     */
/******************************************************************************/
/** @addtogroup Renesas_RZ_A1_H_GPIO Renesas_RZ_A1 GPIO Bit definitions
  @{
*/

#define GPIO_BIT_N0  (1u <<  0)
#define GPIO_BIT_N1  (1u <<  1)
#define GPIO_BIT_N2  (1u <<  2)
#define GPIO_BIT_N3  (1u <<  3)
#define GPIO_BIT_N4  (1u <<  4)
#define GPIO_BIT_N5  (1u <<  5)
#define GPIO_BIT_N6  (1u <<  6)
#define GPIO_BIT_N7  (1u <<  7)
#define GPIO_BIT_N8  (1u <<  8)
#define GPIO_BIT_N9  (1u <<  9)
#define GPIO_BIT_N10 (1u << 10)
#define GPIO_BIT_N11 (1u << 11)
#define GPIO_BIT_N12 (1u << 12)
#define GPIO_BIT_N13 (1u << 13)
#define GPIO_BIT_N14 (1u << 14)
#define GPIO_BIT_N15 (1u << 15)


#define MD_BOOT10_MASK    (0x3)

#define MD_BOOT10_BM0     (0x0)
#define MD_BOOT10_BM1     (0x2)
#define MD_BOOT10_BM3     (0x1)
#define MD_BOOT10_BM4_5   (0x3)

#define MD_CLK        (1u << 2)
#define MD_CLKS       (1u << 3)

/*@}*/ /* end of group Renesas_RZ_A1_GPIO */

#ifdef __cplusplus
}
#endif

#endif  // __Renesas_RZ_A1_H__
