/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : boot_common.h
* Version      : 1.0
* Device       : R7S910017
* Abstract     : Definition for boot mode settings
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : Define the system settings of boot mode.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

#ifndef BOOT_COMMON_H
#define BOOT_COMMON_H


/******************************************************************************
Macro definitions
******************************************************************************/
#define BOOT_OK            ( 0)
#define BOOT_ERR           (-1)
#define BOOT_ERR2          (-2)

#define CACHE_DISABLE      0x00000000
#define CACHE_ENABLE       0x00000001

#define BOOT_BASIC_SIZE    (512)


/******************************************************************************
Typedef definitions
******************************************************************************/
typedef uint32_t    CHECKSUM_TYPE;

typedef struct st_boot_cache
{
    uint32_t    cache_flag;
} stBOOT_CACHE;

typedef struct st_boot_prog_parm
{
    uint32_t    prog_start_addr_nml;
    uint32_t    prog_area_size_nml;
    uint32_t    prog_start_ram_addr_nml;
} stBOOT_PROG_PARM;

typedef struct st_boot_dummy_parm
{
    uint32_t    dummy1;
    uint32_t    dummy2;
    uint32_t    dummy3;
    uint32_t    dummy4;
    uint32_t    dummy5;
    uint32_t    dummy6;
    uint32_t    dummy7;
    uint32_t    dummy8;
    uint32_t    dummy9;
    uint32_t    dummy10;
} stBOOT_DUMMY_PARM;


/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
#if defined (__CC_ARM)
extern uint32_t    Image$$LOADER_RESET_HANDLER$$Base;
extern uint32_t    Load$$LR$$LOAD_MODULE2$$Base;
extern uint32_t    Load$$LR$$LOAD_MODULE2$$Length;
#endif

#endif /* BOOT_COMMON_H */

/* End of File */
