/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : interface header file for ADC fit module.
* File Name    : r_adc_if.h
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for ADC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ADC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_ADC_IF_HEADER_
#define _R_ADC_IF_HEADER_

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"
#include "r_errno.h"

#include "r_adc_config.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
/* Version Number of API. */
#define ADC_VERSION_MAJOR       (1)
#define ADC_VERSION_MINOR       (0)

#define ADC_MASK_GROUPB_OFF     (0)
#define ADC_MASK_ADD_OFF        (0)

/* Bitwise OR these masks together for desired channels */
#define ADC_MASK_CH0            (1<<0)
#define ADC_MASK_CH1            (1<<1)
#define ADC_MASK_CH2            (1<<2)
#define ADC_MASK_CH3            (1<<3)
#define ADC_MASK_CH4            (1<<4)
#define ADC_MASK_CH5            (1<<5)
#define ADC_MASK_CH6            (1<<6)
#define ADC_MASK_CH7            (1<<7)

#define ADC_SST_CNT_MIN         (5)         /* minimum sampling state count */
#define ADC_SST_CNT_MAX         (255)
#define ADC_SST_CNT_DEFAULT     (11)

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/* ADC_OPEN() ARGUMENT DEFINITIONS */
typedef enum e_adc_mode
{
    ADC_MODE_SS_TEMPERATURE,                /* single scan temperature sensor */
    ADC_MODE_SS_ONE_CH,                     /* single scan one channel */
    ADC_MODE_SS_ONE_CH_DBLTRIG,             /* on even triggers save to ADDBLDR & interrupt */
    ADC_MODE_SS_MULTI_CH,                   /* 1 trigger source, scan multiple channels */
    ADC_MODE_SS_MULTI_CH_GROUPED,           /* 2 trigger sources, scan multiple channels */
    ADC_MODE_SS_MULTI_CH_GROUPED_DBLTRIG_A,
    ADC_MODE_CONT_ONE_CH,                   /* continuous scan one channel */
    ADC_MODE_CONT_MULTI_CH,                 /* continuous scan multiple channels */
    ADC_MODE_MAX
} adc_mode_t;

/* trigger sources */
typedef enum e_adc_trig
{
    ADC_TRIG_ADTRG0                 = 0,
    ADC_TRIG_TRGA0N                 = 1,
    ADC_TRIG_TRGA1N                 = 2,
    ADC_TRIG_TRGA2N                 = 3,
    ADC_TRIG_TRGA3N                 = 4,
    ADC_TRIG_TRGA4N                 = 5,
    ADC_TRIG_TRGA6N                 = 6,
    ADC_TRIG_TRGA7N                 = 7,
    ADC_TRIG_TRG0N                  = 8,
    ADC_TRIG_TRG4AN                 = 9,
    ADC_TRIG_TRG4BN                 = 10,
    ADC_TRIG_TRG4AN_OR_TRG4BN       = 11,
    ADC_TRIG_TRG4ABN                = 12,
    ADC_TRIG_TRG7AN                 = 13,
    ADC_TRIG_TRG7BN                 = 14,
    ADC_TRIG_TRG7AN_OR_TRG7BN       = 15,
    ADC_TRIG_TRG7ABN                = 16,
    ADC_TRIG_GTADTRA0N              = 17,
    ADC_TRIG_GTADTRB0N              = 18,
    ADC_TRIG_GTADTRA1N              = 19,
    ADC_TRIG_GTADTRB1N              = 20,
    ADC_TRIG_GTADTRA2N              = 21,
    ADC_TRIG_GTADTRB2N              = 22,
    ADC_TRIG_GTADTRA3N              = 23,
    ADC_TRIG_GTADTRB3N              = 24,
    ADC_TRIG_GTADTRA0N_OR_GTADTRB0N = 25,
    ADC_TRIG_GTADTRA1N_OR_GTADTRB1N = 26,
    ADC_TRIG_GTADTRA2N_OR_GTADTRB2N = 27,
    ADC_TRIG_GTADTRA3N_OR_GTADTRB3N = 28,
    ADC_TRIG_TPTRGAN_0              = 31,
    ADC_TRIG_TPTRG0AN_0             = 32,
    ADC_TRIG_TPTRGAN_1              = 33,
    ADC_TRIG_TPTRG6AN_1             = 34,
    ADC_TRIG_ELCTRG0                = 48,
    ADC_TRIG_SOFTWARE               = 63
} adc_trig_t;

typedef enum e_adc_add
{
    ADC_ADD_OFF           = 0,      /* addition is turned off for chans/sensors */
    ADC_ADD_TWO_SAMPLES,
    ADC_ADD_THREE_SAMPLES,
    ADC_ADD_FOUR_SAMPLES,
    ADC_ADD_MAX
} adc_add_t;

typedef enum e_adc_align
{
    ADC_ALIGN_RIGHT = 0x0000,
    ADC_ALIGN_LEFT  = 0x8000
} adc_align_t;

typedef enum e_adc_clear
{
    ADC_CLEAR_AFTER_READ_OFF = 0x0000,
    ADC_CLEAR_AFTER_READ_ON  = 0x0020
} adc_clear_t;

typedef struct st_adc_cfg
{
    adc_add_t       add_cnt;
    adc_align_t     alignment;          /* ignored if addition used */
    adc_clear_t     clearing;
    adc_trig_t      trigger;            /* default and Group A trigger source */
    adc_trig_t      trigger_groupb;     /* valid only for group modes */
    uint8_t         priority;           /* S12ADIO interrupt priority; 0-15 */
    uint8_t         priority_groupb;    /* GBADI interrupt priority; 0-15 */
} adc_cfg_t;

/* CALLBACK FUNCTION ARGUMENT DEFINITIONS */
/* callback function events */
typedef enum e_adc_cb_evt
{
    ADC_EVT_SCAN_COMPLETE,              /* normal/Group A scan complete */
    ADC_EVT_SCAN_COMPLETE_GROUPB        /* Group B scan complete */
} adc_cb_evt_t;

typedef struct st_adc_cb_args           /* callback arguments */
{
    adc_cb_evt_t    event;
} adc_cb_args_t;

/* ADC_CONTROL() ARGUMENT DEFINITIONS */
typedef enum e_adc_cmd
{
    ADC_CMD_ENABLE_CHANS           = 1, /* enables chans and INT(s) if priority != 0 */
    ADC_CMD_ENABLE_TEMP_SENSOR,         /* enables sensor and INT if priority != 0 */
    ADC_CMD_SET_SAMPLE_STATE_CNT,
    ADC_CMD_ENABLE_TRIG,                /* allows an async/sync trigger to start scan */
    ADC_CMD_DISABLE_TRIG,               /* prevents an async/sync trigger to start scan */
    ADC_CMD_SCAN_NOW,                   /* issue software trigger */
    ADC_CMD_DISABLE_INT,                /* interrupt disable; ADCSR.ADIE=0 */
    ADC_CMD_ENABLE_INT,                 /* interrupt enable;  ADCSR.ADIE=1 */
    ADC_CMD_DISABLE_INT_GROUPB,         /* interrupt disable; ADCSR.GBADIE=0 */
    ADC_CMD_ENABLE_INT_GROUPB,          /* interrupt enable;  ADCSR.GBADIE=1 */
    ADC_CMD_CHECK_SCAN_DONE,            /* for Normal, GroupA or GroupB scan */
    ADC_CMD_CHECK_SCAN_DONE_GROUPA,
    ADC_CMD_CHECK_SCAN_DONE_GROUPB,
    ADC_CMD_MAX
} adc_cmd_t;

/* for ADC_CMD_ENABLE_CHANS */
typedef struct st_adc_ch_cfg            /* bit 0 is ch0; bit 7 is ch7 */
{
    uint32_t        chan_mask;          /* channels/bits 0-7 */
    uint32_t        chan_mask_groupb;   /* valid for group modes */
    uint32_t        add_mask;           /* valid if add enabled in Open() */
} adc_ch_cfg_t;

/* for ADC_CMD_SET_SAMPLE_STATE_CNT */
/* sample state registers */
typedef enum e_adc_sst_reg
{
    ADC_SST_CH0 = 0,
    ADC_SST_CH1,
    ADC_SST_CH2,
    ADC_SST_CH3,
    ADC_SST_CH4,
    ADC_SST_CH5,
    ADC_SST_CH6,
    ADC_SST_CH7,
    ADC_SST_TEMPERATURE,
    ADC_SST_NUM_REGS
} adc_sst_reg_t;

typedef struct st_adc_time
{
    adc_sst_reg_t   reg_id;
    uint8_t         num_states;         /* default=11 */
} adc_time_t;

/* ADC_READ() ARGUMENT DEFINITIONS */
typedef enum e_adc_reg
{
    ADC_REG_CH0 = 0,
    ADC_REG_CH1 = 1,
    ADC_REG_CH2 = 2,
    ADC_REG_CH3 = 3,
    ADC_REG_CH4 = 4,
    ADC_REG_CH5 = 5,
    ADC_REG_CH6 = 6,
    ADC_REG_CH7 = 7,
    ADC_REG_TEMP = 8,
    ADC_REG_DBLTRIG = 9,
    ADC_REG_MAX
} adc_reg_t;

/* ADC_READALL() ARGUMENT DEFINITIONS */
typedef struct st_adc_data
{
    uint16_t    chan[8];
    uint16_t    dbltrig;
} adc_data_t;

/*******************************************************************************
Public Functions
*******************************************************************************/
void     R_ADC_Init(void);
int32_t  R_ADC_Uninit(void);
int32_t  R_ADC_Open(adc_mode_t const mode, adc_cfg_t * const p_cfg, void (* const p_callback)(int32_t event));
int32_t  R_ADC_Control(adc_cmd_t const cmd, void * const p_args);
int32_t  R_ADC_Read(adc_reg_t const reg_id, uint16_t * const p_data);
int32_t  R_ADC_ReadAll(adc_data_t * const p_all_data);
int32_t  R_ADC_Close(void);
uint32_t R_ADC_GetVersion(void);

#endif /* _R_ADC_IF_HEADER_ */

/* End of File */

