/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : interface header file for CMT(W) fit module.
* File Name    : r_cmt_if.h
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMT(W) on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_CMT_IF_HEADER_
#define _R_CMT_IF_HEADER_

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"
#include "r_errno.h"

#include "r_cmt_config.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
/*----------------------------------------------------------------------------*/
/*   Define Version Number of API.                                            */
/*----------------------------------------------------------------------------*/
#define CMT_VERSION_MAJOR                       (1U)
#define CMT_VERSION_MINOR                       (0U)

/*----------------------------------------------------------------------------*/
/*   Define CMTW command for control.                                         */
/*----------------------------------------------------------------------------*/

#define CMT_CMD_SET_TIME_CNT                    (0U)    /* Command of setting time count          */
#define CMT_CMD_SET_MODE                        (1U)    /* Command of setting operation mode      */
#define CMT_CMD_SET_PAUSE                       (2U)    /* Command of counter pause               */
#define CMT_CMD_SET_RESUME                      (3U)    /* Command of counter restart with resume */
#define CMT_CMD_SET_RESTART                     (4U)    /* Command of counter restart with clear  */
#define CMT_CMD_SET_ECM                         (5U)    /* Command of setting ECM                 */
#define CMT_CMD_GET_STATUS                      (6U)    /* Command of getting counter status      */

/*----------------------------------------------------------------------------*/
/*   Define PCLKD divide rate parameter.                                      */
/*----------------------------------------------------------------------------*/
#define CMT_PCLK_DIV_8                          (0U)    /* Divide rate is PCLKD/8                  */
#define CMT_PCLK_DIV_32                         (1U)    /* Divide rate is PCLKD/32                 */
#define CMT_PCLK_DIV_128                        (2U)    /* Divide rate is PCLKD/128                */
#define CMT_PCLK_DIV_512                        (3U)    /* Divide rate is PCLKD/512                */
#define CMT_PCLK_DIV_MAX_OVER                   (4U)    /* Check for invalid divide rate parameter */

/*----------------------------------------------------------------------------*/
/*   Define counter size parameter.                                           */
/*----------------------------------------------------------------------------*/
#define CMTW_CNT_SIZE_32BIT                     (0U)    /* Counter size is 32bit            */
#define CMTW_CNT_SIZE_16BIT                     (1U)    /* Counter size is 16bit            */
#define CMTW_CNT_SIZE_MAX_OVER                  (2U)    /* Check for counter size parameter */

/*----------------------------------------------------------------------------*/
/*   Define counter clear factor parameter.                                   */
/*----------------------------------------------------------------------------*/
#define CMT_CNT_CLEAR_COMPARE_MATCH             (0U)    /* Counter clear factor is compare match    */
#define CMTW_CNT_CLEAR_COMPARE_MATCH            (0U)    /* Counter clear factor is compare match    */
#define CMTW_CNT_CLEAR_INPUT_CAPTURE0           (1U)    /* Counter clear factor is input capture0   */
#define CMTW_CNT_CLEAR_INPUT_CAPTURE1           (2U)    /* Counter clear factor is input capture1   */
#define CMTW_CNT_CLEAR_OUTPUT_COMPARE0          (3U)    /* Counter clear factor is output compare0  */
#define CMTW_CNT_CLEAR_OUTPUT_COMPARE1          (4U)    /* Counter clear factor is output compare1  */
#define CMTW_CNT_CLEAR_NONE                     (5U)    /* Counter clear factor is none             */
#define CMTW_CNT_CLEAR_MAX_OVER                 (6U)    /* Check for counter clear factor parameter */

/*----------------------------------------------------------------------------*/
/*   Define output signal setting for output compare.                         */
/*----------------------------------------------------------------------------*/
#define CMTW_OUT_COMPARE_VALUE_HOLD             (0U)    /* Output signal isn't change on output compare */
#define CMTW_OUT_COMPARE_VALUE_0_TO_TOGGLE      (1U)    /* Output signal is toggle on output compare and init value is 0 */
#define CMTW_OUT_COMPARE_VALUE_1_TO_TOGGLE      (2U)    /* Output signal is toggle on output compare and init value is 1 */
#define CMTW_OUT_COMPARE_VALUE_MAX_OVER         (3U)    /* Check for output compare signal parameter */

/*----------------------------------------------------------------------------*/
/*   Define input trigger setting for input capture.                          */
/*----------------------------------------------------------------------------*/
#define CMTW_IN_CAPTURE_TRIGGER_UP              (0U)    /* Input capture trigger is up edge          */
#define CMTW_IN_CAPTURE_TRIGGER_DOWN            (1U)    /* Input capture trigger is down edge        */
#define CMTW_IN_CAPTURE_TRIGGER_UP_DOWN         (2U)    /* Input capture trigger is up and down edge */
#define CMTW_IN_CAPTURE_TRIGGER_MAX_OVER        (3U)    /* Check for input capture signal parameter  */

/*----------------------------------------------------------------------------*/
/*   Define PCLKD divide rate for nosie filter parameter.                     */
/*----------------------------------------------------------------------------*/
#define CMTW_NOISE_FILTER_PCLK_DIV_1            (0U)    /* Noise filter divide rate is PCLKD/1                  */
#define CMTW_NOISE_FILTER_PCLK_DIV_8            (1U)    /* Noise filter divide rate is PCLKD/8                  */
#define CMTW_NOISE_FILTER_PCLK_DIV_32           (2U)    /* Noise filter divide rate is PCLKD/32                 */
#define CMTW_NOISE_FILTER_PCLK_DIV_64           (3U)    /* Noise filter divide rate is PCLKD/64                 */
#define CMTW_NOISE_FILTER_PCLK_DIV_MAX_OVER     (4U)    /* Check for invalid noise filter divide rate parameter */

/*----------------------------------------------------------------------------*/
/*   Define input capture number parameter.                                   */
/*----------------------------------------------------------------------------*/
#define CMTW_INPUT_CAPTURE_NUM_0                (0U)    /* Input capture 0    */
#define CMTW_INPUT_CAPTURE_NUM_1                (1U)    /* Input capture 1    */
#define CMTW_INPUT_CAPTURE_NUM                  (2U)    /* Input capture element number */

/*----------------------------------------------------------------------------*/
/*   Define output compare number parameter.                                  */
/*----------------------------------------------------------------------------*/
#define CMTW_OUTPUT_COMPARE_NUM_0               (0U)    /* Output compare 0   */
#define CMTW_OUTPUT_COMPARE_NUM_1               (1U)    /* Output compare 1   */
#define CMTW_OUTPUT_COMPARE_NUM                 (2U)    /* Output compare element number */

/*----------------------------------------------------------------------------*/
/*   Define status of timmer running.                                         */
/*----------------------------------------------------------------------------*/
#define CMT_STATUS_STOP                         (0U)    /* Timer is stop      */
#define CMT_STATUS_RUNNING                      (1U)    /* TImer is runnning  */

/*******************************************************************************
Typedef definitions
*******************************************************************************/
typedef enum e_cmt_channel
{
    CMT_CHANNEL_0,
    CMT_CHANNEL_1,
    CMT_CHANNEL_2,
    CMT_CHANNEL_3,
    CMT_CHANNEL_4,
    CMT_CHANNEL_5,
    CMT_CHANNEL_6,          /* CMTW */
    CMT_CHANNEL_7,          /* CMTW */
    CMT_CHANNEL_MAX
} cmt_channel_t;

#define CMTW_CHANNEL_0      CMT_CHANNEL_6
#define CMTW_CHANNEL_1      CMT_CHANNEL_7

/*----------------------------------------------------------------------------*/
/*   Define cmt(w) timer count setting structure.                             */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t                pclk_div;
    uint32_t                cnt_size;
    uint32_t                clear_factor;
} cmt_time_cnt_t;
#define cmtw_time_cnt_t     cmt_time_cnt_t

/*----------------------------------------------------------------------------*/
/*   Define cmt(w) compare match setting structure.                           */
/*----------------------------------------------------------------------------*/
typedef struct
{
    int32_t                 mode_enable;
    uint32_t                compare_match_cnt;
    int32_t                 intr_priority;
    void                    (* p_callback)(void);
} cmt_compare_match_t;
#define cmtw_compare_match_t    cmt_compare_match_t

/*----------------------------------------------------------------------------*/
/*   Define cmt(w) output compare setting structure.                          */
/*----------------------------------------------------------------------------*/
typedef struct
{
    int32_t                 mode_enable;
    uint32_t                output_compare_cnt;
    uint32_t                output_signal;
    int32_t                 intr_priority;
    void                    (* p_callback)(void);
} cmtw_output_compare_t;

/*----------------------------------------------------------------------------*/
/*   Define cmt(w) input capture setting structure.                           */
/*----------------------------------------------------------------------------*/
typedef struct
{
    int32_t                 mode_enable;
    uint32_t                trigger;
    int32_t                 filter_enable;
    int32_t                 intr_priority;
    void                    (* p_callback)(uint32_t cnt_value);
} cmtw_input_capture_t;

/*----------------------------------------------------------------------------*/
/*   Define ECM setting structure.                                            */
/*----------------------------------------------------------------------------*/
typedef struct
{
    int32_t                 ecm_enable;
    uint32_t                output_compare_num;
} cmtw_ecm_t;

/*----------------------------------------------------------------------------*/
/*   Define cmt(w) operation mode setting structure.                          */
/*----------------------------------------------------------------------------*/
typedef struct
{
    cmt_compare_match_t     compare_match;
    cmtw_output_compare_t   output_compare[CMTW_OUTPUT_COMPARE_NUM];
    cmtw_input_capture_t    input_capture[CMTW_INPUT_CAPTURE_NUM];
    uint32_t                noise_filter_clk;
} cmt_mode_t;

/*----------------------------------------------------------------------------*/
/*   Define cmt(w) configuration structure.                                   */
/*----------------------------------------------------------------------------*/
typedef struct
{
    cmt_time_cnt_t          time_cnt_param;
    cmt_mode_t              mode_param;
} cmt_cfg_t;

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

void     R_CMT_Init(void);
int32_t  R_CMT_Uninit(void);
int32_t  R_CMT_Open(cmt_channel_t const channel, cmt_cfg_t * const p_cfg);
int32_t  R_CMT_Close(cmt_channel_t const channel);
int32_t  R_CMT_StartPeriodic(cmt_channel_t const channel);
int32_t  R_CMT_StartOneShot(cmt_channel_t const channel);
int32_t  R_CMT_Control(cmt_channel_t const channel, const uint32_t cmd, void * const p_data);
uint32_t R_CMT_GetVersion(void);

#endif /* _R_CMTW_IF_HEADER_ */

/* End of File */

