/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : interface header file for ELC fit module.
* File Name    : r_elc_if.h
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for ELC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ELC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_ELC_IF_HEADER_
#define _R_ELC_IF_HEADER_

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"
#include "r_errno.h"

#include "r_elc_config.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
/*----------------------------------------------------------------------------*/
/*   Define Version Number of API.                                            */
/*----------------------------------------------------------------------------*/
#define ELC_VERSION_MAJOR                   (1U)
#define ELC_VERSION_MINOR                   (0U)

/*----------------------------------------------------------------------------*/
/*   Define ELC command for control.                                          */
/*----------------------------------------------------------------------------*/
#define ELC_CMD_SET_EVENT_MTU               (0U)        /* Command of setting MTU event link                 */
#define ELC_CMD_SET_EVENT_CMT               (1U)        /* Command of setting CMT event link                 */
#define ELC_CMD_SET_EVENT_DSMIF             (2U)        /* Command of setting delta sigma unit event link    */
#define ELC_CMD_SET_EVENT_S12AD             (3U)        /* Command of setting 12bit A/D converter event link */
#define ELC_CMD_SET_EVENT_INTR              (4U)        /* Command of setting ELC interrupt event link       */
#define ELC_CMD_SET_EVENT_OUT_PORT_GROUP    (5U)        /* Command of setting output port group event link   */
#define ELC_CMD_SET_EVENT_IN_PORT_GROUP     (6U)        /* Command of setting input port group event link    */
#define ELC_CMD_SET_EVENT_SINGLE_PORT       (7U)        /* Command of setting single port event link         */
#define ELC_CMD_SET_EVENT_CMTW              (8U)        /* Command of setting CMTW event link                */
#define ELC_CMD_SET_EVENT_TPU               (9U)        /* Command of setting TPU event link                 */
#define ELC_CMD_SET_EVENT_GPT               (10U)       /* Command of setting GPT event link                 */
#define ELC_CMD_SET_PORT_GROUP              (11U)       /* Command of setting port group parameter           */
#define ELC_CMD_SET_SOFTWARE_EVENT          (12U)       /* Command of translate software event               */
#define ELC_CMD_GET_PORT_GROUP_VALUE        (13U)       /* Command of getting input port group value         */

/*----------------------------------------------------------------------------*/
/*   Define ELC event link resouces.                                          */
/*----------------------------------------------------------------------------*/

#define ELC_RESOURCE_MTU0_COMPARE_MATCH_0A  (0x01U)     /* Event link resouce is MTU0 conmpare match 0A  */
#define ELC_RESOURCE_MTU0_COMPARE_MATCH_0B  (0x02U)     /* Event link resouce is MTU0 conmpare match 0B  */
#define ELC_RESOURCE_MTU0_COMPARE_MATCH_0C  (0x03U)     /* Event link resouce is MTU0 conmpare match 0C  */
#define ELC_RESOURCE_MTU0_COMPARE_MATCH_0D  (0x04U)     /* Event link resouce is MTU0 conmpare match 0D  */
#define ELC_RESOURCE_MTU0_COMPARE_MATCH_0E  (0x05U)     /* Event link resouce is MTU0 conmpare match 0E  */
#define ELC_RESOURCE_MTU0_COMPARE_MATCH_0F  (0x06U)     /* Event link resouce is MTU0 conmpare match 0F  */
#define ELC_RESOURCE_MTU0_OVERFLOW          (0x07U)     /* Event link resouce is MTU0 overflow           */
#define ELC_RESOURCE_MTU3_COMPARE_MATCH_3A  (0x10U)     /* Event link resouce is MTU3 conmpare match 3A  */
#define ELC_RESOURCE_MTU3_COMPARE_MATCH_3B  (0x11U)     /* Event link resouce is MTU3 conmpare match 3B  */
#define ELC_RESOURCE_MTU3_COMPARE_MATCH_3C  (0x12U)     /* Event link resouce is MTU3 conmpare match 3C  */
#define ELC_RESOURCE_MTU3_COMPARE_MATCH_3D  (0x13U)     /* Event link resouce is MTU3 conmpare match 3D  */
#define ELC_RESOURCE_MTU3_OVERFLOW          (0x14U)     /* Event link resouce is MTU3 overflow           */
#define ELC_RESOURCE_MTU4_COMPARE_MATCH_4A  (0x15U)     /* Event link resouce is MTU4 conmpare match 4A  */
#define ELC_RESOURCE_MTU4_COMPARE_MATCH_4B  (0x16U)     /* Event link resouce is MTU4 conmpare match 4B  */
#define ELC_RESOURCE_MTU4_COMPARE_MATCH_4C  (0x17U)     /* Event link resouce is MTU4 conmpare match 4C  */
#define ELC_RESOURCE_MTU4_COMPARE_MATCH_4D  (0x18U)     /* Event link resouce is MTU4 conmpare match 4D  */
#define ELC_RESOURCE_MTU4_OVERFLOW          (0x19U)     /* Event link resouce is MTU4 overflow           */
#define ELC_RESOURCE_MTU4_UNDERFLOW         (0x1AU)     /* Event link resouce is MTU4 underflow          */
#define ELC_RESOURCE_CMT1_COMPARE_MATCH_1   (0x1FU)     /* Event link resouce is CMT1 conmpare match 1   */
#define ELC_RESOURCE_ETHER_TIMER_SYNC       (0x22U)     /* Event link resouce is ETHER timer             */
#define ELC_RESOURCE_RIIC0_EVENT            (0x4EU)     /* Event link resouce is RIIC0 error and event   */
#define ELC_RESOURCE_RIIC0_RX_DATA_FULL     (0x4FU)     /* Event link resouce is RIIC0 receive data full */
#define ELC_RESOURCE_RIIC0_TX_DATA_EMPTY    (0x50U)     /* Event link resouce is RIIC0 trans data empty  */
#define ELC_RESOURCE_RIIC0_TX_END           (0x51U)     /* Event link resouce is RIIC0 trans data end    */
#define ELC_RESOURCE_RSPI0_ERROR            (0x52U)     /* Event link resouce is RSPI0 error             */
#define ELC_RESOURCE_RSPI0_IDLE             (0x53U)     /* Event link resouce is RSPI0 idle              */
#define ELC_RESOURCE_RSPI0_RX_DATA_FULL     (0x54U)     /* Event link resouce is RSPI0 receive data full */
#define ELC_RESOURCE_RSPI0_TX_DATA_EMPTY    (0x55U)     /* Event link resouce is RSPI0 trans data empty  */
#define ELC_RESOURCE_RSPI0_TX_END           (0x56U)     /* Event link resouce is RSPI0 trans data end    */
#define ELC_RESOURCE_S12AD0_CONVERT_END     (0x58U)     /* Event link resouce is S12AD0 A/D convert end  */
#define ELC_RESOURCE_INPUT_PORT_GROUP1      (0x63U)     /* Event link resouce is input port group1       */
#define ELC_RESOURCE_INPUT_PORT_GROUP2      (0x64U)     /* Event link resouce is input port group2       */
#define ELC_RESOURCE_SINGLE_PORT0           (0x65U)     /* Event link resouce is single port0            */
#define ELC_RESOURCE_SINGLE_PORT1           (0x66U)     /* Event link resouce is single port1            */
#define ELC_RESOURCE_SINGLE_PORT2           (0x67U)     /* Event link resouce is single port2            */
#define ELC_RESOURCE_SINGLE_PORT3           (0x68U)     /* Event link resouce is single port3            */
#define ELC_RESOURCE_ELC_SOFT_EVENT         (0x69U)     /* Event link resouce is sotware event           */
#define ELC_RESOURCE_DOC_DATA_CALCULATE     (0x6AU)     /* Event link resouce is DOC data calculate end  */
#define ELC_RESOURCE_S12AD1_CONVERT_END     (0x6CU)     /* Event link resouce is S12AD1 A/D convert end  */
#define ELC_RESOURCE_CMTW0_COMPARE_MATCH    (0x7EU)     /* Event link resouce is CMTW conmpare match     */
#define ELC_RESOURCE_GPT0_COMPARE_MATCH_A   (0x80U)     /* Event link resouce is GPT0 conmpare match A   */
#define ELC_RESOURCE_GPT0_COMPARE_MATCH_B   (0x81U)     /* Event link resouce is GPT0 conmpare match B   */
#define ELC_RESOURCE_GPT0_COMPARE_MATCH_C   (0x82U)     /* Event link resouce is GPT0 conmpare match C   */
#define ELC_RESOURCE_GPT0_COMPARE_MATCH_D   (0x83U)     /* Event link resouce is GPT0 conmpare match D   */
#define ELC_RESOURCE_GPT0_OVERFLOW          (0x86U)     /* Event link resouce is GPT0 overflow           */
#define ELC_RESOURCE_GPT0_UNDERFLOW         (0x87U)     /* Event link resouce is GPT0 underflow          */
#define ELC_RESOURCE_GPT1_COMPARE_MATCH_A   (0x88U)     /* Event link resouce is GPT1 conmpare match A   */
#define ELC_RESOURCE_GPT1_COMPARE_MATCH_B   (0x89U)     /* Event link resouce is GPT1 conmpare match B   */
#define ELC_RESOURCE_GPT1_COMPARE_MATCH_C   (0x8AU)     /* Event link resouce is GPT1 conmpare match C   */
#define ELC_RESOURCE_GPT1_COMPARE_MATCH_D   (0x8BU)     /* Event link resouce is GPT1 conmpare match D   */
#define ELC_RESOURCE_GPT1_OVERFLOW          (0x8EU)     /* Event link resouce is GPT1 overflow           */
#define ELC_RESOURCE_GPT1_UNDERFLOW         (0x8FU)     /* Event link resouce is GPT1 underflow          */
#define ELC_RESOURCE_GPT2_COMPARE_MATCH_A   (0x90U)     /* Event link resouce is GPT2 conmpare match A   */
#define ELC_RESOURCE_GPT2_COMPARE_MATCH_B   (0x91U)     /* Event link resouce is GPT2 conmpare match B   */
#define ELC_RESOURCE_GPT2_COMPARE_MATCH_C   (0x92U)     /* Event link resouce is GPT2 conmpare match C   */
#define ELC_RESOURCE_GPT2_COMPARE_MATCH_D   (0x93U)     /* Event link resouce is GPT2 conmpare match D   */
#define ELC_RESOURCE_GPT2_OVERFLOW          (0x96U)     /* Event link resouce is GPT2 overflow           */
#define ELC_RESOURCE_GPT2_UNDERFLOW         (0x97U)     /* Event link resouce is GPT2 underflow          */
#define ELC_RESOURCE_GPT3_COMPARE_MATCH_A   (0x98U)     /* Event link resouce is GPT3 conmpare match A   */
#define ELC_RESOURCE_GPT3_COMPARE_MATCH_B   (0x99U)     /* Event link resouce is GPT3 conmpare match B   */
#define ELC_RESOURCE_GPT3_COMPARE_MATCH_C   (0x9AU)     /* Event link resouce is GPT3 conmpare match C   */
#define ELC_RESOURCE_GPT3_COMPARE_MATCH_D   (0x9BU)     /* Event link resouce is GPT3 conmpare match D   */
#define ELC_RESOURCE_GPT3_OVERFLOW          (0x9EU)     /* Event link resouce is GPT3 overflow           */
#define ELC_RESOURCE_GPT3_UNDERFLOW         (0x9FU)     /* Event link resouce is GPT3 underflow          */
#define ELC_RESOURCE_MTU6_COMPARE_MATCH_6A  (0xA0U)     /* Event link resouce is MTU6 conmpare match 6A  */
#define ELC_RESOURCE_MTU6_COMPARE_MATCH_6B  (0xA1U)     /* Event link resouce is MTU6 conmpare match 6B  */
#define ELC_RESOURCE_MTU6_COMPARE_MATCH_6C  (0xA2U)     /* Event link resouce is MTU6 conmpare match 6C  */
#define ELC_RESOURCE_MTU6_COMPARE_MATCH_6D  (0xA3U)     /* Event link resouce is MTU6 conmpare match 6D  */
#define ELC_RESOURCE_MTU6_OVERFLOW          (0xA4U)     /* Event link resouce is MTU6 overflow           */
#define ELC_RESOURCE_MTU7_COMPARE_MATCH_7A  (0xA5U)     /* Event link resouce is MTU7 conmpare match 7A  */
#define ELC_RESOURCE_MTU7_COMPARE_MATCH_7B  (0xA6U)     /* Event link resouce is MTU7 conmpare match 7B  */
#define ELC_RESOURCE_MTU7_COMPARE_MATCH_7C  (0xA7U)     /* Event link resouce is MTU7 conmpare match 7C  */
#define ELC_RESOURCE_MTU7_COMPARE_MATCH_7D  (0xA8U)     /* Event link resouce is MTU7 conmpare match 7D  */
#define ELC_RESOURCE_MTU7_OVERFLOW          (0xA9U)     /* Event link resouce is MTU7 overflow           */
#define ELC_RESOURCE_MTU7_UNDERFLOW         (0xAAU)     /* Event link resouce is MTU7 underflow          */
#define ELC_RESOURCE_TPU0_COMPARE_MATCH_A   (0xACU)     /* Event link resouce is TPU0 conmpare match A   */
#define ELC_RESOURCE_TPU0_COMPARE_MATCH_B   (0xADU)     /* Event link resouce is TPU0 conmpare match B   */
#define ELC_RESOURCE_TPU0_COMPARE_MATCH_C   (0xAEU)     /* Event link resouce is TPU0 conmpare match C   */
#define ELC_RESOURCE_TPU0_COMPARE_MATCH_D   (0xAFU)     /* Event link resouce is TPU0 conmpare match D   */
#define ELC_RESOURCE_TPU0_OVERFLOW          (0xB0U)     /* Event link resouce is TPU0 overflow           */
#define ELC_RESOURCE_TPU1_COMPARE_MATCH_A   (0xB1U)     /* Event link resouce is TPU1 conmpare match A   */
#define ELC_RESOURCE_TPU1_COMPARE_MATCH_B   (0xB2U)     /* Event link resouce is TPU1 conmpare match B   */
#define ELC_RESOURCE_TPU1_OVERFLOW          (0xB3U)     /* Event link resouce is TPU1 overflow           */
#define ELC_RESOURCE_TPU1_UNDERFLOW         (0xB4U)     /* Event link resouce is TPU1 underflow          */
#define ELC_RESOURCE_TPU2_COMPARE_MATCH_A   (0xB5U)     /* Event link resouce is TPU2 conmpare match A   */
#define ELC_RESOURCE_TPU2_COMPARE_MATCH_B   (0xB6U)     /* Event link resouce is TPU2 conmpare match B   */
#define ELC_RESOURCE_TPU2_OVERFLOW          (0xB7U)     /* Event link resouce is TPU2 overflow           */
#define ELC_RESOURCE_TPU2_UNDERFLOW         (0xB8U)     /* Event link resouce is TPU2 underflow          */
#define ELC_RESOURCE_TPU3_COMPARE_MATCH_A   (0xB9U)     /* Event link resouce is TPU0 conmpare match A   */
#define ELC_RESOURCE_TPU3_COMPARE_MATCH_B   (0xBAU)     /* Event link resouce is TPU0 conmpare match B   */
#define ELC_RESOURCE_TPU3_COMPARE_MATCH_C   (0xBBU)     /* Event link resouce is TPU0 conmpare match C   */
#define ELC_RESOURCE_TPU3_COMPARE_MATCH_D   (0xBCU)     /* Event link resouce is TPU0 conmpare match D   */
#define ELC_RESOURCE_TPU3_OVERFLOW          (0xBDU)     /* Event link resouce is TPU0 overflow           */

/*----------------------------------------------------------------------------*/
/*   Define MTU channel for event link.                                       */
/*----------------------------------------------------------------------------*/

#define ELC_MTU_CH_0                        (0U)        /* Select MTU0                      */
#define ELC_MTU_CH_3                        (1U)        /* Select MTU3                      */
#define ELC_MTU_CH_4                        (2U)        /* Select MTU4                      */
#define ELC_MTU_CH_NUM                      (3U)        /* Check for invalid channel number */

/*----------------------------------------------------------------------------*/
/*   Define MTU action of event link.                                         */
/*----------------------------------------------------------------------------*/

#define ELC_MTU_COUNT_START                 (0U)        /* Action is count start            */
#define ELC_MTU_COUNT_RESTART               (1U)        /* Action is count restart          */
#define ELC_MTU_INPUT_CAPTURE               (2U)        /* Action is input capture          */
#define ELC_MTU_MAX_OVER                    (3U)        /* Check for invalid action setting */

/*----------------------------------------------------------------------------*/
/*   Define CMT action of event link.                                         */
/*----------------------------------------------------------------------------*/

#define ELC_CMT_COUNT_START                 (0U)        /* Action is count start            */
#define ELC_CMT_COUNT_RESTART               (1U)        /* Action is count restart          */
#define ELC_CMT_COUNT_INCREMENT             (2U)        /* Action is count increment        */
#define ELC_CMT_MAX_OVER                    (3U)        /* Check for invalid action setting */

/*----------------------------------------------------------------------------*/
/*   Define delta sigma unit channel and trigger for event link.              */
/*----------------------------------------------------------------------------*/

#define ELC_DSMIF0_TRIGGER_0                (0U)        /* Select delta sigma unit0, trigger0 */
#define ELC_DSMIF0_TRIGGER_1                (1U)        /* Select delta sigma unit0, trigger1 */
#define ELC_DSMIF1_TRIGGER_0                (2U)        /* Select delta sigma unit1, trigger0 */
#define ELC_DSMIF1_TRIGGER_1                (3U)        /* Select delta sigma unit1, trigger1 */
#define ELC_DSMIF_TRIGGER_NUM               (4U)        /* Check for invalid channel number   */

/*----------------------------------------------------------------------------*/
/*   Define 12bit A/D converter channel for event link.                       */
/*----------------------------------------------------------------------------*/

#define ELC_S12AD_CH_0                      (0U)        /* Select 12bit A/D converter0      */
#define ELC_S12AD_CH_1                      (1U)        /* Select 12bit A/D converter1      */
#define ELC_S12AD_CH_NUM                    (2U)        /* Check for invalid channel number */

/*----------------------------------------------------------------------------*/
/*   Define ELC interrupt number for event link.                              */
/*----------------------------------------------------------------------------*/

#define ELC_INTR_NUM_1                      (0U)        /* Select ELC interrupt1            */
#define ELC_INTR_NUM_2                      (1U)        /* Select ELC interrupt2            */
#define ELC_INTR_NUM                        (2U)        /* Check for invalid channel number */

/*----------------------------------------------------------------------------*/
/*   Define port group number.                                                */
/*----------------------------------------------------------------------------*/

#define ELC_PORT_GROUP_NUM_0                (0U)        /* Select ELC port group0           */
#define ELC_PORT_GROUP_NUM_1                (1U)        /* Select ELC port group1           */
#define ELC_PORT_GROUP_NUM                  (2U)        /* Check for invalid channel number */

/*----------------------------------------------------------------------------*/
/*   Define port group action for event link.                                 */
/*----------------------------------------------------------------------------*/
#define ELC_OUT_GROUP_OUTPUT_0              (0U)        /* Output signal value of 0                */
#define ELC_OUT_GROUP_OUTPUT_1              (1U)        /* Output signal value of 1                */
#define ELC_OUT_GROUP_TOGGLE                (2U)        /* Output toggle signal                    */
#define ELC_OUT_GROUP_BUFFER                (3U)        /* Output signal on buffer value           */
#define ELC_OUT_GROUP_ROTATE                (4U)        /* Output signal on buffer value as rotate */
#define ELC_OUT_GROUP_MAX_OVER              (5U)        /* Check for invalid action setting        */

/*----------------------------------------------------------------------------*/
/*   Define single port number.                                               */
/*----------------------------------------------------------------------------*/

#define ELC_SINGLE_PORT_NUM_0               (0U)        /* Select single port0              */
#define ELC_SINGLE_PORT_NUM_1               (1U)        /* Select single port1              */
#define ELC_SINGLE_PORT_NUM_2               (2U)        /* Select single port2              */
#define ELC_SINGLE_PORT_NUM_3               (3U)        /* Select single port3              */
#define ELC_SINGLE_PORT_NUM                 (4U)        /* Check for invalid channel number */

/*----------------------------------------------------------------------------*/
/*   Define single port event direction.                                      */
/*----------------------------------------------------------------------------*/

#define ELC_SINGLE_EVENT_INPUT              (0U)        /* Output signal value of 0 */
#define ELC_SINGLE_EVENT_OUTPUT             (1U)        /* Output signal value of 1 */
#define ELC_SINGLE_EVENT_MAX_OVER           (2U)        /* Output toggle signal     */

/*----------------------------------------------------------------------------*/
/*   Define single port action for event link.                                */
/*----------------------------------------------------------------------------*/

#define ELC_SINGLE_PORT_OUTPUT_0            (0U)        /* Output signal value of 0         */
#define ELC_SINGLE_PORT_OUTPUT_1            (1U)        /* Output signal value of 1         */
#define ELC_SINGLE_PORT_TOGGLE              (2U)        /* Output toggle signal             */
#define ELC_SINGLE_PORT_ACTION_MAX_OVER     (3U)        /* Check for invalid action setting */

/*----------------------------------------------------------------------------*/
/*   Define CMTW action of event link.                                        */
/*----------------------------------------------------------------------------*/

#define ELC_CMTW_COUNT_START                (0U)        /* Action is count start            */
#define ELC_CMTW_COUNT_RESTART              (1U)        /* Action is count restart          */
#define ELC_CMTW_COUNT_INCREMENT            (2U)        /* Action is count increment        */
#define ELC_CMTW_COUNT_MAX_OVER             (3U)        /* Check for invalid action setting */

/*----------------------------------------------------------------------------*/
/*   Define TPU channel for event link.                                       */
/*----------------------------------------------------------------------------*/

#define ELC_TPU_CH_0                        (0U)        /* Select TPU0                      */
#define ELC_TPU_CH_1                        (1U)        /* Select TPU1                      */
#define ELC_TPU_CH_2                        (2U)        /* Select TPU2                      */
#define ELC_TPU_CH_3                        (3U)        /* Select TPU3                      */
#define ELC_TPU_CH_NUM                      (4U)        /* Check for invalid channel number */

/*----------------------------------------------------------------------------*/
/*   Define TPU action of event link.                                         */
/*----------------------------------------------------------------------------*/

#define ELC_TPU_COUNT_START                 (0U)        /* Action is count start            */
#define ELC_TPU_COUNT_RESTART               (1U)        /* Action is count restart          */
#define ELC_TPU_INPUT_CAPTURE               (2U)        /* Action is input capture          */
#define ELC_TPU_MAX_OVER                    (3U)        /* Check for invalid action setting */

/*----------------------------------------------------------------------------*/
/*   Define GPT channel for event link.                                       */
/*----------------------------------------------------------------------------*/

#define ELC_GPT_CH_0                        (0U)        /* Select GPT0                      */
#define ELC_GPT_CH_1                        (1U)        /* Select GPT1                      */
#define ELC_GPT_CH_2                        (2U)        /* Select GPT2                      */
#define ELC_GPT_CH_3                        (3U)        /* Select GPT3                      */
#define ELC_GPT_CH_NUM                      (4U)        /* Check for invalid channel number */

/*----------------------------------------------------------------------------*/
/*   Define GPT action of event link.                                         */
/*----------------------------------------------------------------------------*/

#define ELC_GPT_COUNT_START                 (0U)        /* Action is count start            */
#define ELC_GPT_COUNT_RESTART               (1U)        /* Action is count restart          */
#define ELC_GPT_COUNT_STOP                  (2U)        /* Action is count stop             */
#define ELC_GPT_COUNT_CAPTURE               (3U)        /* Action is input capture          */
#define ELC_GPT_MAX_OVER                    (4U)        /* Check for invalid action setting */

/*----------------------------------------------------------------------------*/
/*   Define port symol.                                                       */
/*----------------------------------------------------------------------------*/

#define ELC_PORT_B                          (0U)        /* Select PORTB                  */
#define ELC_PORT_E                          (1U)        /* Select PORTE                  */
#define ELC_PORT_NUM                        (2U)        /* Check for invalid port symbol */

/*----------------------------------------------------------------------------*/
/*   Define port group trigger setting.                                       */
/*----------------------------------------------------------------------------*/

#define ELC_PORT_GROUP_TRIGGER_UP           (0U)        /* Input trigger is up edge          */
#define ELC_PORT_GROUP_TRIGGER_DOWN         (1U)        /* Input trigger is down edge        */
#define ELC_PORT_GROUP_TRIGGER_UP_DOWN      (2U)        /* Input trigger is up and down edge */
#define ELC_PORT_GROUP_MAX_OVER             (3U)        /* Check for invalid trigger         */

/*----------------------------------------------------------------------------*/
/*   Define single port trigger setting.                                      */
/*----------------------------------------------------------------------------*/

#define ELC_SINGLE_PORT_TRIGGER_UP          (0U)        /* Input trigger is up edge          */
#define ELC_SINGLE_PORT_TRIGGER_DOWN        (1U)        /* Input trigger is down edge        */
#define ELC_SINGLE_PORT_TRIGGER_UP_DOWN     (2U)        /* Input trigger is up and down edge */
#define ELC_SINGLE_PORT_TRIGGER_MAX_OVER    (3U)        /* Check for invalid trigger         */

/*******************************************************************************
Typedef definitions
*******************************************************************************/
/*----------------------------------------------------------------------------*/
/*   Define MTU parameter for event link.                                     */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_mtu_ch;
    int32_t         event_link_enable;
    uint32_t        resource;
    uint32_t        action;
}elc_cmd_mtu_t;

/*----------------------------------------------------------------------------*/
/*   Define CMT parameter for event link.                                     */
/*----------------------------------------------------------------------------*/
typedef struct
{
    int32_t         event_link_enable;
    uint32_t        resource;
    uint32_t        action;
}elc_cmd_cmt_t;

/*----------------------------------------------------------------------------*/
/*   Define delta sigma unit parameter for event link.                        */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_dsmif_ch;
    int32_t         event_link_enable;
    uint32_t        resource;
}elc_cmd_dsmif_t;

/*----------------------------------------------------------------------------*/
/*   Define 12bit A/D converter parameter for event link.                     */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_s12ad_ch;
    int32_t         event_link_enable;
    uint32_t        resource;
}elc_cmd_s12ad_t;

/*----------------------------------------------------------------------------*/
/*   Define ELC interrupt for event link.                                     */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_intr_num;
    int32_t         event_link_enable;
    uint32_t        resource;
    int32_t         intr_priority;
    void            (* p_callback)(void);
}elc_cmd_intr_t;

/*----------------------------------------------------------------------------*/
/*   Define output port group parameter for event link.                       */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_out_port_group_num;
    int32_t         event_link_enable;
    uint32_t        resource;
    uint32_t        action;
    uint8_t         init_value;
}elc_cmd_out_port_group_t;

/*----------------------------------------------------------------------------*/
/*   Define input port group parameter for event link.                        */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_in_port_group_num;
    int32_t         event_link_enable;
    uint8_t         resource;
    int32_t         overwrite_enable;
}elc_cmd_in_port_group_t;

/*----------------------------------------------------------------------------*/
/*   Define input port group parameter for event link.                        */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_single_port_num;
    uint32_t        port_symbol;
    uint32_t        port_num;
    int32_t         event_link_enable;
    uint32_t        event_direction;
    uint32_t        resource;
    uint32_t        output_action;
    uint32_t        input_trigger;
}elc_cmd_single_port_t;

/*----------------------------------------------------------------------------*/
/*   Define CMTW parameter for event link.                                    */
/*----------------------------------------------------------------------------*/
typedef struct
{
    int32_t         event_link_enable;
    uint32_t        resource;
    uint32_t        action;
}elc_cmd_cmtw_t;

/*----------------------------------------------------------------------------*/
/*   Define TPU parameter for event link.                                     */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_tpu_ch;
    int32_t         event_link_enable;
    uint32_t        resource;
    uint32_t        action;
}elc_cmd_tpu_t;

/*----------------------------------------------------------------------------*/
/*   Define GPT parameter for event link.                                     */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_gpt_ch;
    int32_t         event_link_enable;
    uint32_t        resource;
    uint32_t        action;
}elc_cmd_gpt_t;

/*----------------------------------------------------------------------------*/
/*   Define port group parameter for port group setting.                      */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        port_group_num;
    uint8_t         port_group_bit;
    uint32_t        trigger;
}elc_cmd_port_group_t;

/*----------------------------------------------------------------------------*/
/*   Define port group parameter for port group value get.                    */
/*----------------------------------------------------------------------------*/
typedef struct
{
    uint32_t        elc_port_group_num;
    uint8_t         port_value;
}elc_get_port_value_t;

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
/* ---- ELC Driver API functions ---- */
void     R_ELC_Init(void);
int32_t  R_ELC_Uninit(void);
int32_t  R_ELC_Open(void);
int32_t  R_ELC_Close(void);
int32_t  R_ELC_LinkStart(void);
int32_t  R_ELC_LinkStop(void);
int32_t  R_ELC_Control(uint32_t const cmd, void * const p_data);
uint32_t R_ELC_GetVersion(void);

#endif /* _R_ELC_IF_HEADER_ */

/* End of File */

