/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : configures file for MTU3a fit module.
* File Name    : r_mtu3_config.h
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for MTU3a driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using MTU3a on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_MTU3_CONFIG_HEADER_
#define _R_MTU3_CONFIG_HEADER_

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "vic.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/* Enable the MTU channels to use in this build. (0) = not used. (1) = used. */
#define MTU_CFG_USE_CH0         (1)
#define MTU_CFG_USE_CH1         (1)
#define MTU_CFG_USE_CH2         (1)
#define MTU_CFG_USE_CH3         (1)
#define MTU_CFG_USE_CH4         (1)
#define MTU_CFG_USE_CH5         (0)
#define MTU_CFG_USE_CH6         (1)
#define MTU_CFG_USE_CH7         (1)
#define MTU_CFG_USE_CH8         (1)

/* Code for unused MTU functions can be excluded from the build to reduce size. (0) = not used. (1) = used. */
#define MTU_CFG_USE_TIMER       (1)
#define MTU_CFG_USE_CAPTURE     (1)
#define MTU_CFG_USE_PWM         (1)

/* Set interrupt priority levels for each channel present.
 * Priority is shared by all interrupt sources in a channel.
 * Values must be in the range 0 (interrupt disabled) to 15 (highest)*/
#define MTU_IR_PRIORITY_CHAN0   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN1   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN2   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN3   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN4   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN5   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN6   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN7   (VIC_PRIORITY_15)
#define MTU_IR_PRIORITY_CHAN8   (VIC_PRIORITY_15)


/* Set the MTU input capture noise filter clock for each channels to used for input capture.
 * See r_mtu_timer_rx_if.h for allowable selections. The following settings are default and
 * should be changed as needed. The available settings are defined in "r_mtu_timer_rx_if.h" */
#define MTU_CFG_FILT_CHAN0      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN1      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN2      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN3      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN4      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN5      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN6      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN7      (MTU_FILTER_PCLK_DIV_1)
#define MTU_CFG_FILT_CHAN8      (MTU_FILTER_PCLK_DIV_1)

/* SPECIFY WHETHER TO INCLUDE CODE FOR API PARAMETER CHECKING
     0 : Disable the checking of API parameters,
     1 : Enable the checking of API parameters                */
#define MTU3_CFG_PARAM_CHECKING_ENABLE  (1)  // Enables


#endif

/* End of File */
