/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : adc driver functions for adc fit module.
* File Name    : r_adc.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for ADC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ADC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_ADC_HEADER_
#define _R_ADC_HEADER_

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"
#include "r_adc_if.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define ADC_INVALID_CH_MASK     (0xFFFF0000)    // all channels valid
#define BSP_MCU_IPL_MAX         VIC_PRIORITY_15 // This is BSP Macro. Want to remove BSP when ready.

/*******************************************************************************
Typedef definitions
*******************************************************************************/
/* ADCSR register ADCS field */
typedef enum e_adc_adcs
{
    ADC_ADCS_SINGLE_SCAN = 0,
    ADC_ADCS_GROUP_SCAN  = 1,
    ADC_ADCS_CONT_SCAN   = 2,
    ADC_ADCS_MAX
} adc_adcs_t;

/* ADC API error codes */
typedef enum e_adc_err
{
    ADC_SUCCESS           = 0,
    ADC_ERR_AD_NOT_CLOSED,          // peripheral still running in another mode
    ADC_ERR_MISSING_PTR,            // missing required pointer argument
    ADC_ERR_INVALID_ARG,            // argument is not valid for parameter
    ADC_ERR_ILLEGAL_ARG,            // argument is illegal for mode
    ADC_ERR_SCAN_NOT_DONE           // default, Group A, or Group B scan not done
} adc_err_t;

typedef enum e_adc_state
{
    ADC_UNINIT = 0,
    ADC_INIT   = 1,
    ADC_CLOSE  = 1,
    ADC_OPEN,
    ADC_RUN,
    ADC_PAUSE,
    ADC_END
} adc_state_t;

typedef struct st_adc_control
{
    adc_state_t     state;                          /* ADC state                 */
    adc_mode_t      mode;                           /* ADC mode                  */
    bool_t          complete;                       /* ADC complete state        */
    bool_t          complete_groupb;                /* ADC complete groupb state */
    osSemaphoreId   sem_state;                      /* I/F semaphore handle      */
    void            (*p_callback)(int32_t event);    /* interrupt callback        */
} adc_control_t;

/*******************************************************************************
Exported global variables
*******************************************************************************/
extern adc_control_t adc_control;

/*******************************************************************************
Exported global functions (to be accessed by other files)
*******************************************************************************/
adc_err_t ADC_Open(adc_mode_t const mode, adc_cfg_t * const p_cfg, void (* const p_callback)(void *p_args));
adc_err_t ADC_Control(adc_cmd_t const cmd, void * const p_args);
adc_err_t ADC_Read(adc_reg_t const reg_id, uint16_t * const p_data);
adc_err_t ADC_ReadAll(adc_data_t * const p_all_data);
void      ADC_Close(void);

#endif /* _R_ADC_HEADER_ */

/* End of File */

