/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : cmt(w) driver functions for cmt(W) fit module.
* File Name    : r_cmt.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : source file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMT on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
/* Access to peripherals and board defines. */
#include "cmsis_os.h"
#include "r_errno.h"
#include "iodefine.h"
#include "r_system.h"
#include "r_reset.h"
#include "Renesas_RZ_T1.h"

#include "vic.h"

#include "r_cmt.h"
#include "r_cmt_userdef.h"


/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*******************************************************************************
Imported global variables and functions (from other files)
*******************************************************************************/

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/

/*******************************************************************************
Private global variables and functions
*******************************************************************************/
cmt_err_t cmt_check_time_cnt_param(cmt_time_cnt_t * const p_time_cnt_param);

cmt_info_ch_t gb_cmt_info[CMT_CH_NUM] =
    {
        {
            CMT_CLOSE,
            CMT_PERIODIC,
            (uint16_t)CMT_CNT_CLEAR_COMPARE_MATCH,
            false,
            NULL,
            &CMT0,
            &CMT.CMSTR0.WORD,
            CMT_MASK_CMSTR0_STR0
        },
        {
            CMT_CLOSE,
            CMT_PERIODIC,
            (uint16_t)CMT_CNT_CLEAR_COMPARE_MATCH,
            false,
            NULL,
            &CMT1,
            &CMT.CMSTR0.WORD,
            CMT_MASK_CMSTR0_STR1
        },
        {
            CMT_CLOSE,
            CMT_PERIODIC,
            (uint16_t)CMT_CNT_CLEAR_COMPARE_MATCH,
            false,
            NULL,
            &CMT2,
            &CMT.CMSTR1.WORD,
            CMT_MASK_CMSTR1_STR2
        },
        {
            CMT_CLOSE,
            CMT_PERIODIC,
            (uint16_t)CMT_CNT_CLEAR_COMPARE_MATCH,
            false,
            NULL,
            &CMT3,
            &CMT.CMSTR1.WORD,
            CMT_MASK_CMSTR1_STR3
        },
        {
            CMT_CLOSE,
            CMT_PERIODIC,
            (uint16_t)CMT_CNT_CLEAR_COMPARE_MATCH,
            false,
            NULL,
            &CMT4,
            &CMT.CMSTR2.WORD,
            CMT_MASK_CMSTR2_STR4
        },
        {
            CMT_CLOSE,
            CMT_PERIODIC,
            (uint16_t)CMT_CNT_CLEAR_COMPARE_MATCH,
            false,
            NULL,
            &CMT5,
            &CMT.CMSTR2.WORD,
            CMT_MASK_CMSTR2_STR5
        }
    };

/* CMT driver information */
cmt_control_t cmt_control = {
    CMT_UNINIT,
    0,
    &gb_cmt_info[0],                    /* CMT 0  */
    &gb_cmt_info[1],                    /* CMT 1  */
    &gb_cmt_info[2],                    /* CMT 2  */
    &gb_cmt_info[3],                    /* CMT 3  */
    &gb_cmt_info[4],                    /* CMT 4  */
    &gb_cmt_info[5],                    /* CMT 5  */
    (cmt_info_ch_t*)&gb_cmtw_info[0],   /* CMTW 0 */
    (cmt_info_ch_t*)&gb_cmtw_info[1]    /* CMTW 1 */
};

/*******************************************************************************
* Function Name: CMT_Open
* Description  :
* Arguments    : uint32_t channel           : 0 ~ 3
*              : uint16_t compare_match_cnt : compare match count
*              : void * callback            : callback function
* Return Value : cmt_err_t      : CMT_SECCESS
*                               : CMT_ERR_INVALID_CH
*                               : CMT_ERR_INVALID_ARG
*******************************************************************************/
cmt_err_t CMT_Open(cmt_channel_t const channel, cmt_cfg_t * const p_cfg)
{
    cmt_err_t   retval = CMT_SUCCESS;

    volatile uint32_t dummy32t;
    volatile uint16_t dummy16t;

    switch (channel)
    {
        case CMT_CH_0:
        case CMT_CH_1:
        case CMT_CH_2:
        case CMT_CH_3:
        case CMT_CH_4:
        case CMT_CH_5:

        break;

        default:

            retval = CMT_ERR_INVALID_CH;

        break;
    }

    if (retval == CMT_SUCCESS)
    {
        retval = cmt_check_time_cnt_param(&p_cfg->time_cnt_param);
    }

    if (retval == CMT_SUCCESS)
    {
        if (CMT_CLOSE != gb_cmt_info[channel].state)
        {
            retval = CMT_ERR_NOT_CLOSED;
        }
    }

    if (retval == CMT_SUCCESS)
    {
        gb_cmt_info[channel].state = CMT_OPEN;

        R_RST_WriteEnable();
        switch (channel)
        {
            case CMT_CH_0:
            case CMT_CH_1:

                MSTP(CMT0) = (uint32_t)0U;
                dummy32t = SYSTEM.MSTPCRA.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */

            break;

            case CMT_CH_2:
            case CMT_CH_3:

                MSTP(CMT1) = (uint32_t)0U;
                dummy32t = SYSTEM.MSTPCRA.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */

            break;

            case CMT_CH_4:
            case CMT_CH_5:

                MSTP(CMT2) = (uint32_t)0U;
                dummy32t = SYSTEM.MSTPCRA.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */

            break;

            default:

                /* DO NOTHING */

            break;
        }
        R_RST_WriteDisable();
        switch (channel)
        {
            case CMT_CH_0:
            case CMT_CH_1:

                dummy16t = CMT0.CMCR.WORD;      /* Workaround for lack of release procedure from the module-stop state(2/2) */

            break;

            case CMT_CH_2:
            case CMT_CH_3:

                dummy16t = CMT2.CMCR.WORD;      /* Workaround for lack of release procedure from the module-stop state(2/2) */

            break;

            case CMT_CH_4:
            case CMT_CH_5:

                dummy16t = CMT4.CMCR.WORD;      /* Workaround for lack of release procedure from the module-stop state(2/2) */

            break;

            default:

                /* DO NOTHING */

            break;
        }

        userdef_cmt_open(channel, p_cfg);

    }

    return retval;
}

/*******************************************************************************
* Function Name: CMT_Close
* Description  : This function does close CMT driver.
*                Module stop control is set to ON.
* Arguments    : channel-
*                    CMT channel
* Return Value : cmt_err_t      : CMT_SECCESS
*                               : CMT_ERR_INVALID_CH
*                               : CMT_ERR_INVALID_ARG
*******************************************************************************/
cmt_err_t CMT_Close(uint32_t const channel)
{
    cmt_err_t retval = CMT_SUCCESS;

    /* check channel */
    if (CMT_CH_NUM <= channel)
    {
        retval = CMT_ERR_INVALID_CH;
    }
    else
    {
        /* uninit CMTW driver */
        gb_cmt_info[channel].state = CMT_CLOSE;
        cmtw_count_stop_hw(channel);

        R_RST_WriteEnable();
        switch (channel)
        {
            case CMT_CH_0:
            case CMT_CH_1:

                if ((gb_cmt_info[CMT_CH_0].state == CMT_CLOSE)
                    && (gb_cmt_info[CMT_CH_1].state == CMT_CLOSE))
                {
                    MSTP(CMT0) = (uint32_t)1U;
                }

            break;

            case CMT_CH_2:
            case CMT_CH_3:

                if ((gb_cmt_info[CMT_CH_2].state == CMT_CLOSE)
                    && (gb_cmt_info[CMT_CH_3].state == CMT_CLOSE))
                {
                    MSTP(CMT1) = (uint32_t)1U;
                }

            break;

            case CMT_CH_4:
            case CMT_CH_5:

                if ((gb_cmt_info[CMT_CH_4].state == CMT_CLOSE)
                    && (gb_cmt_info[CMT_CH_5].state == CMT_CLOSE))
                {
                    MSTP(CMT2) = (uint32_t)1U;
                }

            break;

            default:

                /* DO NOTHING */

            break;
        }
        R_RST_WriteDisable();
    }

    return retval;
}
/*******************************************************************************
End of function CMT_Close
*******************************************************************************/

/*******************************************************************************
* Function Name: CMT_StartPeriodic
* Description  : This function start make start timer with periodic operation.
* Arguments    : channel-
*                    CMT channel
* Return Value : int32          : CMT_SECCESS
*                               : CMT_ERR_INVALID_CH
*                               : CMT_ERR_NOT_OPENED
*                               : CMT_ERR_TIMER_RUNNING
*******************************************************************************/
cmt_err_t CMT_StartPeriodic(uint32_t const channel)
{
    cmt_err_t   retval = CMT_SUCCESS;

    switch (channel)
    {
        case CMT_CH_0:
        case CMT_CH_1:
        case CMT_CH_2:
        case CMT_CH_3:
        case CMT_CH_4:
        case CMT_CH_5:

        break;

        default:

            retval = CMT_ERR_INVALID_CH;

        break;
    }

    if (retval == CMT_SUCCESS)
    {
        if (CMT_OPEN > gb_cmt_info[channel].state)
        {
            retval = CMT_ERR_NOT_OPENED;
        }
        else if (CMT_RUN == gb_cmt_info[channel].state)
        {
            retval = CMT_ERR_TIMER_RUNNING;
        }
    }

    if (retval == CMT_SUCCESS)
    {
        /* call count start function with CMTW periodic operation */
        gb_cmt_info[channel].state = CMT_RUN;
        userdef_cmt_start(channel, CMT_PERIODIC);
    }

    return retval;
}
/*******************************************************************************
End of function CMT_StartPeriodic
*******************************************************************************/

/*******************************************************************************
* Function Name: CMT_StartOneShot
* Description  : This function start make start timer with one shot operation.
* Arguments    : channel-
*                    CMT channel
* Return Value : int32          : CMT_SECCESS
*                               : CMT_ERR_INVALID_CH
*                               : CMT_ERR_NOT_OPENED
*                               : CMT_ERR_TIMER_RUNNING
*******************************************************************************/
cmt_err_t CMT_StartOneShot(uint32_t const channel)
{
    cmt_err_t   retval = CMT_SUCCESS;

    switch (channel)
    {
        case CMT_CH_0:
        case CMT_CH_1:
        case CMT_CH_2:
        case CMT_CH_3:
        case CMT_CH_4:
        case CMT_CH_5:

        break;

        default:

            retval = CMT_ERR_INVALID_CH;

        break;
    }

    if (retval == CMT_SUCCESS)
    {
        if (CMT_OPEN > gb_cmt_info[channel].state)
        {
            retval = CMT_ERR_NOT_OPENED;
        }
        else if (CMT_RUN == gb_cmt_info[channel].state)
        {
            retval = CMT_ERR_TIMER_RUNNING;
        }
    }

    if (retval == CMT_SUCCESS)
    {
        /* call count start function with CMTW periodic operation */
        gb_cmt_info[channel].state = CMT_RUN;
        userdef_cmt_start(channel, CMT_ONE_SHOT);
    }

    return retval;
}
/*******************************************************************************
End of function CMT_StartOneShot
*******************************************************************************/

/*******************************************************************************
* Function Name: CMT_Control
* Description  : This function provides commands for changing CMT setting,
*                operating counter.
*
* Arguments    : channel-
*                    CMT channel
*                cmd-
*                    Command to CMT operation
*                p_data-
*                    Pointer to optional configuration structure
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_CH-
*                    Selected CMTW channel is invalid.
*                CMT_ERR_INVALID_ARG-
*                    CMTW setting parameters is invalid.
*                CMT_ERR_NOT_OPENED-
*                    Function is called when status is not closed.
*                CMT_ERR_TIMER_RUNNING-
*                    Function is called while running counter.
*                CMT_ERR_MISSING_PTR-
*                    CMTW setting parameter is NULL pointer.
*******************************************************************************/
cmt_err_t CMT_Control(const uint32_t channel, const uint32_t cmd, void * const p_data)
{
    cmt_err_t       retval = CMT_SUCCESS;
    cmt_mode_t *    pmode_param;
    uint32_t *      pcmt_status;

    /* check channel */
    if (channel >= CMT_CH_NUM)
    {
        retval = CMT_ERR_INVALID_CH;
    }

    if (CMT_SUCCESS == retval)
    {
        /* check wheter CMTW status is open or running */
        if (CMT_CLOSE == gb_cmt_info[channel].state)
        {
            retval = CMT_ERR_NOT_OPENED;
        }
    }

    if (CMT_SUCCESS == retval)
    {
        /* operation for every CMTW command */
        switch (cmd)
        {
        /* check wheter CMTW status is open */
        case CMT_CMD_SET_TIME_CNT :
            if (CMT_RUN == gb_cmt_info[channel].state)
            {
                retval = CMT_ERR_TIMER_RUNNING;
            }
            else
            {
                /* casting the void pointer to a cmt_time_cnt_t type is valid structure,
                   because structure member has justification checked separately */
                retval = cmt_check_time_cnt_param((cmt_time_cnt_t *)p_data);
                if (CMT_SUCCESS == retval)
                {
                    userdef_cmt_set_time_cnt_param(channel, (cmt_time_cnt_t *)p_data);
                }
            }
            break;

        case CMT_CMD_SET_MODE:

            /* casting the void pointer to a cmt_mode_t type is valid structure,
               because structure member has justification checked separately */
            pmode_param = (cmt_mode_t *)p_data;

            if (NULL == pmode_param)
            {
                retval = CMT_ERR_MISSING_PTR;
            }
            else
            {
                /* check wheter CMTW status is open */
                if (CMT_RUN == gb_cmt_info[channel].state)
                {
                    retval = CMT_ERR_TIMER_RUNNING;
                }
                else
                {
                    userdef_cmt_set_compare_match_param(channel, &pmode_param->compare_match);
                }
            }
            break;

        case CMT_CMD_SET_PAUSE:
            if (CMT_RUN == gb_cmt_info[channel].state)
            {
                gb_cmt_info[channel].state = CMT_PAUSE;
                userdef_cmt_stop(channel);
            }
            else
            {
                retval = CMT_ERR_TIMER_STOP;
            }
            break;

        case CMT_CMD_SET_RESUME:
            if (CMT_PAUSE == gb_cmt_info[channel].state)
            {
                gb_cmt_info[channel].state = CMT_RUN;
                userdef_cmt_start(channel, gb_cmt_info[channel].operate_mode);
            }
            else if (CMT_RUN == gb_cmt_info[channel].state)
            {
                retval = CMT_ERR_TIMER_RUNNING;
            }
            else
            {
                retval = CMT_ERR_TIMER_STOP;
            }
            break;

        case CMT_CMD_SET_RESTART :
            if ((CMT_OPEN <= gb_cmt_info[channel].state)
                && (CMT_RUN > gb_cmt_info[channel].state))
            {
                gb_cmt_info[channel].p_cmt_ch_reg->CMCNT = 0U;
                gb_cmt_info[channel].state = CMT_RUN;
                userdef_cmt_start(channel, gb_cmt_info[channel].operate_mode);
            }
            else if (CMT_RUN == gb_cmt_info[channel].state)
            {
                retval = CMT_ERR_TIMER_RUNNING;
            }
            else
            {
                retval = CMT_ERR_TIMER_STOP;
            }
            break;

        case CMT_CMD_GET_STATUS :

            /* casting the void pointer to a uint32_t type is valid structure,
               because it is used in order to return value of CMT_STATUS_RUNNING and
               CMTW_STATUS_STOP is a member of uint32_t */
            pcmt_status = (uint32_t *)p_data;

            /* check NULL pointer for reporting status */
            if (NULL == pcmt_status)
            {
                retval = CMT_ERR_MISSING_PTR;
            }
            else
            {
                if (CMT_RUN == gb_cmt_info[channel].state)
                {
                    /* set timer running status */
                    *pcmt_status = CMT_STATUS_RUNNING;
                }
                else
                {
                    /* set timer stop status */
                    *pcmt_status = CMT_STATUS_STOP;
                }
            }
            break;

        default:
            retval = CMT_ERR_INVALID_ARG;
            break;

        }
    }

    return retval;
}
/*******************************************************************************
End of function R_CMT_Control
*******************************************************************************/

/*******************************************************************************
* Function Name: cmt_check_time_cnt_param
* Description  : This function check time count parameter.
* Arguments    : p_time_cnt_param-
*                    Pointer to time count parameter
* Return Value : CMT_SUCCESS-
*                    Successful
*                CMT_ERR_INVALID_ARG-
*                    CMT setting parameters is invalid.
*******************************************************************************/
cmt_err_t cmt_check_time_cnt_param(cmt_time_cnt_t * const p_time_cnt_param)
{
    cmt_err_t       retval = CMT_SUCCESS;

    if (p_time_cnt_param == NULL)
    {
        retval = CMT_ERR_MISSING_PTR;
    }
    else
    {
        if (CMT_PCLK_DIV_MAX_OVER <= p_time_cnt_param->pclk_div)
        {
            retval = CMT_ERR_INVALID_ARG;
        }
    }

    return retval;
}
/*******************************************************************************
End of function cmt_check_time_cnt_param
*******************************************************************************/

/* End of File */

