/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : cmt(w) driver functions for cmt(w) fit module.
* File Name    : r_cmt.h
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMT on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_CMT_HEADER_
#define _R_CMT_HEADER_

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"
#include "r_cmt_if.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define CMT_CH_NUM                              (6U)
#define CMTW_CH_NUM                             (2U)

#define CMT_CH_TOTAL                            (CMT_CH_NUM + CMTW_CH_NUM)

#define CMT_CH_0                                (0U)
#define CMT_CH_1                                (1U)
#define CMT_CH_2                                (2U)
#define CMT_CH_3                                (3U)
#define CMT_CH_4                                (4U)
#define CMT_CH_5                                (5U)
#define CMTW_CH_0                               (0U)
#define CMTW_CH_1                               (1U)

/*----------------------------------------------------------------------------*/
/*   Define register setting value.                                           */
/*----------------------------------------------------------------------------*/
#define CMTW_CMWSTR_INIT                        (0x0000U)
#define CMTW_CMWCR_INIT                         (0x0000U)
#define CMTW_CMWIOR_INIT                        (0x0000U)
#define CMTW_CMWCNT_INIT                        (0x00000000U)
#define CMTW_CMWCOR_INIT                        (0xFFFFFFFFU)
#define CMTW_CMWOCR0_INIT                       (0xFFFFFFFFU)
#define CMTW_CMWOCR1_INIT                       (0xFFFFFFFFU)
#define CMTW_NFCR0_INIT                         (0x00000000U)
#define CMTW_NFCR1_INIT                         (0x00000000U)
#define CMTW_ECDMESLR_INIT                      (0x00000000U)
#define CMTW_CMWCR_CCLR_SET_CMWCOR              (0x00U)
#define CMTW_CMWCR_CCLR_SET_CMWICR0             (0x04U)
#define CMTW_CMWCR_CCLR_SET_CMWICR1             (0x05U)
#define CMTW_CMWCR_CCLR_SET_CMWOCR0             (0x06U)
#define CMTW_CMWCR_CCLR_SET_CMWOCR1             (0x07U)
#define CMTW_CMWCR_CCLR_SET_NONE                (0x01U)
#define CMTW_CMWCOR_MASK_16BIT                  (0x0000FFFFU)
#define CMTW_CMWOCR0_MASK_16BIT                 (0x0000FFFFU)
#define CMTW_CMWCOR1_MASK_16BIT                 (0x0000FFFFU)
#define CMTW_ECDMESLR_DMERSL_SET_NONE           (0x00U)
#define CMTW_ECDMESLR_DMERSL_SET_CMTW0_OUT0     (0x01U)
#define CMTW_ECDMESLR_DMERSL_SET_CMTW0_OUT1     (0x02U)
#define CMTW_ECDMESLR_DMERSL_SET_CMTW1_OUT0     (0x03U)
#define CMTW_ECDMESLR_DMERSL_SET_CMTW1_OUT1     (0x04U)

/*----------------------------------------------------------------------------*/
/*   Define CMTW operation mode.                                              */
/*----------------------------------------------------------------------------*/
#define CMT_PERIODIC                            (0U)    /* CMTW timer is periodic mode               */
#define CMT_ONE_SHOT                            (1U)    /* CMTW timer is one shot mode               */

/*******************************************************************************
Typedef definitions
*******************************************************************************/
/*----------------------------------------------------------------------------*/
/*   Define CMTW statUs.                                                      */
/*----------------------------------------------------------------------------*/
typedef enum e_cmt_state
{
    CMT_UNINIT = 0,
    CMT_INIT   = 1,
    CMT_CLOSE  = 1,
    CMT_OPEN,
    CMT_PAUSE,
    CMT_END,
    CMT_RUN
} cmt_state_t;

/*----------------------------------------------------------------------------*/
/*   Define return values and values of channel state flag.                   */
/*----------------------------------------------------------------------------*/
typedef enum e_cmt_err
{
    CMT_SUCCESS            = 0,               /* Successful operation                      */
    CMT_ERR_INVALID_CH     = 1,               /* Select invalid CMTW channel               */
    CMT_ERR_INVALID_ARG    = 2,               /* Set invalid arg value                     */
    CMT_ERR_NOT_OPENED     = 3,               /* CMTW status is closed after open          */
    CMT_ERR_NOT_CLOSED     = 4,               /* CMTW status is opend before open          */
    CMT_ERR_TIMER_RUNNING  = 5,               /* CMTW timer running on going set parameter */
    CMT_ERR_TIMER_STOP     = 6,               /* CMTW timer counter temporary              */
    CMT_ERR_MISSING_PTR    = 7                /* Function argument pointer is NULL         */
} cmt_err_t;

typedef struct st_cmt_info_ch
{
    uint32_t                    state;
    uint32_t                    operate_mode;
    uint32_t                    clear_factor;
    int32_t                     compare_match_enable;
    void                        (* p_compare_match_callback)(void);

    volatile struct st_cmt0 *   p_cmt_ch_reg;
    volatile unsigned short *   p_cmt_ch_str;
    unsigned short              str_mask;
} cmt_info_ch_t;

typedef struct st_cmtw_info_ch
{
    uint32_t                    state;
    uint32_t                    operate_mode;
    uint32_t                    clear_factor;
    int32_t                     compare_match_enable;
    void                        (* p_compare_match_callback)(void);

    int32_t                     input_capture_enable[CMTW_INPUT_CAPTURE_NUM];
    int32_t                     output_compare_enable[CMTW_OUTPUT_COMPARE_NUM];
    void                        (* p_input_capture_callback[CMTW_INPUT_CAPTURE_NUM])(uint32_t cnt_value);
    void                        (* p_output_compare_callback[CMTW_OUTPUT_COMPARE_NUM])(void);
    volatile struct st_cmtw0 *  p_cmtw_ch_reg;
} cmtw_info_ch_t;

typedef struct st_cmt_control
{
    cmt_state_t                 state;
    osSemaphoreId               sem_state;                      /* I/F semaphore handle      */
    cmt_info_ch_t *             p_ch_info[CMT_CH_TOTAL];
} cmt_control_t;

typedef cmt_cfg_t               cmtw_cfg_t;

/*******************************************************************************
Exported global variables
*******************************************************************************/
extern cmt_info_ch_t gb_cmt_info[CMT_CH_NUM];
extern cmtw_info_ch_t gb_cmtw_info[CMTW_CH_NUM];
extern cmt_control_t cmt_control;

/*******************************************************************************
Exported global functions (to be accessed by other files)
*******************************************************************************/
cmt_err_t CMT_Open(cmt_channel_t const channel, cmt_cfg_t * const p_cfg);
cmt_err_t CMT_Close(uint32_t const channel);
cmt_err_t CMT_StartPeriodic(uint32_t const channel);
cmt_err_t CMT_StartOneShot(uint32_t const channel);
cmt_err_t CMT_Control(const uint32_t channel, const uint32_t cmd, void * const p_data);
void   cmtw_count_stop_hw(const uint32_t channel);

cmt_err_t CMTW_Open(const uint32_t channel, const cmtw_cfg_t * const p_cfg);
cmt_err_t CMTW_Close(const uint32_t channel);
cmt_err_t CMTW_StartPeriodic(const uint32_t channel);
cmt_err_t CMTW_StartOneShot(const uint32_t channel);
cmt_err_t CMTW_Control(const uint32_t channel, const uint32_t cmd, void * const p_data);


#endif /* _R_CMT_HEADER_ */

/* End of File */

