/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : cmtw driver interrupt functions for cmtw fit module.
* File Name    : r_cmtw_int.c
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for CMT(W) driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using CMTW on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_CMTW_INT_HEADER_
#define _R_CMTW_INT_HEADER_

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*----------------------------------------------------------------------------*/
/*   Define register setting value for interrupt.                             */
/*----------------------------------------------------------------------------*/
#define CMTW_CMWCR_CMWIE_SET                    (1U << 3)
#define CMTW_CMWCR_IC0IE_SET                    (1U << 4)
#define CMTW_CMWCR_IC1IE_SET                    (1U << 5)
#define CMTW_CMWCR_OC0IE_SET                    (1U << 6)
#define CMTW_CMWCR_OC1IE_SET                    (1U << 7)

/*----------------------------------------------------------------------------*/
/*   Define interrupt vector setting.                                 */
/*----------------------------------------------------------------------------*/
#define CMTW0_INTVEC_CMWI                       (ICU_VEC_NUM_25)
#define CMTW0_INTVEC_IC0                        (ICU_VEC_NUM_26)
#define CMTW0_INTVEC_IC1                        (ICU_VEC_NUM_27)
#define CMTW0_INTVEC_OC0                        (ICU_VEC_NUM_28)
#define CMTW0_INTVEC_OC1                        (ICU_VEC_NUM_29)
#define CMTW1_INTVEC_CMWI                       (ICU_VEC_NUM_30)
#define CMTW1_INTVEC_IC0                        (ICU_VEC_NUM_31)
#define CMTW1_INTVEC_IC1                        (ICU_VEC_NUM_32)
#define CMTW1_INTVEC_OC0                        (ICU_VEC_NUM_33)
#define CMTW1_INTVEC_OC1                        (ICU_VEC_NUM_34)
#define CMTW_INT_MAX_PRIORITY                   (ICU_PRIORITY_15)

/*******************************************************************************
Typedef definitions
*******************************************************************************/
#define CMTW_INTR_COMPARE_MATCH         (0U)      /* Compare match interrupt              */
#define CMTW_INTR_INPUT_CAPTURE0        (1U)      /* Input capture0 interrupt             */
#define CMTW_INTR_INPUT_CAPTURE1        (2U)      /* Input capture1 interrupt             */
#define CMTW_INTR_OUTPUT_COMPARE0       (3U)      /* Output compare0 interrupt            */
#define CMTW_INTR_OUTPUT_COMPARE1       (4U)      /* Output compare1 interrupt            */
#define CMTW_INTR_TYPE_NUM              (5U)      /* Interrupt type number                */

/*----------------------------------------------------------------------------*/
/*   Define CMTW interrupt table structure.                                   */
/*----------------------------------------------------------------------------*/
typedef struct
{
    IRQn_Type   vic_irq;
    uint32_t    type;
    void *      intr_handler;
    uint16_t    cmwcr_mask_clear_val;
}cmtw_intr_t;

/*******************************************************************************
Exported global variables
*******************************************************************************/

/*******************************************************************************
Exported global functions (to be accessed by other files)
*******************************************************************************/

void cmtw_intr_register(const uint32_t channel, const uint32_t cmtw_intr_num, const int32_t intr_priority);
void cmtw_intr_enable(const uint32_t channel, const uint32_t cmtw_intr_num);
void cmtw_intr_disable(const uint32_t channel, const uint32_t cmtw_intr_num);

#endif /* _R_CMTW_INT_HEADER_ */

/* End of File */

