/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : elc driver functions for elc fit module.
* File Name    : r_elc.h
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for ELC driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using ELC on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_ELC_HEADER_
#define _R_ELC_HEADER_

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"
#include "r_elc_if.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

/*----------------------------------------------------------------------------*/
/*   Define register setting value.                                           */
/*----------------------------------------------------------------------------*/
#define ELC_ELCR_INIT                        (0x7FU)
#define ELC_ELSR_INIT                        (0x00U)
#define ELC_ELOPA_INIT                       (0xFFU)
#define ELC_ELOPA_NONE_EVENT                 (0x03U)
#define ELC_ELOPB_INIT                       (0xFFU)
#define ELC_ELOPB_NONE_EVENT                 (0x03U)
#define ELC_ELOPC_INIT                       (0xFFU)
#define ELC_ELOPC_NONE_EVENT                 (0x03U)
#define ELC_ELOPD_INIT                       (0xFFU)
#define ELC_ELOPD_EVENT_ENABLE               (0x00U)
#define ELC_ELOPD_EVENT_DISABLE              (0x03U)
#define ELC_PGR_INIT                         (0x00U)
#define ELC_PGC_INIT                         (0x88U)
#define ELC_PDBF_INIT                        (0x00U)
#define ELC_PEL_INIT                         (0x80U)
#define ELC_ELSEGR_INIT                      (0xBEU)
#define ELC_ELOPF_INIT                       (0xFFU)
#define ELC_ELOPF_NONE_EVENT                 (0x03U)
#define ELC_ELOPH_INIT                       (0xFFU)
#define ELC_ELOPH_NONE_EVENT                 (0x03U)
#define ELC_ELOPI_INIT                       (0xFFU)
#define ELC_ELOPI_NONE_EVENT                 (0x07U)
#define ELC_ELOPJ_INIT                       (0xFFU)
#define ELC_ELOPJ_NONE_EVENT                 (0x07U)
#define ELC_ELOPD_DSU0MD_ENABLE              (0x00U)
#define ELC_ELOPD_DSU0MD_DISABLE             (0x03U)
#define ELC_ELOPD_DSU1MD_ENABLE              (0x00U)
#define ELC_ELOPD_DSU1MD_DISABLE             (0x03U)
#define ELC_ELOPD_DSU2MD_ENABLE              (0x00U)
#define ELC_ELOPD_DSU2MD_DISABLE             (0x03U)
#define ELC_ELOPD_DSU3MD_ENABLE              (0x00U)
#define ELC_ELOPD_DSU3MD_DISABLE             (0x03U)
#define ELC_ELSEGR_SET_SEG_ENABLE            (0x7EU)
#define ELC_ELSEGR_SET_SOFT_EVENT            (0x7FU)
#define ELC_ELSEGR_CLR_SOFT_EVENT            (0x7EU)
#define ELC_ELSEGR_CLR_SEG_ENABLE            (0x3EU)
#define ELC_ELSEGR_WRITE_DISABLE             (0xBEU)

/*----------------------------------------------------------------------------*/
/*   Define I/O port setting value.                                           */
/*----------------------------------------------------------------------------*/
#define ELC_IO_PORT_NUM                      (7U)

/*******************************************************************************
Typedef definitions
*******************************************************************************/

/*----------------------------------------------------------------------------*/
/*   Define ELC status.                                                       */
/*----------------------------------------------------------------------------*/
typedef enum e_elc_state
{
    ELC_UNINIT = 0,
    ELC_INIT   = 1,
    ELC_CLOSE  = 1,
    ELC_OPEN,
    ELC_RUN,
    ELC_PAUSE,
    ELC_END
} elc_state_t;

/*----------------------------------------------------------------------------*/
/*   Define return values and values of channel state flag.                   */
/*----------------------------------------------------------------------------*/
typedef enum
{
    ELC_SUCCESS             = 0,    /* Successful operation                   */
    ELC_ERR_INVALID_ARG     = 1,    /* Set invalid arg value                  */
    ELC_ERR_NOT_OPENED      = 2,    /* ELC status is closed after open        */
    ELC_ERR_NOT_CLOSED      = 3,    /* ELC status is opend before open        */
    ELC_ERR_MISSING_PTR     = 4     /* Function argument pointer is NULL      */
} elc_err_t;

/*----------------------------------------------------------------------------*/
/*   Define ELC driver information structure.                                 */
/*----------------------------------------------------------------------------*/
typedef struct st_elc_control
{
    elc_state_t     state;                                  /* ELC state            */
    osSemaphoreId   sem_state;                              /* I/F semaphore handle */
    void            ( * p_callback[ELC_INTR_NUM])(void);    /* interrupt callback   */
} elc_control_t;


/*******************************************************************************
Exported global variables
*******************************************************************************/

/*******************************************************************************
Exported global functions (to be accessed by other files)
*******************************************************************************/
extern elc_control_t elc_control;

elc_err_t ELC_Open(void);
void      ELC_Close(void);
elc_err_t ELC_LinkStart(void);
elc_err_t ELC_LinkStop(void);
elc_err_t ELC_Control(uint32_t const cmd, void * const p_data);
uint32_t  ELC_GetVersion(void);

#endif /* _R_ELC_HEADER_ */

/* End of File */

