/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : mtu3a driver functions for mtu3a fit module.
* File Name    : r_mtu3.h
* Version      : 1.00
* Device       : RZ/T1
* Abstract     : header file for MTU3a driver interface
* Tool-Chain   : KPIT Tool Chain Ver.15.01 /
* OS           : RTX
* H/W Platform : RZ/T1
* Description  : Functions for using MTU3a on RZ/T1 devices.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 02.29.2016 1.0      First Release
*******************************************************************************/

#ifndef _R_MTU_HEADER_
#define _R_MTU_HEADER_

/*******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include "r_typedefs.h"
#include "r_mtu3_if.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/

#define MTU_BIT0        (0x01)
#define MTU_BIT1        (0x02)
#define MTU_BIT2        (0x04)
#define MTU_BIT3        (0x08)
#define MTU_BIT4        (0x10)
#define MTU_BIT5        (0x20)
#define MTU_BIT6        (0x40)
#define MTU_BIT7        (0x80)

#define MTU_L_BIT0      (0x00000001)
#define MTU_L_BIT1      (0x00000002)
#define MTU_L_BIT2      (0x00000004)
#define MTU_L_BIT3      (0x00000008)
#define MTU_L_BIT4      (0x00000010)
#define MTU_L_BIT5      (0x00000020)
#define MTU_L_BIT6      (0x00000040)
#define MTU_L_BIT7      (0x00000080)
#define MTU_L_BIT8      (0x00000100)
#define MTU_L_BIT9      (0x00000200)
#define MTU_L_BIT10     (0x00000400)
#define MTU_L_BIT11     (0x00000800)
#define MTU_L_BIT12     (0x00001000)
#define MTU_L_BIT13     (0x00002000)
#define MTU_L_BIT14     (0x00004000)
#define MTU_L_BIT15     (0x00008000)
#define MTU_L_BIT16     (0x00010000)
#define MTU_L_BIT17     (0x00020000)
#define MTU_L_BIT18     (0x00040000)
#define MTU_L_BIT19     (0x00080000)
#define MTU_L_BIT20     (0x00100000)
#define MTU_L_BIT21     (0x00200000)
#define MTU_L_BIT22     (0x00400000)
#define MTU_L_BIT23     (0x00800000)
#define MTU_L_BIT24     (0x01000000)
#define MTU_L_BIT25     (0x02000000)
#define MTU_L_BIT26     (0x04000000)
#define MTU_L_BIT27     (0x08000000)
#define MTU_L_BIT28     (0x10000000)
#define MTU_L_BIT29     (0x20000000)
#define MTU_L_BIT30     (0x40000000)
#define MTU_L_BIT31     (0x80000000)

#define RZT1_PCLKB_HZ   (150000000)

#define MTU_NOT_SUPP    (0xFF) // A value used to mark register settings table entries where setting not available.
#define MTU_NOT_SUP2    (0xFFFFFFFFU)

/* Define register addresses assigned to each MTU register. */
#define MTU0_REGS { &MTU0.TCR.BYTE,   \
                    &MTU0.TCR2.BYTE,  \
                    &MTU0.TMDR1.BYTE, \
                    &MTU0.TIORH.BYTE, \
                    &MTU0.TIORL.BYTE, \
                    &MTU0.TIER.BYTE,  \
                    &MTU0.TBTM.BYTE,  \
                    &MTU0.NFCR0.BYTE, \
                    &MTU0.TCNT,       \
                    &MTU0.TGRA,       \
                    &MTU0.TGRB,       \
                    &MTU0.TGRC,       \
                    &MTU0.TGRD,       \
                    &MTU0.TGRE,       \
                    &MTU0.TGRF,       \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU1_REGS { &MTU1.TCR.BYTE,   \
                    &MTU1.TCR2.BYTE,  \
                    &MTU1.TMDR1.BYTE, \
                    &MTU1.TIOR.BYTE,  \
                    NULL,             \
                    &MTU1.TIER.BYTE,  \
                    NULL,             \
                    &MTU1.NFCR1.BYTE, \
                    &MTU1.TCNT,       \
                    &MTU1.TGRA,       \
                    &MTU1.TGRB,       \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU2_REGS { &MTU2.TCR.BYTE,   \
                    &MTU2.TCR2.BYTE,  \
                    &MTU2.TMDR1.BYTE, \
                    &MTU2.TIOR.BYTE,  \
                    NULL,             \
                    &MTU2.TIER.BYTE,  \
                    NULL,             \
                    &MTU2.NFCR2.BYTE, \
                    &MTU2.TCNT,       \
                    &MTU2.TGRA,       \
                    &MTU2.TGRB,       \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU3_REGS { &MTU3.TCR.BYTE,   \
                    &MTU3.TCR2.BYTE,  \
                    &MTU3.TMDR1.BYTE, \
                    &MTU3.TIORH.BYTE, \
                    &MTU3.TIORL.BYTE, \
                    &MTU3.TIER.BYTE,  \
                    &MTU3.TBTM.BYTE,  \
                    &MTU3.NFCR3.BYTE, \
                    &MTU3.TCNT,       \
                    &MTU3.TGRA,       \
                    &MTU3.TGRB,       \
                    &MTU3.TGRC,       \
                    &MTU3.TGRD,       \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU4_REGS { &MTU4.TCR.BYTE,   \
                    &MTU4.TCR2.BYTE,  \
                    &MTU4.TMDR1.BYTE, \
                    &MTU4.TIORH.BYTE, \
                    &MTU4.TIORL.BYTE, \
                    &MTU4.TIER.BYTE,  \
                    &MTU4.TBTM.BYTE,  \
                    &MTU4.NFCR4.BYTE, \
                    &MTU4.TCNT,       \
                    &MTU4.TGRA,       \
                    &MTU4.TGRB,       \
                    &MTU4.TGRC,       \
                    &MTU4.TGRD,       \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU5_REGS { NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU6_REGS { &MTU6.TCR.BYTE,   \
                    &MTU6.TCR2.BYTE,  \
                    &MTU6.TMDR1.BYTE, \
                    &MTU6.TIORH.BYTE, \
                    &MTU6.TIORL.BYTE, \
                    &MTU6.TIER.BYTE,  \
                    &MTU6.TBTM.BYTE,  \
                    &MTU6.NFCR6.BYTE, \
                    &MTU6.TCNT,       \
                    &MTU6.TGRA,       \
                    &MTU6.TGRB,       \
                    &MTU6.TGRC,       \
                    &MTU6.TGRD,       \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU7_REGS { &MTU7.TCR.BYTE,   \
                    &MTU7.TCR2.BYTE,  \
                    &MTU7.TMDR1.BYTE, \
                    &MTU7.TIORH.BYTE, \
                    &MTU7.TIORL.BYTE, \
                    &MTU7.TIER.BYTE,  \
                    &MTU7.TBTM.BYTE,  \
                    &MTU7.NFCR7.BYTE, \
                    &MTU7.TCNT,       \
                    &MTU7.TGRA,       \
                    &MTU7.TGRB,       \
                    &MTU7.TGRC,       \
                    &MTU7.TGRD,       \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL}

#define MTU8_REGS { &MTU8.TCR.BYTE,   \
                    &MTU8.TCR2.BYTE,  \
                    &MTU8.TMDR1.BYTE, \
                    &MTU8.TIORH.BYTE, \
                    &MTU8.TIORL.BYTE, \
                    &MTU8.TIER.BYTE,  \
                    NULL,             \
                    &MTU8.NFCR8.BYTE, \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    NULL,             \
                    &MTU8.TCNT,       \
                    &MTU8.TGRA,       \
                    &MTU8.TGRB,       \
                    &MTU8.TGRC,       \
                    &MTU8.TGRD}

/* ICU.IER.IEN  bits for each TGI source.   0xFF = not available this channel.      */
/*                       { TGIA          TGIB          TGIC          TGID         } */
#define MTU0_TGI_EN      { TGIA0_IRQn,   TGIB0_IRQn,   TGIC0_IRQn,   TGID0_IRQn   }
#define MTU1_TGI_EN      { TGIA1_IRQn,   TGIB1_IRQn,   MTU_NOT_SUP2, MTU_NOT_SUP2 }
#define MTU2_TGI_EN      { TGIA2_IRQn,   TGIB2_IRQn,   MTU_NOT_SUP2, MTU_NOT_SUP2 }
#define MTU3_TGI_EN      { TGIA3_IRQn,   TGIB3_IRQn,   TGIC3_IRQn,   TGID3_IRQn   }
#define MTU4_TGI_EN      { TGIA4_IRQn,   TGIB4_IRQn,   TGIC4_IRQn,   TGID4_IRQn   }
#define MTU5_TGI_EN      { MTU_NOT_SUP2, MTU_NOT_SUP2, MTU_NOT_SUP2, MTU_NOT_SUP2 }
#define MTU6_TGI_EN      { TGIA6_IRQn,   TGIB6_IRQn,   TGIC6_IRQn,   TGID6_IRQn   }
#define MTU7_TGI_EN      { TGIA7_IRQn,   TGIB7_IRQn,   TGIC7_IRQn,   TGID7_IRQn   }
#define MTU8_TGI_EN      { TGIA8_IRQn,   TGIB8_IRQn,   TGIC8_IRQn,   TGID8_IRQn   }

/*                       { TGIA          TGIB          TGIC          TGID         }  */
#define MTU0_TGI_ST_MASK { MTU_L_BIT17,  MTU_L_BIT18,  MTU_L_BIT19,  MTU_L_BIT20  }
#define MTU1_TGI_ST_MASK { MTU_L_BIT24,  MTU_L_BIT25,  MTU_NOT_SUP2, MTU_NOT_SUP2 }
#define MTU2_TGI_ST_MASK { MTU_L_BIT28,  MTU_L_BIT29,  MTU_NOT_SUP2, MTU_NOT_SUP2 }
#define MTU3_TGI_ST_MASK { MTU_L_BIT0,   MTU_L_BIT1,   MTU_L_BIT2,   MTU_L_BIT3   }
#define MTU4_TGI_ST_MASK { MTU_L_BIT5,   MTU_L_BIT6,   MTU_L_BIT7,   MTU_L_BIT8   }
#define MTU5_TGI_ST_MASK { MTU_NOT_SUP2, MTU_NOT_SUP2, MTU_NOT_SUP2, MTU_NOT_SUP2 }
#define MTU6_TGI_ST_MASK { MTU_L_BIT22,  MTU_L_BIT23,  MTU_L_BIT24,  MTU_L_BIT25  }
#define MTU7_TGI_ST_MASK { MTU_L_BIT27,  MTU_L_BIT28,  MTU_L_BIT29,  MTU_L_BIT30  }
#define MTU8_TGI_ST_MASK { MTU_L_BIT13,  MTU_L_BIT14,  MTU_L_BIT15,  MTU_L_BIT16  }

#if MTU_CFG_USE_CH0 == 1
#define MTU0_INTR_P { (void const *)mtu_tgia0_isr, (void const *)mtu_tgib0_isr, (void const *)mtu_tgic0_isr, (void const *)mtu_tgid0_isr  }
#else
#define MTU0_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif
#if MTU_CFG_USE_CH1 == 1
#define MTU1_INTR_P { (void const *)mtu_tgia1_isr, (void const *)mtu_tgib1_isr, (void const *)NULL, (void const *)NULL }
#else
#define MTU1_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif
#if MTU_CFG_USE_CH2 == 1
#define MTU2_INTR_P { (void const *)mtu_tgia2_isr, (void const *)mtu_tgib2_isr, (void const *)NULL, (void const *)NULL }
#else
#define MTU2_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif
#if MTU_CFG_USE_CH3 == 1
#define MTU3_INTR_P { (void const *)mtu_tgia3_isr, (void const *)mtu_tgib3_isr, (void const *)mtu_tgic3_isr, (void const *)mtu_tgid3_isr }
#else
#define MTU5_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif
#if MTU_CFG_USE_CH4 == 1
#define MTU4_INTR_P { (void const *)mtu_tgia4_isr, (void const *)mtu_tgib4_isr, (void const *)mtu_tgic4_isr, (void const *)mtu_tgid4_isr }
#else
#define MTU4_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif

#define MTU5_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }

#if MTU_CFG_USE_CH6 == 1
#define MTU6_INTR_P { (void const *)mtu_tgia6_isr, (void const *)mtu_tgib6_isr, (void const *)mtu_tgic6_isr, (void const *)mtu_tgid6_isr }
#else
#define MTU6_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif
#if MTU_CFG_USE_CH7 == 1
#define MTU7_INTR_P { (void const *)mtu_tgia7_isr, (void const *)mtu_tgib7_isr, (void const *)mtu_tgic7_isr, (void const *)mtu_tgid7_isr }
#else
#define MTU7_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif
#if MTU_CFG_USE_CH8 == 1
#define MTU8_INTR_P { (void const *)mtu_tgia8_isr, (void const *)mtu_tgib8_isr, (void const *)mtu_tgic8_isr, (void const *)mtu_tgid8_isr }
#else
#define MTU8_INTR_P { (void const *)NULL, (void const *)NULL, (void const *)NULL, (void const *)NULL }
#endif

/* TCR.TPSC[2:0] bits for internal clock selection. From Tables.  0xFF = not available this channel. */
/* PCLK divisor        { PCLK/1 | PCLKC/2 | PCLK/4 | PCLKC/8 | PCLK/16 | PCLKC/32 | PCLK/64 | PCLK/256 | PCLK/1024 }                */
#define MTU0_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x40,      0x50    }
#define MTU1_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x06,      0x40    }
#define MTU2_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x40,      0x07    }
#define MTU3_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x04,      0x05    }
#define MTU4_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x04,      0x05    }
#define MTU5_PCLK_DIVS {  0xFF,     0xFF,    0xFF,     0xFF,     0xFF,     0xFF,      0xFF,     0xFF,      0xFF    }
#define MTU6_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x04,      0x05    }
#define MTU7_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x04,      0x05    }
#define MTU8_PCLK_DIVS {  0x00,     0x10,    0x01,     0x20,     0x02,     0x30,      0x03,     0x04,      0x05    }

/* TCR.TPSC[2:0]  bits for external clock selection. From Tables.   0xFF = not available this channel.               */
/*                    { MTCLKA MTCLKB MTCLKC MTCLKD OFL_UFL }  */
#define MTU0_EXT_CLKS { 0x04,  0x05,  0x06,  0x07,  0xFF    }
#define MTU1_EXT_CLKS { 0x04,  0x05,  0xFF,  0xFF,  0x07    }
#define MTU2_EXT_CLKS { 0x04,  0x05,  0x06,  0xFF,  0xFF    }
#define MTU3_EXT_CLKS { 0x06,  0x07,  0xFF,  0xFF,  0xFF    }
#define MTU4_EXT_CLKS { 0x06,  0x07,  0xFF,  0xFF,  0xFF    }
#define MTU5_EXT_CLKS { 0xFF,  0xFF,  0xFF,  0xFF,  0xFF    }
#define MTU6_EXT_CLKS { 0x06,  0x07,  0xFF,  0xFF,  0xFF    }
#define MTU7_EXT_CLKS { 0x06,  0x07,  0xFF,  0xFF,  0xFF    }
#define MTU8_EXT_CLKS { 0x06,  0x07,  0xFF,  0xFF,  0xFF    }

/* TCR.CCLR[2:0] bits for counter clearing selection. From Tables.   0xFF = not available this channel.     */
/* TCNT clear source { TGRA  TGRB  TGRC  TGRD  Sync  disabled}  */
#define MTU0_CLR_SRC { 0x20, 0x40, 0xA0, 0xC0, 0x60,  0x00 }
#define MTU1_CLR_SRC { 0x20, 0x40, 0xFF, 0xFF, 0x60,  0x00 }
#define MTU2_CLR_SRC { 0x20, 0x40, 0xFF, 0xFF, 0x60,  0x00 }
#define MTU3_CLR_SRC { 0x20, 0x40, 0xA0, 0xC0, 0x60,  0x00 }
#define MTU4_CLR_SRC { 0x20, 0x40, 0xA0, 0xC0, 0x60,  0x00 }
#define MTU5_CLR_SRC { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,  0xFF }
#define MTU6_CLR_SRC { 0x20, 0x40, 0xA0, 0xC0, 0x60,  0x00 }
#define MTU7_CLR_SRC { 0x20, 0x40, 0xA0, 0xC0, 0x60,  0x00 }
#define MTU8_CLR_SRC { 0x20, 0x40, 0xA0, 0xC0, 0x60,  0x00 }

#define MTU_MAX_TIMER_TICKS         (0xFFFF)
#define MTU_MAX_TIMER_TICKS2        (0xFFFFFFFF)
#define MTU_NUM_CLK_DIVISORS        (9)
#define MTU_NUM_EXT_CLK_SRCS        (5)
#define MTU_NUM_CLR_SRCS            (8)
#define MTU_NUM_TGIS                (4)
#define MTU_NUM_CH_CMPL_PWM         (2)

#define MTU_ADC_TRIG                (0x80)  // TIER.TTGE bit

#define MTU_TSYR_MASK               (0xC0C7) // Protect reserved TSYR bits.
#define MTU_TSYRA_MASK              (0x00C7) // Protect reserved TSYR bits.
#define MTU_TSYRB_MASK              (0x00C0) // Protect reserved TSYR bits.
#define MTU_TSTR_MASK               (0xC0CF) // Protect reserved TSTR bits.
#define MTU_TSTRA_MASK              (0x00CF) // Protect reserved TSTRA bits.
#define MTU_TSTRB_MASK              (0x00C0) // Protect reserved TSTRB bits.

#define MTU_CLR_TGRA                (0)
#define MTU_CLR_TGRB                (1)
#define MTU_CLR_TGRC                (2)
#define MTU_CLR_TGRD                (3)

#define MTU_POWER_ON                (0)
#define MTU_POWER_OFF               (1)

/*******************************************************************************
Typedef definitions
*******************************************************************************/
typedef void(*mtu_callback)(void *pargs);

typedef enum e_mtu_state
{
    MTU_UNINIT = 0,
    MTU_INIT   = 1,
    MTU_CLOSE  = 1,
    MTU_OPEN,
    MTU_RUN,
    MTU_PAUSE,
    MTU_END
} mtu_state_t;

typedef enum e_mtu_mode
{
    MTU_MODE_NONE          = 0,
    MTU_MODE_COMPARE_MATCH,
    MTU_MODE_INPUT_CAPTURE,
    MTU_MODE_PWM_MODE_1,
    MTU_MODE_PWM_MODE_2,
    MTU_MODE_CMPL_PWM,
} mtu_mode_t;

typedef enum
{
    MTU_TGIEA = MTU_BIT0, // TGR Interrupt Enable A
    MTU_TGIEB = MTU_BIT1, // TGR Interrupt Enable B
    MTU_TGIEC = MTU_BIT2, // TGR Interrupt Enable C
    MTU_TGIED = MTU_BIT3, // TGR Interrupt Enable D
    MTU_TCIEV = MTU_BIT4, // Overflow Interrupt Enable
    MTU_TCIEU = MTU_BIT5, // Underflow Interrupt Enable
    MTU_TTGE2 = MTU_BIT6, // A/D Converter Start Request Enable
    MTU_TTGE  = MTU_BIT7  // A/D Converter Start Request Enable
} mtu_int_src_t;

typedef enum mtu_filter_en_e
{
    MTU_FILT_EN_A = MTU_BIT0,
    MTU_FILT_EN_B = MTU_BIT1,
    MTU_FILT_EN_C = MTU_BIT2,
    MTU_FILT_EN_D = MTU_BIT3
} mtu_filter_en_t;

/* Create a standardized structure for MTU register addresses used in this module to cover all channels. */
typedef struct mtu_timer_regs_s
{
    volatile unsigned char *    tcr;    // TCR
    volatile unsigned char *    tcr2;   // TCR2
    volatile unsigned char *    tmdr;   // TMDR
    volatile unsigned char *    tiorh;  // TIORH or TIOR
    volatile unsigned char *    tiorl;  // TIORL
    volatile unsigned char *    tier;   // TIER
    volatile unsigned char *    tbtm;   // TBTM
    volatile unsigned char *    nfcr;   // NFCR
    volatile unsigned short *   tcnt;   // TCNT
    volatile unsigned short *   tgra;   // TGRA
    volatile unsigned short *   tgrb;   // TGRB
    volatile unsigned short *   tgrc;   // TGRC
    volatile unsigned short *   tgrd;   // TGRD
    volatile unsigned short *   tgre;   // TGRE
    volatile unsigned short *   tgrf;   // TGRF
    volatile unsigned long *    tcnt_l; // TCNT for MTU8
    volatile unsigned long *    tgra_l; // TGRA for MTU8
    volatile unsigned long *    tgrb_l; // TGRB for MTU8
    volatile unsigned long *    tgrc_l; // TGRC for MTU8
    volatile unsigned long *    tgrd_l; // TGRD for MTU8
} mtu_timer_regs_t;

/* configuration and control structure */
typedef struct mtu_config_block_s
{
    mtu_callback *const         p_callback;                    // pointer to callback function pointer.
    mtu_timer_chnl_settings_t * p_mtu_chnl_tmr_settings; // location of variable timer settings struct.
    mtu_timer_regs_t            regs;
    uint8_t                     channel;
    uint8_t                     priority;        // interrupt priority
    uint8_t                     filt_clk;        // input capture noise filter clock source
} mtu_config_block_t;

typedef struct st_mtu_ch_info
{
    mtu_state_t                 state;
    mtu_mode_t                  mode;
} mtu_ch_info_t;

typedef struct st_mtu_cmpl_pwm_ch_info
{
    mtu_state_t                 state;
    mtu_ch_info_t *             p_ch_info[MTU_NUM_CH_CMPL_PWM];
} mtu_cmpl_pwm_ch_info_t;

typedef struct st_mtu_control
{
    mtu_state_t                 state;                                      /* MTU state                 */
    osSemaphoreId               sem_state;                                  /* I/F semaphore handle      */
    mtu_cmpl_pwm_ch_info_t      cmpl_pwm_ch_info[MTU_CMPL_PWM_CHANNEL_MAX]; /* Information pointer of Compliment PWM MTU ch*/
    mtu_ch_info_t *             p_ch_info[MTU_CHANNEL_MAX];                 /* Information MTU ch */
} mtu_control_t;

/* Abstraction of channel handle data structure. */
typedef struct mtu_config_block_s *mtu_handle_t;

typedef struct mtu_tcr_reg_s
{
    struct
    {
        uint8_t tpsc:3;
        uint8_t ckeg:2;
        uint8_t cclr:3;
    } bits;
} mtu_tcr_reg_t;

typedef enum
{
    MTU_TSTR_CH0 = MTU_BIT0,
    MTU_TSTR_CH1 = MTU_BIT1,
    MTU_TSTR_CH2 = MTU_BIT2,
    MTU_TSTR_CH3 = MTU_BIT6,
    MTU_TSTR_CH4 = MTU_BIT7,
    MTU_TSTR_CH5 = 0xFF,
    MTU_TSTR_CH6 = MTU_BIT6,
    MTU_TSTR_CH7 = MTU_BIT7,
    MTU_TSTR_CH8 = MTU_BIT3,
} mtu_tstr_bits_t;

/* The possible return codes from the API functions. */
typedef enum mtu_pwm_err_e
{
    MTU_SUCCESS = 0,
    MTU_ERR_BAD_CHAN,              // Invalid channel number.
    MTU_ERR_CH_NOT_OPENED,         // Channel not yet opened.
    MTU_ERR_CH_NOT_CLOSED,         // Channel still open from previous open.
    MTU_ERR_UNKNOWN_CMD,           // Control command is not recognized.
    MTU_ERR_INVALID_ARG,           // Argument is not valid for parameter.
    MTU_ERR_ARG_RANGE,             // Argument is out of range for parameter.
    MTU_ERR_NULL_PTR,              // Received null pointer; missing required argument.
    MTU_ERR_LOCK,                  // The lock procedure failed.
    MTU_ERR_UNDEF                  // Undefined/unknown error
} mtu_err_t;

/*******************************************************************************
Pre-declaration of private global variables
*******************************************************************************/
extern const uint32_t       g_mtu_clock_divisors[MTU_NUM_CLK_DIVISORS];
extern volatile uint8_t     g_num_channels_in_use;
extern uint8_t              g_mtu_channel_clr_src[MTU_CHANNEL_MAX];
extern uint8_t              g_mtu_channel_repeats[MTU_CHANNEL_MAX];
extern uint8_t              g_mtu_tgr_callbacks[MTU_CHANNEL_MAX][MTU_NUM_TGIS];
extern uint8_t              g_mtu_tgi_icu_en_flags[MTU_CHANNEL_MAX][MTU_NUM_TGIS];
extern const uint8_t        g_chnl_clk_divs[MTU_CHANNEL_MAX][MTU_NUM_CLK_DIVISORS];
extern const uint8_t        g_chnl_ext_clks[MTU_CHANNEL_MAX][MTU_NUM_EXT_CLK_SRCS];
extern const uint8_t        g_chnl_clear_src[MTU_CHANNEL_MAX][MTU_NUM_CLR_SRCS];
extern const uint8_t        g_mtu_tstr_bits[];
extern const mtu_handle_t   g_mtu_handles[];

extern mtu_control_t        mtu_control;

/*******************************************************************************
Pre-declaration of private local functions
*******************************************************************************/
extern void     mtu_interrupts_enable(uint8_t channel);
extern void     mtu_interrupts_disable(uint8_t channel);
extern void     mtu_channel_clear(uint8_t channel);
extern void     mtu_interrupts_group_enable(uint16_t group);
extern void     mtu_interrupts_group_disable(uint16_t group);
extern bool_t   mtu_check_group(uint16_t group);
extern void     power_on_off (uint8_t on_or_off);
extern bool_t   mtu_calc_clock_divisor(uint8_t chan, uint8_t *div_idx, uint32_t frq_target);
extern uint16_t mtu_calc_tgr_ticks(uint16_t pclk_div, uint32_t frq_target );
extern uint32_t mtu_calc_tgr_ticks2(uint16_t pclk_div, uint32_t frq_target );
extern void     mtu_interrupts_regster(uint8_t channel);
extern bool_t   mtu_check_close(void);

#if MTU_CFG_USE_CH0 == 1
void mtu_tgia0_isr(void);
void mtu_tgib0_isr(void);
void mtu_tgic0_isr(void);
void mtu_tgid0_isr(void);
#endif

#if MTU_CFG_USE_CH1 == 1
void mtu_tgia1_isr(void);
void mtu_tgib1_isr(void);
#endif

#if MTU_CFG_USE_CH2 == 1
void mtu_tgia2_isr(void);
void mtu_tgib2_isr(void);
#endif

#if MTU_CFG_USE_CH3 == 1
void mtu_tgia3_isr(void);
void mtu_tgib3_isr(void);
void mtu_tgic3_isr(void);
void mtu_tgid3_isr(void);
#endif

#if MTU_CFG_USE_CH4 == 1
void mtu_tgia4_isr(void);
void mtu_tgib4_isr(void);
void mtu_tgic4_isr(void);
void mtu_tgid4_isr(void);
#endif

#if MTU_CFG_USE_CH6 == 1
void mtu_tgia6_isr(void);
void mtu_tgib6_isr(void);
void mtu_tgic6_isr(void);
void mtu_tgid6_isr(void);
#endif

#if MTU_CFG_USE_CH7 == 1
void mtu_tgia7_isr(void);
void mtu_tgib7_isr(void);
void mtu_tgic7_isr(void);
void mtu_tgid7_isr(void);
#endif

#if MTU_CFG_USE_CH8 == 1
void mtu_tgia8_isr(void);
void mtu_tgib8_isr(void);
void mtu_tgic8_isr(void);
void mtu_tgid8_isr(void);
#endif

/*******************************************************************************
Exported global functions (to be accessed by other files)
*******************************************************************************/
mtu_err_t MTU_PWM_Compliment_Open(mtu_cmpl_pwm_channel_t const channel, mtu_cmpl_pwm_chnl_settings_t * const pconfig);
mtu_err_t MTU_PWM_Compliment_Close(mtu_cmpl_pwm_channel_t const channel);
mtu_err_t MTU_PWM_Compliment_Control(mtu_cmpl_pwm_channel_t const channel, mtu_cmd_t const cmd, void * const pcmd_data);
mtu_err_t MTU_Timer_Open(mtu_channel_t const channel, mtu_timer_chnl_settings_t * const pconfig, void (* const pcallback)(void * pdata));
mtu_err_t MTU_Capture_Open(mtu_channel_t const channel, mtu_capture_chnl_settings_t * const pconfig, void (* const pcallback)(void * pdata));
mtu_err_t MTU_PWM_Open(mtu_channel_t const channel, mtu_pwm_chnl_settings_t * const pconfig, void (* const pcallback)(void * pdata));
mtu_err_t MTU_Control(mtu_channel_t const channel, mtu_cmd_t const cmd, void * const pcmd_data);
mtu_err_t MTU_Close(mtu_channel_t const channel);

#endif /* _R_MTU_HEADER_ */

/* End of File */

