;*******************************************************************************
; DISCLAIMER
; This software is supplied by Renesas Electronics Corporation and is only
; intended for use with Renesas products. No other uses are authorized. This
; software is owned by Renesas Electronics Corporation and is protected under
; all applicable laws, including copyright laws.
; THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
; THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
; LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
; AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
; TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
; ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
; FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
; ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
; BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
; Renesas reserves the right, without notice, to make changes to this software
; and to discontinue the availability of this software. By using this software,
; you agree to the additional terms and conditions found by accessing the
; following link:
; http://www.renesas.com/disclaimer
;
; Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
;******************************************************************************
;*******************************************************************************
; System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
; File Name    : loader_reset_handlers.s
; Version      : 1.0
; Device       : R7S910017
; Abstract     : Sample program for loader reset handler
; Tool-Chain   : ARM DS-5 Development Studio
; OS           : CMSIS-RTOS RTX
; H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
; Description  : Loader reset handler program of RZ/T1
; Limitation   : none
;******************************************************************************
;*******************************************************************************
; History      : DD.MM.YYYY Version  Description
;                   : 02.12.2016     1.0          First Release
;******************************************************************************

;==================================================================
; Entry point for the Reset handler
;==================================================================
    PRESERVE8
    AREA LOADER_RESET_HANDLER, CODE, READONLY

    IMPORT    cpu_init

    ENTRY

    EXPORT Start_Loader

Start_Loader

;******************************************************************************
; Function Name : loader_reset_handler
; Description   : This function is the assembler function executed after reset
;               : cancellation. After initial setting for the stack pointer or 
;               : the MMU and reset cancellation, executes initial setting for 
;               : the minimum required peripheral functions. Calls the __main
;               : of the standard library function to execute the main function.
;******************************************************************************
loader_reset_handler    FUNCTION {}


;==================================================================
; Enable access to VFP by enabling access to Coprocessors 10 and 11.
; Enables Full Access i.e. in both privileged and non privileged modes
;==================================================================
    IF {TARGET_FPU_VFP}

    MRC  p15, 0, r0, c1, c0, 2                      ;;; Read Coprocessor Access Control Register (CPACR)
    ORR  r0, r0, #(0xF << 20)                       ;;; Enable access to CP 10 & 11
    MCR  p15, 0, r0, c1, c0, 2                      ;;; Write Coprocessor Access Control Register (CPACR)
    ISB                                             ;;; Ensure subsequent insts execute wrt new MPU settings


;==================================================================
; Switch on the VFP hardware
;=================================================================
    MOV  r0, #0x40000000
    VMSR FPEXC, r0                                  ;;; Write FPEXC register, EN bit set

    ENDIF


;==================================================================
; Jump to loader program
;==================================================================
    BL   cpu_init

    ENDFUNC


    END
