/*******************
 *
 * Copyright 1998-2010 IAR Systems AB.
 *
 * This is a template implementation of the "__write" function used by
 * the standard library.  Replace it with a system-specific
 * implementation.
 *
 * The "__write" function should output "size" number of bytes from
 * "buffer" in some application-specific way.  It should return the
 * number of characters written, or _LLIO_ERROR on failure.
 *
 * If "buffer" is zero then __write should perform flushing of
 * internal buffers, if any.  In this case "handle" can be -1 to
 * indicate that all handles should be flushed.
 *
 * The template implementation below assumes that the application
 * provides the function "MyLowLevelPutchar".  It should return the
 * character written, or -1 on failure.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 ********************/

#include "r_typedefs.h"
#include "Serial.h"
#include "sio_char.h"

/*
 * If the __write implementation uses internal buffering, uncomment
 * the following line to ensure that we are called with "buffer" as 0
 * (i.e. flush) when the application terminates.
 */

size_t SioWrite(int handle, const unsigned char * buffer, size_t size)
{
	/* Remove the #if #endif pair to enable the implementation */
#if 1

	size_t nChars = 0;

	if (buffer == 0)
	{
		/*
		 * This means that we should flush internal buffers.  Since we
		 * don't we just return.  (Remember, "handle" == -1 means that all
		 * handles should be flushed.)
		 */
		return 0;
	}

	/* This template only writes to "standard out" and "standard err",
	 * for all other file handles it returns failure. */
	if (handle != STDOUT && handle != STDERR)
	{
		return STDERR;
	}

	for (/* Empty */; size != 0; --size)
	{
#if 0
		if (MyLowLevelPutchar(*buffer++) < 0)
		{
			return _LLIO_ERROR;
		}
#else
		if (*buffer == '\n')
		{
			SER_PutChar('\r');
		}
		SER_PutChar(*buffer++);
#endif

		++nChars;
	}

  return nChars;

#else

	/* Always return error code when implementation is disabled. */
	return STDERR;

#endif

}

/*EOF*/
