/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : Serial.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : Simple polled UART driver
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : Simple polled UART driver
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

#include "cmsis_os.h"
#include "Serial.h"
#include <iodefine.h>

void SER_Init(void)
{
    volatile uint32_t dummy32t;
    volatile uint16_t dummy16t;

    /* SERICLK = 150MHz */
    R_CPG_WriteEnable();
#if   SERICLK==0
    SYSTEM.SCKCR.BIT.SERICK = 0u;
#elif SERICLK==1
    SYSTEM.SCKCR.BIT.SERICK = 1u;
#else
#error Invalid SERICLK speed.
#endif
    R_CPG_WriteDisable();

    /* Enable clock of SCI2 */
    R_RST_WriteEnable();
    SYSTEM.MSTPCRB.BIT.MSTPCRB7 = 0u;
    dummy32t = SYSTEM.MSTPCRB.LONG; /* Workaround for lack of release procedure from the module-stop state(1/2) */
    R_RST_WriteDisable();
    dummy16t = SCIFA2.SMR.WORD;     /* Workaround for lack of release procedure from the module-stop state(2/2) */

    /* mode setting of SCI2 */
    SCIFA2.SMR.BIT.CM    = 0u;    /* Async (UART) */
    SCIFA2.SMR.BIT.CHR   = 0u;    /* Char: 8bit */
    SCIFA2.SMR.BIT.PE    = 0u;    /* Parity: none */
    SCIFA2.SMR.BIT.STOP  = 0u;    /* Stopbit: 1bit */

    SCIFA2.SCR.BIT.TIE   = 0u;    /* Disable TXI */
    SCIFA2.SCR.BIT.RIE   = 0u;    /* Disable RXI, DRI, ERI, BRI */
    SCIFA2.SCR.BIT.REIE  = 0u;    /* Disable ERI, BRI */
    SCIFA2.SCR.BIT.TEIE  = 0u;    /* Disable TEI */

    SCIFA2.FCR.BIT.MCE   = 0u;    /* Disable modem signal */
    SCIFA2.FCR.BIT.LOOP  = 0u;    /* Disable loopback test mode */

    SCIFA2.SEMR.BIT.NFEN = 0u;    /* Disable RxD2 noise erase */
    SCIFA2.SEMR.BIT.DIR  = 0u;    /* LSB first transfer */
    SCIFA2.SEMR.BIT.BRME = 0u;    /* Disable bit modulation */

    /* setup baudrate of SCI2 */
#if   SERICLK==0
    SCIFA2.SMR.BIT.CKS     = 0u;  /* SERICLK/1: n = 0 */
    SCIFA2.SCR.BIT.CKE     = 0u;  /* SCK pin: Input mode (signal ignored). */
    SCIFA2.SEMR.BIT.BGDM   = 0u;  /* normal mode */
    SCIFA2.SEMR.BIT.ABCS0  = 0u;  /* x16 mode */
    SCIFA2.SEMR.BIT.MDDRS  = 0u;  /* Allow access to BRR */
    SCIFA2.BRR_MDDR.BRR    = 40u; /* 115200bps */
#elif SERICLK==1
    SCIFA2.SMR.BIT.CKS     = 0u;  /* SERICLK/1: n = 0 */
    SCIFA2.SCR.BIT.CKE     = 0u;  /* SCK pin: Input mode (signal ignored). */
    SCIFA2.SEMR.BIT.BGDM   = 0u;  /* normal mode */
    SCIFA2.SEMR.BIT.ABCS0  = 0u;  /* x16 mode */
    SCIFA2.SEMR.BIT.MDDRS  = 0u;  /* Allow access to BRR */
    SCIFA2.BRR_MDDR.BRR    = 32u; /* 115200bps */
#else
#error Invalid SERICLK speed.
#endif

    /* pin setting of SCI2 */
    PORT0.PDR.BIT.B1    = 0u;    /* P91: Input mode */
    PORT9.PMR.BIT.B1    = 0u;    /* P91: Port  mode */
    PORT0.PDR.BIT.B2    = 0u;    /* P92: Input mode */
    PORT9.PMR.BIT.B2    = 0u;    /* P92: Port  mode */

    R_MPC_WriteEnable();

    MPC.P91PFS.BIT.ASEL = 0u;    /* Not analog pin */
    MPC.P91PFS.BIT.ISEL = 0u;    /* Not IRQ pin */
    MPC.P91PFS.BIT.PSEL = 0x0Bu; /* Function: TXD2(001011b) */

    MPC.P92PFS.BIT.ASEL = 0u;    /* Not analog pin */
    MPC.P92PFS.BIT.ISEL = 0u;    /* Not IRQ pin */
    MPC.P92PFS.BIT.PSEL = 0x0Bu; /* Function: RXD2(001011b) */

    R_MPC_WriteDisable();

    PORT9.PMR.BIT.B1    = 1u;    /* P91: Peripheral mode */
    PORT9.PMR.BIT.B2    = 1u;    /* P92: Peripheral mode */

    /* Clear status(Just in case). */
    SCIFA2.FCR.BIT.TFRST = 1u;    /* Set reset TxFIFO */
    SCIFA2.FCR.BIT.RFRST = 1u;    /* Set reset RxFIFO */
    SCIFA2.LSR.WORD      = 0u;    /* Clear error status */
    SCIFA2.FCR.BIT.TFRST = 0u;    /* Clear reset TxFIFO */
    SCIFA2.FCR.BIT.RFRST = 0u;    /* Clear reset RxFIFO */

    SER_Enable();
}

void SER_Enable(void)
{
    /* Enable SCI2 */
    SCIFA2.SCR.BIT.RE = 1u;       /* Enable receive */
    SCIFA2.SCR.BIT.TE = 1u;       /* Enable transmit */
}


/*----------------------------------------------------------------------------
  Read character from Serial Port   (blocking read)
 *----------------------------------------------------------------------------*/
void SER_Disable(void)
{
    /* Disable SCI2 */
    SCIFA2.SCR.BIT.RE = 0u;       /* Disable receive */
    SCIFA2.SCR.BIT.TE = 0u;       /* Disable transmit */
}

/*----------------------------------------------------------------------------
  Set baud rate
 *----------------------------------------------------------------------------*/
void SER_Set_baud_rate(uint32_t baud_rate)
{
    //your code here
}


/*----------------------------------------------------------------------------
  Write character to Serial Port
 *----------------------------------------------------------------------------*/
void SER_PutChar(uint8_t buffer)
{
    /* Check if it is possible to transmit (TDFE flag) */
    while (0u == (uint32_t)SCIFA2.FSR.BIT.TDFE)
    {
        /* Busy-waiting */
    }

    /* Write the receiving data in TDR */
    SCIFA2.FTDR = buffer;

    /* Clear TDFE and TEND flag */
    SCIFA2.FSR.BIT.TDFE = 0u;
    SCIFA2.FSR.BIT.TEND = 0u;

    return;
}

/*----------------------------------------------------------------------------
  Read character from Serial Port
 *----------------------------------------------------------------------------*/
uint32_t SER_GetChar (void)
{
    int32_t data;

    /* Confirming receive error(ER,BRK,FER,PER,ORER) */
    if (((((0u != (uint32_t)SCIFA2.FSR.BIT.ER)
           || (0u != (uint32_t)SCIFA2.FSR.BIT.BRK))
          || (0u != (uint32_t)SCIFA2.FSR.BIT.FER))
         || (0u != (uint32_t)SCIFA2.FSR.BIT.PER))
        || (0u != (uint32_t)SCIFA2.LSR.BIT.ORER))
    {
        /* Detect receive error */
        SCIFA2.SCR.BIT.RE    = 0u;    /* Disable reception */
        SCIFA2.FCR.BIT.RFRST = 1u;    /* Reset receiving FIFO */
        SCIFA2.FCR.BIT.RFRST = 0u;    /* Clearing FIFO reception reset */
        SCIFA2.FSR.BIT.ER    = 0u;    /* ER clear */
        SCIFA2.FSR.BIT.BRK   = 0u;    /* BRK clear */
        SCIFA2.FSR.BIT.FER   = 0u;    /* FER clear */
        SCIFA2.FSR.BIT.PER   = 0u;    /* PER clear */
        SCIFA2.LSR.BIT.ORER  = 0u;    /* ORER clear */
        SCIFA2.SCR.BIT.RE    = 1u;    /* Enable reception */

        data = -1;
    }
    else
    {
        /* Is there receive FIFO data? */
        while (0u == (uint32_t)SCIFA2.FSR.BIT.RDF)
        {
            /* Busy-waiting */
            osDelay(50);
        }

        /* Read receive data */
        data = (int32_t)SCIFA2.FRDR;

        /* Clear RDF,DR */
        SCIFA2.FSR.BIT.RDF = 0u;
        SCIFA2.FSR.BIT.DR  = 0u;
    }

    return data;
}

/*----------------------------------------------------------------------------
  Serial Uart interrupt handler
 *----------------------------------------------------------------------------*/
void interrupt_SER(void)
{
    //your code here
}
