;/**************************************************************************//**
; * @file     startup_Renesas_RZ_A1.s
; * @brief    CMSIS Core Device Startup File for
; *           Renesas_RZ_A1 Device Series
; * @version  V1.00
; * @date     23 March 2015
; *
; * @note
; *
; ******************************************************************************/
;/* Copyright (c) 2011 - 2015 ARM LIMITED
;
;   All rights reserved.
;   Redistribution and use in source and binary forms, with or without
;   modification, are permitted provided that the following conditions are met:
;   - Redistributions of source code must retain the above copyright
;     notice, this list of conditions and the following disclaimer.
;   - Redistributions in binary form must reproduce the above copyright
;     notice, this list of conditions and the following disclaimer in the
;     documentation and/or other materials provided with the distribution.
;   - Neither the name of ARM nor the names of its contributors may be used
;     to endorse or promote products derived from this software without
;     specific prior written permission.
;   *
;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
;   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
;   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
;   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
;   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
;   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
;   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
;   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
;   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
;   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
;   POSSIBILITY OF SUCH DAMAGE.
;   ---------------------------------------------------------------------------*/
;/*
;//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
;*/

VIC_BASE        EQU     0xA0010000
VICPIC0_OFFSET  EQU     0x00000120
VICHVA0_OFFSET  EQU     0x00000200

GICI_BASE       EQU     0xe8202000
ICCIAR_OFFSET   EQU     0x0000000C
ICCEOIR_OFFSET  EQU     0x00000010
ICCHPIR_OFFSET  EQU     0x00000018

GICD_BASE       EQU     0xe8201000
ICDISER0_OFFSET EQU     0x00000100
ICDICER0_OFFSET EQU     0x00000180
ICDISPR0_OFFSET EQU     0x00000200
ICDABR0_OFFSET  EQU     0x00000300
ICDIPR0_OFFSET  EQU     0x00000400

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F

I_Bit           EQU     0x80            ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40            ; when F bit is set, FIQ is disabled
T_Bit           EQU     0x20            ; when T bit is set, core is in Thumb state

 

Sect_Normal     EQU     0x00005c06 ;outer & inner wb/wa, non-shareable, executable, rw, domain 0, base addr 0
Sect_Normal_Cod EQU     0x0000dc06 ;outer & inner wb/wa, non-shareable, executable, ro, domain 0, base addr 0
Sect_Normal_RO  EQU     0x0000dc16 ;as Sect_Normal_Cod, but not executable
Sect_Normal_RW  EQU     0x00005c16 ;as Sect_Normal_Cod, but writeable and not executable
Sect_SO         EQU     0x00000c12 ;strongly-ordered (therefore shareable), not executable, rw, domain 0, base addr 0
Sect_Device_RO  EQU     0x00008c12 ;device, non-shareable, non-executable, ro, domain 0, base addr 0
Sect_Device_RW  EQU     0x00000c12 ;as Sect_Device_RO, but writeable
Sect_Fault      EQU     0x00000000 ;this translation will fault (the bottom 2 bits are important, the rest are ignored)

RAM_BASE        EQU     0x80000000
VRAM_BASE       EQU     0x18000000
SRAM_BASE       EQU     0x2e000000
ETHERNET        EQU     0x1a000000
CS3_PERIPHERAL_BASE EQU 0x1c000000

; <h> Stack Configuration
;   <o> Stack Size (in Bytes, per mode) <0x0-0xFFFFFFFF:8>
; </h>

UND_Stack_Size  EQU     0x00000100
SVC_Stack_Size  EQU     0x00000100
ABT_Stack_Size  EQU     0x00000100
FIQ_Stack_Size  EQU     0x00000000
IRQ_Stack_Size  EQU     0x00000100
USR_Stack_Size  EQU     0x00000100

ISR_Stack_Size  EQU     (UND_Stack_Size + SVC_Stack_Size + ABT_Stack_Size + \
                         FIQ_Stack_Size + IRQ_Stack_Size)

        EXPORT Stack_Mem
        EXPORT __initial_sp
        EXPORT  Stack_Top

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   USR_Stack_Size
__initial_sp    SPACE   ISR_Stack_Size
Stack_Top


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

;@@ Heap configuration: Moved to ex1.icf, block HEAP

Heap_Size       EQU     0x0001000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit

                PRESERVE8
                ARM

    AREA |.text|,CODE
                export __arm_program_start
                EXPORT  user_init
                EXPORT  Undef_Handler
                EXPORT  PAbt_Handler
                EXPORT  IRQ_Handler
                EXPORT  FIQ_Handler

__arm_program_start

user_init
                IMPORT  SystemInit
                IMPORT  __main
                IMPORT  submain

                ; Put any cores other than 0 to sleep
                MRC     p15, 0, R0, c0, c0, 5     ; Read MPIDR
                ANDS    R0, R0, #3

goToSleep
                WFINE
                BNE     goToSleep

                MRC     p15, 0, R0, c1, c0, 0       ; Read CP15 System Control register
                BIC     R0, R0, #(0x1 << 12)        ; Clear I bit 12 to disable I Cache
                BIC     R0, R0, #(0x1 <<  2)        ; Clear C bit  2 to disable D Cache
                BIC     R0, R0, #0x1                ; Clear M bit  0 to disable MMU
                BIC     R0, R0, #(0x1 << 11)        ; Clear Z bit 11 to disable branch prediction
                BIC     R0, R0, #(0x1 << 13)        ; Clear V bit 13 to disable hivecs
                MCR     p15, 0, R0, c1, c0, 0       ; Write value back to CP15 System Control register
                ISB
                

;  Setup Stack for each exceptional mode
                LDR     R0, =Stack_Top

;  Enter Undefined Instruction Mode and set its Stack Pointer
                MSR     CPSR_C, #Mode_UND :OR: I_Bit :OR: F_Bit
                MOV     SP, R0
                SUB     R0, R0, #UND_Stack_Size

;  Enter Abort Mode and set its Stack Pointer
                MSR     CPSR_C, #Mode_ABT :OR: I_Bit :OR: F_Bit
                MOV     SP, R0
                SUB     R0, R0, #ABT_Stack_Size

;  Enter FIQ Mode and set its Stack Pointer
                MSR     CPSR_C, #Mode_FIQ :OR: I_Bit :OR: F_Bit
                MOV     SP, R0
                SUB     R0, R0, #FIQ_Stack_Size

;  Enter IRQ Mode and set its Stack Pointer
                MSR     CPSR_C, #Mode_IRQ :OR: I_Bit :OR: F_Bit
                MOV     SP, R0
                SUB     R0, R0, #IRQ_Stack_Size

;  Enter Supervisor Mode and set its Stack Pointer
                MSR     CPSR_C, #Mode_SVC :OR: I_Bit :OR: F_Bit
                MOV     SP, R0

;  Enter System Mode to complete initialization and enter kernel
                MSR     CPSR_C, #Mode_SYS :OR: I_Bit :OR: F_Bit
                MOV     SP, R0


 IF :DEF:RAM_Boot
                BL      call_SystemInit
 ELSE
data_init

; Initialize variables has initialized value of loader_init2.
; Variables has no initialized value already be initialized to zero 
; in boot sequence(Clear ATCM and BTCM).

                IMPORT  ||Image$$DATA$$Base||
                IMPORT  ||Image$$DATA$$Length||
                IMPORT  ||Load$$DATA$$Base||

                LDR     R0, =||Load$$DATA$$Base||
                LDR     R1, =||Image$$DATA$$Base||
                LDR     R2, =||Image$$DATA$$Length||
                CMP     R2, #0
                BEQ     call_SystemInit

copy_to_LDR_DATA
                ldrb  r3, [r0], #1
                strb  r3, [r1], #1
                subs  r2, r2, #1
                bne   copy_to_LDR_DATA    
                dsb                     ; Ensuring data-changing

 ENDIF

call_SystemInit
;  USR/SYS stack pointer will be set during kernel init

                LDR     R0, =SystemInit
                BLX     R0

;  Enter System Mode to complete initialization and enter kernel
                MSR     CPSR_C, #Mode_SYS :OR: I_Bit :OR: F_Bit

                LDR     R0, =__main
                BLX     R0

Undef_Handler\

                EXPORT  Undef_Handler
                IMPORT  CUndefHandler
                SRSFD   SP!, #Mode_UND
                PUSH    {R0-R4, R12}              ; Save APCS corruptible registers to UND mode stack

                MRS     R0, SPSR
                TST     R0, #T_Bit                ; Check mode
                MOVEQ   R1, #4                    ; R1 = 4 ARM mode
                MOVNE   R1, #2                    ; R1 = 2 Thumb mode
                SUB     R0, LR, R1
                LDREQ   R0, [R0]                  ; ARM mode - R0 points to offending instruction
                BEQ     undef_cont

                ;Thumb instruction
                ;Determine if it is a 32-bit Thumb instruction
                LDRH    R0, [R0]
                MOV     R2, #0x1c
                CMP     R2, R0, LSR #11
                BHS     undef_cont                ;16-bit Thumb instruction

                ;32-bit Thumb instruction. Unaligned - we need to reconstruct the offending instruction.
                LDRH    R2, [LR]
                ORR     R0, R2, R0, LSL #16
undef_cont
                MOV     R2, LR                    ; Set LR to third argument
                
                AND     R12, SP, #4               ; Ensure stack is 8-byte aligned
                SUB     SP, SP, R12               ; Adjust stack
                PUSH    {R12, LR}                 ; Store stack adjustment and dummy LR

                ;R0 Offending instruction
                ;R1 =2 (Thumb) or =4 (ARM)
                BL      CUndefHandler

                POP     {R12, LR}                 ; Get stack adjustment & discard dummy LR
                ADD     SP, SP, R12               ; Unadjust stack

                LDR     LR, [SP, #24]             ; Restore stacked LR and possibly adjust for retry
                SUB     LR, LR, R0
                LDR     R0, [SP, #28]             ; Restore stacked SPSR
                MSR     SPSR_CXSF, R0
                POP     {R0-R4, R12}              ; Restore stacked APCS registers
                ADD     SP, SP, #8                ; Adjust SP for already-restored banked registers
                MOVS    PC, LR

PAbt_Handler\

                EXPORT  PAbt_Handler
                IMPORT  CPAbtHandler
                SUB     LR, LR, #4                ; Pre-adjust LR
                SRSFD   SP!, #Mode_ABT            ; Save LR and SPRS to ABT mode stack
                PUSH    {R0-R4, R12}              ; Save APCS corruptible registers to ABT mode stack
                MRC     p15, 0, R0, c5, c0, 1     ; IFSR
                MRC     p15, 0, R1, c6, c0, 2     ; IFAR

                MOV     R2, LR                    ; Set LR to third argument

                AND     R12, SP, #4               ; Ensure stack is 8-byte aligned
                SUB     SP, SP, R12               ; Adjust stack
                PUSH    {R12, LR}                 ; Store stack adjustment and dummy LR

                BL      CPAbtHandler

                POP     {R12, LR}                 ; Get stack adjustment & discard dummy LR
                ADD     SP, SP, R12               ; Unadjust stack

                POP     {R0-R4, R12}              ; Restore stack APCS registers
                RFEFD   SP!                       ; Return from exception


DAbt_Handler\

                EXPORT  DAbt_Handler
                IMPORT  CDAbtHandler
                IMPORT  CDAbtHandler
                SUB     LR, LR, #8                ; Pre-adjust LR
                SRSFD   SP!, #Mode_ABT            ; Save LR and SPRS to ABT mode stack
                PUSH    {R0-R4, R12}              ; Save APCS corruptible registers to ABT mode stack
                CLREX                             ; State of exclusive monitors unknown after taken data abort
                MRC     p15, 0, R0, c5, c0, 0     ; DFSR
                MRC     p15, 0, R1, c6, c0, 0     ; DFAR

                MOV     R2, LR                    ; Set LR to third argument

                AND     R12, SP, #4               ; Ensure stack is 8-byte aligned
                SUB     SP, SP, R12               ; Adjust stack
                PUSH    {R12, LR}                 ; Store stack adjustment and dummy LR

                BL      CDAbtHandler

                POP     {R12, LR}                 ; Get stack adjustment & discard dummy LR
                ADD     SP, SP, R12               ; Unadjust stack

                POP     {R0-R4, R12}              ; Restore stacked APCS registers
                RFEFD   SP!                       ; Return from exception

FIQ_Handler\

                EXPORT  FIQ_Handler
                ;; An FIQ might occur between the dummy read and the real read of the GIC in IRQ_Handler,
                ;; so if a real FIQ Handler is implemented, this will be needed before returning:
                ;; LDR     R1, =GICI_BASE
                ;; LDR     R0, [R1, #ICCHPIR_OFFSET]   ; Dummy Read ICCHPIR (GIC CPU Interface register) to avoid GIC 390 errata 801120
                B       .

IRQ_Handler\

                B   IRQ_Handler

; User Initial Stack & Heap

                IF      :DEF:__MICROLIB
                
                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit

                ELSE

                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + USR_Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ENDIF

;*******************************************************************************
; Macro definitions
;*******************************************************************************

SCTLR_BR        dcd  0x00020000
SCTLR_M         dcd  0x00000001
SCTLR_I_C       dcd  0x00001004

; I-RAM(512KB)
DRBAR_REGION_0  dcd  0x04000000  ; Base address = 0400_0000h
DRACR_REGION_0  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share
DRSR_REGION_0   dcd  0x00000025  ; Size 512KB, MPU enable

; SPI Multi I/O(64MB)
DRBAR_REGION_1  dcd  0x10000000  ; Base address = 1000_0000h 
DRACR_REGION_1  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share
DRSR_REGION_1   dcd  0x00000033  ; Size 64MB, MPU enable 

; D-RAM, D-RAM&I-RAM Mirror(128MB)
DRBAR_REGION_2  dcd  0x20000000  ; Base address = 2000_0000h 
DRACR_REGION_2  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share
DRSR_REGION_2   dcd  0x00000035  ; Size 128MB, MPU enable

; SPI Multi I/O(64MB) Mirror
DRBAR_REGION_3  dcd  0x30000000  ; Base address = 3000_0000h 
DRACR_REGION_3  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share
DRSR_REGION_3   dcd  0x00000033  ; Size 64MB, MPU enable 

; CS0-CS1:NOR(128MB) Mirror
DRBAR_REGION_4  dcd  0x40000000  ; Base address = 4000_0000h 
DRACR_REGION_4  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share 
DRSR_REGION_4   dcd  0x00000035  ; Size 128MB, MPU enable  

; CS2-CS3:SDRAM(128MB) Mirror
DRBAR_REGION_5  dcd  0x48000000  ; Base address = 4800_0000h 
DRACR_REGION_5  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share 
DRSR_REGION_5   dcd  0x00000035  ; Size 128MB, MPU enable  

; CS4-CS5:No use(128MB) Mirror
DRBAR_REGION_6  dcd  0x50000000  ; Base address = 5000_0000h 
DRACR_REGION_6  dcd  0x00001305  ; R/W(full), XN, Device, share 
DRSR_REGION_6   dcd  0x00000035  ; Size 128MB, MPU enable  

; CS0-CS1:NOR(128MB)
DRBAR_REGION_7  dcd  0x60000000  ; Base address = 6000_0000h 
DRACR_REGION_7  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share 
DRSR_REGION_7   dcd  0x00000035  ; Size 128MB, MPU enable  

; CS2-CS3:SDRAM(128MB)
DRBAR_REGION_8  dcd  0x68000000  ; Base address = 6800_0000h
DRACR_REGION_8  dcd  0x0000030C  ; R/W(full), Normal, Non-cache, share 
DRSR_REGION_8   dcd  0x00000035  ; Size 128MB, MPU enable  

; CS4-CS5:No use(128MB)
DRBAR_REGION_9  dcd  0x70000000  ; Base address = 7000_0000h 
DRACR_REGION_9  dcd  0x00001305  ; R/W(full), XN, Device, share 
DRSR_REGION_9   dcd  0x00000035  ; Size 128MB, MPU enable  

; I/O Registers(2GB)
DRBAR_REGION_10  dcd  0x80000000 ; Base address = 8000_0000h 
DRACR_REGION_10  dcd  0x00001305 ; R/W(full), XN, Device, share 
DRSR_REGION_10   dcd  0x0000003D ; Size 2GB, MPU enable  

__enable_mpu 
                EXPORT  __enable_mpu
                push  {lr}

                ; Adopt default memory map as background map.
                ldr  r0, SCTLR_BR           ; Set SCTLR.BR bit to 1
                mrc  p15, 0, r1, c1, c0, 0  
                orr  r1, r1, r0
                dsb
                mcr  p15, 0, r1, c1, c0, 0  
                isb                         ; Ensuring Context-changing

                ; Initialize MPU settings (region 0 to 11)
                ; Define region 0
                mov  r0,  #0
                ldr  r1, DRBAR_REGION_0
                ldr  r2, DRACR_REGION_0
                ldr  r3, DRSR_REGION_0
                bl  mpu_init

                ; Define region 1
                mov  r0,  #1
                ldr  r1, DRBAR_REGION_1
                ldr  r2, DRACR_REGION_1
                ldr  r3, DRSR_REGION_1
                bl  mpu_init

                ; Define region 2
                mov  r0,  #2
                ldr  r1, DRBAR_REGION_2
                ldr  r2, DRACR_REGION_2
                ldr  r3, DRSR_REGION_2
                bl  mpu_init

                ; Define region 3
                mov  r0,  #3
                ldr  r1, DRBAR_REGION_3
                ldr  r2, DRACR_REGION_3
                ldr  r3, DRSR_REGION_3
                bl  mpu_init

                ; Define region 4
                mov  r0,  #4
                ldr  r1, DRBAR_REGION_4
                ldr  r2, DRACR_REGION_4
                ldr  r3, DRSR_REGION_4
                bl  mpu_init

                ; Define region 5
                mov  r0,  #5
                ldr  r1, DRBAR_REGION_5
                ldr  r2, DRACR_REGION_5
                ldr  r3, DRSR_REGION_5
                bl  mpu_init

                ; Define region 6
                mov  r0,  #6
                ldr  r1, DRBAR_REGION_6
                ldr  r2, DRACR_REGION_6
                ldr  r3, DRSR_REGION_6
                bl  mpu_init

                ; Define region 7
                mov  r0,  #7
                ldr  r1, DRBAR_REGION_7
                ldr  r2, DRACR_REGION_7
                ldr  r3, DRSR_REGION_7
                bl  mpu_init

                ; Define region 8
                mov  r0,  #8
                ldr  r1, DRBAR_REGION_8
                ldr  r2, DRACR_REGION_8
                ldr  r3, DRSR_REGION_8
                bl  mpu_init

                ; Define region 9
                mov  r0,  #9
                ldr  r1, DRBAR_REGION_9
                ldr  r2, DRACR_REGION_9
                ldr  r3, DRSR_REGION_9
                bl  mpu_init

                ; Define region 10
                mov  r0,  #10
                ldr  r1, DRBAR_REGION_10
                ldr  r2, DRACR_REGION_10
                ldr  r3, DRSR_REGION_10
                bl  mpu_init

                ; Enables MPU operation
                ldr  r0, SCTLR_M            ; Set SCTLR.M bit to 1
                mrc  p15, 0, r1, c1, c0, 0  
                orr  r1, r1, r0
                dsb
                mcr  p15, 0, r1, c1, c0, 0  
                isb                         ; Ensuring Context-changing

                ; Enables I1,D1 cache operation
                ldr  r0, SCTLR_I_C          ; Set SCTLR.I and C bit to 1
                mrc  p15, 0, r1, c1, c0, 0  
                orr  r1, r1, r0
                dsb
                mcr  p15, 0, r1, c1, c0, 0  
                isb                         ; Ensuring Context-changing

                pop  {pc}
                bx  lr

;***********************************************************************
; Function Name : mpu_init
; Description   : Initialize MPU settings
; Arguments     : none
; Return Value  : none
;***********************************************************************
mpu_init
                ; RGNR(MPU Memory Region Number Register)
                mcr p15, #0, r0, c6, c2, #0
                isb                             ; Ensuring Context-changing

                ; DRBAR(Data Region Base Address Register)
                mcr  p15, #0, r1, c6, c1, #0
                isb                             ; Ensuring Context-changing

                ; DRACR(Data Region Access Control Register)
                mcr p15, #0, r2, c6, c1, #4
                isb                             ; Ensuring Context-changing

                ; DRSR(Data Region Size and Enable Register)
                mcr p15, #0, r3, c6, c1, #2
                isb                             ; Ensuring Context-changing
                
                bx      lr
                
                
                END
