/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : syscalls.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : _read, _write, __exit function
* Tool-Chain   : e2studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : _read, _write, __exit function
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

#include "r_typedefs.h"

/* prototypes.  */
void    __exit(int);
int     _write(int, char *, int);
int     _read(int, char *, int);

int32_t SioWrite(int32_t file_no, const int_t * buffer, uint32_t writing_b);
int32_t SioRead(int32_t file_no, int_t * buffer, uint32_t reading_b);

/********************************************************/
int _read(int file, char * ptr, int len)
{
    int32_t x;
    int32_t idx;

    if (len < 1)
    {
        x = 0;
    }
    else if (len < 3)
    {
        int_t code;
        x = SioRead(file, &code, 1);
        *ptr = code;
    }
    else
    {
        x = SioRead(file, (int_t*)ptr, (len/sizeof(int_t))-1);
        if (x >= 0)
        {
            for (idx  = 0; idx < x; ++idx)
            {
                ptr[idx] = ((int_t*)ptr)[idx];
            }
            ptr[idx] = 0;
        }
    }

    if (x < 0)
    {
        return -1;
    }

    return x;
}

/********************************************************/
int _write(int file, char * ptr, int len)
{
    int nChars;

    for (nChars = len; nChars > 0; nChars--)
    {
        int_t   code = *ptr++;
        if (SioWrite(file, &code, 1) < 0)
        {
            break;
        }
    }


  if ((-1 == nChars) || (nChars == len))
  {
      return -1;
  }
  
  return len - nChars;
}

/********************************************************/
void __exit(int status)
{
    (void) status;

    while (1)
    {
    }
}

/* End of File */
