/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RZ/T1 CMSIS-RTOS RTX BSP sample program
* File Name    : RZ_T1_RSK_Init.c
* Version      : 1.0
* Device       : R7S910017
* Abstract     : RZ_T1_RSK Board Initialize
* Tool-Chain   : IAR Embedded Workbench for ARM / e2studio / ARM DS-5 Development Studio
* OS           : CMSIS-RTOS RTX
* H/W Platform : RZ/T1 Evaluation Board (RTK7910022C00000BR)
* Description  : RZ_T1_RSK Board Initialize functions
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*                   : 02.12.2016     1.0          First Release
*******************************************************************************/

#if (defined (__ICCARM__)) /*---------------- ICC Compiler ---------------------*/

#pragma section="VECTOR_RBLOCK"
#pragma section="VECTOR_WBLOCK"
#pragma section="USER_PRG_RBLOCK"
#pragma section="USER_PRG_WBLOCK"
#pragma section="USER_DATA_RBLOCK"
#pragma section="USER_DATA_WBLOCK"

#endif  // __ICCARM__

#if (defined (__CC_ARM)) /*---------------- ARMCC Compiler ---------------------*/
#pragma arm section code = "LOADER_CODE"
#pragma arm section rodata = "LOADER_CONST"
#pragma arm section rwdata = "LOADER_DATA"
#pragma arm section zidata = "LOADER_BSS"
#endif

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "cmsis_os.h"
#include "Renesas_RZ_T1.h"
#include "RZ_T1_RSK_Init.h"
#include "r_cpg.h"
#include "r_reset.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/
//#define CS2_SDRAM_MODE_16BIT_CAS2_BR_BW (*(volatile uint16_t*)0x3FFFD040)
//#define CS3_SDRAM_MODE_16BIT_CAS2_BR_BW (*(volatile uint16_t*)0x3FFFE040)

#define GPIO_PORT0_BOOTMODE_BITMASK (0x000fu)

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/
#if defined (__CC_ARM)
extern uint32_t Image$$INIT$$Base;
extern uint32_t Load$$LR$$LOAD_MODULE4$$Base;
extern uint32_t Load$$LR$$LOAD_MODULE4$$Length;
#endif

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
#if defined (__ICCARM__)
void reset_check(void);
void copy_4byte(uint32_t *src, uint32_t *dst, uint32_t bytesize);
void copy_to_atcm(void);
#elif (defined (__GNUC__))
void reset_check(void) __attribute__ ((section (".loader_text2")));
void cpg_init(void) __attribute__ ((section (".loader_text2")));
#else
void reset_check(void);
void cpg_init(void);
void copy_to_atcm(void);
#endif

/**************************************************************************//**
* Function Name: RZ_T1_RSK_InitClock
* @brief         Initialize Board settings
*
*                Description:<br>
*                Initialize Clock
* @param         none
* @retval        none
******************************************************************************/
void RZ_T1_RSK_InitClock(void)
{
    volatile uint32_t dummy;

    /* Enables writing to the registers related to CPG function */
    R_CPG_WriteEnable();

    /* Enables LOCO clock operation */
    SYSTEM.LOCOCR.BIT.LCSTP = CPG_LOCO_ENABLE;

    /* Set CPUCLK to 450MHz, and dummy read at three times */
    SYSTEM.PLL1CR.LONG = CPG_CPUCLK_450_MHz;
    dummy = SYSTEM.PLL1CR.LONG;
    dummy = SYSTEM.PLL1CR.LONG;
    dummy = SYSTEM.PLL1CR.LONG;

    /* Enables PLL1 operation */
    SYSTEM.PLL1CR2.LONG = CPG_PLL1_ON;

    /* Disables writing to the registers related to CPG function */
    R_CPG_WriteDisable();

    /* Wait about 100us for PLL1 (and LOCO) stabilization */
    R_CPG_PLL_Wait();

    /* Enables writing to the registers related to CPG function */
    R_CPG_WriteEnable();

    /* Selects the PLL1 as clock source */
    SYSTEM.SCKCR2.LONG = CPG_SELECT_PLL1;

    /* Set BSC CKIO clock to 75MHz */
    SYSTEM.SCKCR.BIT.CKIO = CPG_CKIO_75_MHz;

    /* Disables writing to the registers related to CPG function */
    R_CPG_WriteDisable();

    return;
}

/*******************************************************************************
* Function Name : reset_check
* Description   : Check the reset source and execute the each sequence.
*
* Arguments    : none
* Return Value : none
*******************************************************************************/
void reset_check(void)
{
    volatile uint8_t result;
    volatile uint32_t dummy;

    /* Check the reset status flag and execute the each sequence */
    if (RST_SOURCE_ECM == SYSTEM.RSTSR0.LONG) // ECM reset is generated
    {
        /* Clear reset status flag */
        R_RST_WriteEnable();              // Enable writing to the RSTSR0 register
        SYSTEM.RSTSR0.LONG = 0x00000000;  // Clear reset factor flag
        R_RST_WriteDisable();             // Disable writing to the RSTSR0 register

        /* Please coding the User program */

    }
    else if (RST_SOURCE_SWR1 == SYSTEM.RSTSR0.LONG) // Software reset 1 is generated
    {
        /* Clear reset status flag */
        R_RST_WriteEnable();              // Enable writing to the RSTSR0 register
        SYSTEM.RSTSR0.LONG = 0x00000000;  // Clear reset factor flag
        R_RST_WriteDisable();             // Disable writing to the RSTSR0 register

        /* Please coding the User program */

    }
    else if (RST_SOURCE_RES == SYSTEM.RSTSR0.LONG) // RES# pin reset is generated
    {
        /* Clear reset status flag */
        R_RST_WriteEnable();              // Enable writing to the RSTSR0 register
        SYSTEM.RSTSR0.LONG = 0x00000000;  // Clear reset factor flag
        R_RST_WriteDisable();             // Disable writing to the RSTSR0 register

        /* Please coding the User program */

    }
    else // Any reset is not generated
    {
        /* Please coding the User program */
    }

}

/*******************************************************************************
* Function Name : cpg_init
* Description   : Set CPU clock and LOCO clock by CPG function
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cpg_init(void)
{
    volatile uint32_t dummy;

    /* Enables writing to the registers related to CPG function */
    R_CPG_WriteEnable();

    /* Enables LOCO clock operation */
    SYSTEM.LOCOCR.BIT.LCSTP = CPG_LOCO_ENABLE;

    /* Set CPUCLK to 450MHz, and dummy read at three times */
    SYSTEM.PLL1CR.LONG = CPG_CPUCLK_450_MHz;
    dummy = SYSTEM.PLL1CR.LONG;
    dummy = SYSTEM.PLL1CR.LONG;
    dummy = SYSTEM.PLL1CR.LONG;

    /* Enables PLL1 operation */
    SYSTEM.PLL1CR2.LONG = CPG_PLL1_ON;

    /* Disables writing to the registers related to CPG function */
    R_CPG_WriteDisable();

    /* Wait about 100us for PLL1 (and LOCO) stabilization */
    R_CPG_PLL_Wait();

    /* Enables writing to the registers related to CPG function */
    R_CPG_WriteEnable();

    /* Selects the PLL1 as clock source */
    SYSTEM.SCKCR2.LONG = CPG_SELECT_PLL1;

    /* Set BSC CKIO clock to 75MHz */
    SYSTEM.SCKCR.BIT.CKIO = CPG_CKIO_75_MHz;

    /* Disables writing to the registers related to CPG function */
    R_CPG_WriteDisable();

}
#if (defined (__ICCARM__)) /*---------------- ICC Compiler ---------------------*/
/*******************************************************************************
* Function Name : copy_to_atcm
* Description   : Copy the application program from external Flash to ATCM
* Arguments    : none
* Return Value : none
*******************************************************************************/
void copy_to_atcm(void)
{
    /* Define destination/source address pointer and block size */
    uint32_t * src;
    uint32_t * dst;
    uint32_t size;

    /* Copy exception vector block */
    /* Casting the pointer and address to a (uint32_t *) and (uint32_t) are valid
       because these will reference 32 bit address and address size */
    src = (uint32_t *) __section_begin("VECTOR_RBLOCK");
    dst  = (uint32_t *) __section_begin("VECTOR_WBLOCK");
    size = (uint32_t) __section_size("VECTOR_WBLOCK");
    copy_4byte(src, dst, size);

    /* Copy user program block */
    /* Casting the pointer and address to a (uint32_t *) and (uint32_t) are valid
       because these will reference 32 bit address and address size */
    src = (uint32_t *) __section_begin("USER_PRG_RBLOCK");
    dst  = (uint32_t *) __section_begin("USER_PRG_WBLOCK");
    size = (uint32_t) __section_size("USER_PRG_WBLOCK");
    copy_4byte(src, dst, size);

    /* Copy user data block */
    /* Casting the pointer and address to a (uint32_t *) and (uint32_t) are valid
       because these will reference 32 bit address and address size */
    src = (uint32_t *) __section_begin("USER_DATA_RBLOCK");
    dst  = (uint32_t *) __section_begin("USER_DATA_WBLOCK");
    size = (uint32_t) __section_size("USER_DATA_WBLOCK");
    copy_4byte(src, dst, size);

}

/*******************************************************************************
 End of function copy_to_atcm
*******************************************************************************/

/*******************************************************************************
* Function Name : copy_4byte
* Description   : Copy the memory block from Source address to Destination
*                 address by the 4byte unit.
* Arguments    : src
                     Source starting address
                 dst
                     Destination starting address
                 bytesize
                     Copy memory size
* Return Value : none
*******************************************************************************/
void copy_4byte(uint32_t *src, uint32_t *dst, uint32_t bytesize)
{
    uint32_t i;
    uint32_t cnt;

    /* Copy Count in 4Byte unit */
    cnt = (bytesize + 3) >> 2;

    for (i=0 ; i<cnt ; i++)
    {
        *dst++ = *src++;
    }

    asm("dsb");                  // Ensuring data-changing
}

/*******************************************************************************
 End of function copy_4byte
*******************************************************************************/
#endif /* __ICCARM__ */

#if defined (__CC_ARM)

/*******************************************************************************
* Function Name : copy_to_atcm
* Description   : Copy the application program from external Flash to ATCM
* Arguments    : none
* Return Value : none
*******************************************************************************/
void copy_to_atcm(void)
{
    uint32_t    loop;
    uint32_t    app_size;
    uint32_t    trans_size;
    uint32_t    *src;
    uint32_t    *dst;

    app_size = (uint32_t)&Load$$LR$$LOAD_MODULE4$$Length;

    trans_size = ((app_size + 3) / sizeof(uint32_t));

    src = (uint32_t *)&Load$$LR$$LOAD_MODULE4$$Base;
    dst = (uint32_t *)&Image$$INIT$$Base;

    for ( loop = 0; loop < trans_size; loop++ )
    {
        *dst++ = *src++;
    }
}
#endif

/******************************************************************************
End of file
******************************************************************************/
